(herald "DHCR: unified model (UM) original" (bound 20) (limit 12000)
  (algebra diffie-hellman))

(comment "CPSA 4.4.0")
(comment "All input read from dhcr_um_expt_assume.scm")
(comment "Step count limited to 12000")
(comment "Strand count bounded at 20")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b) (undisclosed la) (undisclosed beta))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 0)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed la)
    (undisclosed beta))
  (rule assume-init-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 1)
  (parent 0)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (beta expt) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l)
    (undisclosed beta))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 2)
  (parent 1)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 3)
  (parent 2)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 4)
  (parent 3)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 5)
  (parent 3)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 6)
  (parent 4)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 7)
  (parent 4)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 8)
  (parent 5)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 9)
  (parent 6)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 10)
  (parent 6)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 11)
  (parent 7)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 12)
  (parent 7)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 13)
  (parent 7)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 14)
  (parent 8)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 15)
  (parent 8)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l) (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 16)
  (parent 9)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (beta lb) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 17)
  (parent 9)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 18)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 19)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 20)
  (parent 10)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 21)
  (parent 10)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 22)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 23)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 24)
  (parent 10)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 25)
  (parent 11)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l)
    (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 26)
  (parent 17)
  (seen 9)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 27)
  (parent 18)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 28)
  (parent 18)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 29)
  (parent 18)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l) (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 30)
  (parent 21)
  (seen 16)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 31)
  (parent 21)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 32)
  (parent 22)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 33)
  (parent 22)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 34)
  (parent 22)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 35)
  (parent 25)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 36)
  (parent 25)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l)
    (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 37)
  (parent 31)
  (seen 26)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b) (undisclosed beta))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 38)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la rndx)
    (beta expt) (x rndx) (upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed beta))
  (rule assume-init-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 39)
  (parent 38)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (beta expt) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed beta))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 40)
  (parent 39)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 41)
  (parent 40)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 42)
  (parent 41)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 43)
  (parent 41)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 44)
  (parent 42)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 45)
  (parent 42)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 46)
  (parent 43)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 47)
  (parent 44)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 48)
  (parent 44)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 49)
  (parent 45)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 50)
  (parent 45)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 51)
  (parent 45)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 52)
  (parent 46)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 53)
  (parent 46)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 54)
  (parent 47)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (beta lb) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 55)
  (parent 47)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 56)
  (parent 48)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 57)
  (parent 48)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 58)
  (parent 48)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 59)
  (parent 48)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 60)
  (parent 48)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 61)
  (parent 48)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 62)
  (parent 48)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 63)
  (parent 49)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 64)
  (parent 51)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 65)
  (parent 53)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 66)
  (parent 55)
  (seen 47)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 67)
  (parent 56)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 68)
  (parent 56)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 69)
  (parent 56)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 70)
  (parent 58)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 71)
  (parent 59)
  (seen 54)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 72)
  (parent 59)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 73)
  (parent 60)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 74)
  (parent 60)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 75)
  (parent 60)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 76)
  (parent 63)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 77)
  (parent 63)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 78)
  (parent 64)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 79)
  (parent 64)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 80)
  (parent 65)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 81)
  (parent 65)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b)
    (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 82)
  (parent 70)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 83)
  (parent 70)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed lb))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 84)
  (parent 72)
  (seen 66)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 85)
  (parent 77)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 86)
  (parent 78)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 87)
  (parent 80)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 88)
  (parent 81)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (7 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 89)
  (parent 81)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 90)
  (parent 81)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 91)
  (parent 81)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 92)
  (parent 81)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 93)
  (parent 81)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 94)
  (parent 81)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 95)
  (parent 82)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 96)
  (parent 83)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 97)
  (parent 85)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 98)
  (parent 85)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 99)
  (parent 86)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 100)
  (parent 86)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 101)
  (parent 87)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 102)
  (parent 88)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 103)
  (parent 88)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 104)
  (parent 88)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 105)
  (parent 92)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 106)
  (parent 92)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 107)
  (parent 92)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 108)
  (parent 93)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 109)
  (parent 95)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 110)
  (parent 96)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 111)
  (parent 97)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 112)
  (parent 97)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 113)
  (parent 98)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 114)
  (parent 98)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 115)
  (parent 98)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 116)
  (parent 98)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 117)
  (parent 98)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 118)
  (parent 98)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 119)
  (parent 98)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 120)
  (parent 98)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 121)
  (parent 99)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 122)
  (parent 99)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 123)
  (parent 100)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 124)
  (parent 100)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 125)
  (parent 100)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 126)
  (parent 100)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 127)
  (parent 100)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 128)
  (parent 100)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 129)
  (parent 100)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 130)
  (parent 100)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 131)
  (parent 101)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (beta l-0) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 132)
  (parent 108)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 133)
  (parent 109)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (beta lb) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 134)
  (parent 110)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 135)
  (parent 111)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 136)
  (parent 112)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 137)
  (parent 113)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 138)
  (parent 113)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 139)
  (parent 113)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 140)
  (parent 113)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 141)
  (parent 114)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 142)
  (parent 114)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 143)
  (parent 114)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 144)
  (parent 114)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 145)
  (parent 117)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 146)
  (parent 117)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 147)
  (parent 121)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 148)
  (parent 122)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 149)
  (parent 123)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 150)
  (parent 123)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 151)
  (parent 123)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 152)
  (parent 123)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 153)
  (parent 124)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 154)
  (parent 124)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 155)
  (parent 124)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 156)
  (parent 124)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 157)
  (parent 127)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 158)
  (parent 127)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 159)
  (parent 132)
  (seen 131)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 160)
  (parent 134)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 161)
  (parent 135)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 162)
  (parent 136)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 163)
  (parent 145)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 164)
  (parent 146)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 165)
  (parent 147)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 166)
  (parent 148)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 167)
  (parent 157)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 168)
  (parent 158)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed lb))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 169)
  (parent 160)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (beta lb) (x x) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 170)
  (parent 161)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 171)
  (parent 162)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 172)
  (parent 163)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 173)
  (parent 164)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 174)
  (parent 165)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 175)
  (parent 166)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 176)
  (parent 167)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (7 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 177)
  (parent 168)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 178)
  (parent 170)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 179)
  (parent 171)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 180)
  (parent 172)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 181)
  (parent 173)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 182)
  (parent 174)
  (seen 186)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 183)
  (parent 175)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 184)
  (parent 176)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (6 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 185)
  (parent 177)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 186)
  (parent 178)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (3 0)) ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 187)
  (parent 179)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 188)
  (parent 180)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 189)
  (parent 181)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 190)
  (parent 183)
  (seen 194)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 191)
  (parent 184)
  (seen 195)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 2) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 192)
  (parent 185)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 193)
  (parent 186)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (beta l-0) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 194)
  (parent 187)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 195)
  (parent 188)
  (seen 186)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 196)
  (parent 189)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 197)
  (parent 192)
  (seen 199)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 198)
  (parent 194)
  (seen 131)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (5 0)) ((4 1) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 199)
  (parent 196)
  (seen 194)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b) (undisclosed la))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 200)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed la))
  (rule assume-init-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 201)
  (parent 200)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (beta upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 202)
  (parent 201)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 203)
  (parent 202)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 204)
  (parent 203)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 205)
  (parent 203)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 206)
  (parent 204)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 207)
  (parent 204)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 208)
  (parent 205)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 209)
  (parent 206)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 210)
  (parent 206)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 211)
  (parent 207)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 212)
  (parent 207)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 213)
  (parent 207)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 214)
  (parent 208)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 215)
  (parent 208)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 216)
  (parent 209)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (x x) (beta lb) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 217)
  (parent 209)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 218)
  (parent 210)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 219)
  (parent 210)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 220)
  (parent 210)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 221)
  (parent 210)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 222)
  (parent 210)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 223)
  (parent 210)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 224)
  (parent 210)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 225)
  (parent 211)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 226)
  (parent 212)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 227)
  (parent 214)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 228)
  (parent 217)
  (seen 209)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 229)
  (parent 218)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 230)
  (parent 218)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 231)
  (parent 218)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 232)
  (parent 221)
  (seen 216)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 233)
  (parent 221)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 234)
  (parent 222)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 235)
  (parent 222)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 236)
  (parent 222)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 237)
  (parent 225)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 238)
  (parent 225)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 239)
  (parent 226)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 240)
  (parent 226)
  (unrealized (0 3) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 241)
  (parent 227)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 242)
  (parent 227)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 243)
  (parent 233)
  (seen 228)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 244)
  (parent 237)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 245)
  (parent 239)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 246)
  (parent 241)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 247)
  (parent 242)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 248)
  (parent 242)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 249)
  (parent 242)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 250)
  (parent 242)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 251)
  (parent 242)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 252)
  (parent 242)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 253)
  (parent 242)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 254)
  (parent 244)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 255)
  (parent 244)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 256)
  (parent 245)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 257)
  (parent 245)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 258)
  (parent 246)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 259)
  (parent 247)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 260)
  (parent 247)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 261)
  (parent 247)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 262)
  (parent 250)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 263)
  (parent 251)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 264)
  (parent 251)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 265)
  (parent 251)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 266)
  (parent 254)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 267)
  (parent 254)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 268)
  (parent 255)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 269)
  (parent 255)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 270)
  (parent 255)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 271)
  (parent 255)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 272)
  (parent 255)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 273)
  (parent 255)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 274)
  (parent 255)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 275)
  (parent 255)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 276)
  (parent 256)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 277)
  (parent 256)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 278)
  (parent 257)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 279)
  (parent 257)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 280)
  (parent 257)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 281)
  (parent 257)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 282)
  (parent 257)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 283)
  (parent 257)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 284)
  (parent 257)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 285)
  (parent 257)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 286)
  (parent 258)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 287)
  (parent 262)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 288)
  (parent 266)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 289)
  (parent 267)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 290)
  (parent 268)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 291)
  (parent 268)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 292)
  (parent 268)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 293)
  (parent 268)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 294)
  (parent 269)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 295)
  (parent 269)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 296)
  (parent 269)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 297)
  (parent 269)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 298)
  (parent 273)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 299)
  (parent 273)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 300)
  (parent 276)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 301)
  (parent 277)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 302)
  (parent 278)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 303)
  (parent 278)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 304)
  (parent 278)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 305)
  (parent 278)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 306)
  (parent 279)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 307)
  (parent 279)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 308)
  (parent 279)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 309)
  (parent 279)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 310)
  (parent 283)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-1))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 311)
  (parent 283)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 312)
  (parent 287)
  (seen 286)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 313)
  (parent 288)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 314)
  (parent 289)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 315)
  (parent 298)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 316)
  (parent 299)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 317)
  (parent 300)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 318)
  (parent 301)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 319)
  (parent 310)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 320)
  (parent 311)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 321)
  (parent 313)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 322)
  (parent 314)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 323)
  (parent 315)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 324)
  (parent 316)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 0)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 325)
  (parent 317)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 326)
  (parent 318)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 327)
  (parent 319)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 328)
  (parent 320)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 329)
  (parent 321)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 330)
  (parent 322)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 331)
  (parent 323)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 332)
  (parent 324)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 333)
  (parent 325)
  (seen 337)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (6 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 334)
  (parent 326)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 335)
  (parent 327)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 336)
  (parent 328)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 337)
  (parent 329)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((4 2) (2 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 338)
  (parent 330)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 339)
  (parent 331)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 340)
  (parent 332)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 341)
  (parent 334)
  (seen 345)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 342)
  (parent 335)
  (seen 346)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (7 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 343)
  (parent 336)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 344)
  (parent 337)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 0)) ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 345)
  (parent 338)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 346)
  (parent 339)
  (seen 337)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (2 0)) ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 347)
  (parent 340)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 348)
  (parent 343)
  (seen 350)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 349)
  (parent 345)
  (seen 286)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (5 0))
    ((3 1) (5 0)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b)
    (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 350)
  (parent 347)
  (seen 345)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 351)
  (unrealized (0 1))
  (origs (na (0 2)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (la x rndx)
    (beta upsilon expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" b))
  (uniq-orig na)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (rule assume-init-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb)))
  (label 352)
  (parent 351)
  (unrealized (0 0) (0 1))
  (origs (na (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (beta upsilon expt) (l rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l)))))
  (label 353)
  (parent 352)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 354)
  (parent 353)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 355)
  (parent 354)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 356)
  (parent 354)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 357)
  (parent 355)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 358)
  (parent 355)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 359)
  (parent 356)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 360)
  (parent 357)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 361)
  (parent 357)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 362)
  (parent 358)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 363)
  (parent 358)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 364)
  (parent 358)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 365)
  (parent 359)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 366)
  (parent 359)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 367)
  (parent 360)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l) (x x) (beta lb) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor-0))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 368)
  (parent 360)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 369)
  (parent 361)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 370)
  (parent 361)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 371)
  (parent 361)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 372)
  (parent 361)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 373)
  (parent 361)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 374)
  (parent 361)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 375)
  (parent 361)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 376)
  (parent 362)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 377)
  (parent 363)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 378)
  (parent 364)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 379)
  (parent 365)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 380)
  (parent 366)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 381)
  (parent 368)
  (seen 360)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 382)
  (parent 369)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 383)
  (parent 369)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 384)
  (parent 369)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 385)
  (parent 371)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 386)
  (parent 372)
  (seen 367)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 387)
  (parent 372)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 388)
  (parent 373)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 389)
  (parent 373)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (3 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-1)))))
  (label 390)
  (parent 373)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 391)
  (parent 376)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 392)
  (parent 376)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 393)
  (parent 377)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 394)
  (parent 377)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 395)
  (parent 378)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 396)
  (parent 378)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 397)
  (parent 379)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 398)
  (parent 379)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 399)
  (parent 380)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 400)
  (parent 380)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 401)
  (parent 385)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 402)
  (parent 385)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b lb)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 403)
  (parent 387)
  (seen 381)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 404)
  (parent 391)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 405)
  (parent 392)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 406)
  (parent 393)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 407)
  (parent 394)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 408)
  (parent 395)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 409)
  (parent 396)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 410)
  (parent 397)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 411)
  (parent 398)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 412)
  (parent 398)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 413)
  (parent 398)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 414)
  (parent 398)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 415)
  (parent 398)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 416)
  (parent 398)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 417)
  (parent 398)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 418)
  (parent 399)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 419)
  (parent 400)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (7 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 420)
  (parent 400)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 421)
  (parent 400)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 422)
  (parent 400)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 423)
  (parent 400)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 424)
  (parent 400)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 425)
  (parent 400)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 426)
  (parent 401)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 427)
  (parent 402)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 428)
  (parent 404)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 429)
  (parent 404)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 430)
  (parent 405)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 431)
  (parent 405)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 432)
  (parent 406)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 433)
  (parent 406)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 434)
  (parent 407)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 435)
  (parent 408)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 436)
  (parent 408)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 437)
  (parent 409)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 438)
  (parent 410)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 439)
  (parent 411)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 440)
  (parent 411)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 441)
  (parent 411)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 442)
  (parent 413)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 443)
  (parent 414)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 444)
  (parent 415)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 445)
  (parent 415)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 446)
  (parent 415)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 447)
  (parent 418)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 448)
  (parent 419)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 449)
  (parent 419)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 450)
  (parent 419)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 451)
  (parent 421)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon alpha zeta expt) (l y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 452)
  (parent 423)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (alpha zeta expt) (y l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 453)
  (parent 423)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 454)
  (parent 423)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 455)
  (parent 424)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 456)
  (parent 426)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 457)
  (parent 427)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 458)
  (parent 428)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 459)
  (parent 428)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 460)
  (parent 429)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 461)
  (parent 429)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 462)
  (parent 429)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 463)
  (parent 429)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 464)
  (parent 429)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 465)
  (parent 429)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 466)
  (parent 429)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 467)
  (parent 429)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 468)
  (parent 430)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 469)
  (parent 430)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 470)
  (parent 431)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 471)
  (parent 431)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 472)
  (parent 431)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 473)
  (parent 431)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 474)
  (parent 431)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 475)
  (parent 431)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 476)
  (parent 431)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 477)
  (parent 431)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 478)
  (parent 432)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 479)
  (parent 432)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 480)
  (parent 433)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 481)
  (parent 433)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 482)
  (parent 433)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 483)
  (parent 433)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 484)
  (parent 433)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 485)
  (parent 433)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 486)
  (parent 433)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 487)
  (parent 433)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 488)
  (parent 434)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 489)
  (parent 434)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 490)
  (parent 434)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 491)
  (parent 434)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 492)
  (parent 434)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 493)
  (parent 434)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 494)
  (parent 434)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 495)
  (parent 434)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 496)
  (parent 435)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 497)
  (parent 435)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 498)
  (parent 436)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 499)
  (parent 436)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 500)
  (parent 436)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 501)
  (parent 436)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 502)
  (parent 436)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 503)
  (parent 436)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 504)
  (parent 436)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 505)
  (parent 436)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 506)
  (parent 437)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 507)
  (parent 437)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 508)
  (parent 437)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 509)
  (parent 437)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 510)
  (parent 437)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 511)
  (parent 437)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 512)
  (parent 437)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 513)
  (parent 437)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 514)
  (parent 438)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 515)
  (parent 442)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 516)
  (parent 443)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 517)
  (parent 447)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (x x) (beta l-0) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 518)
  (parent 451)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 519)
  (parent 455)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 520)
  (parent 456)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (x x) (beta lb) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 521)
  (parent 457)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 522)
  (parent 458)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 523)
  (parent 459)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 524)
  (parent 460)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 525)
  (parent 460)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 526)
  (parent 460)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 527)
  (parent 460)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 528)
  (parent 461)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 529)
  (parent 461)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 530)
  (parent 461)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 531)
  (parent 461)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 532)
  (parent 462)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 533)
  (parent 465)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 534)
  (parent 465)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 535)
  (parent 468)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 536)
  (parent 469)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 537)
  (parent 470)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 538)
  (parent 470)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 539)
  (parent 470)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 540)
  (parent 470)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 541)
  (parent 471)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 542)
  (parent 471)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 543)
  (parent 471)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 544)
  (parent 471)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 545)
  (parent 472)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 546)
  (parent 474)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 547)
  (parent 474)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 548)
  (parent 478)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 549)
  (parent 479)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 550)
  (parent 480)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 551)
  (parent 480)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 552)
  (parent 480)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 553)
  (parent 480)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 554)
  (parent 481)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 555)
  (parent 481)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 556)
  (parent 481)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 557)
  (parent 481)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 558)
  (parent 482)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 559)
  (parent 485)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 560)
  (parent 485)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 561)
  (parent 488)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 562)
  (parent 488)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 563)
  (parent 488)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 564)
  (parent 488)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 565)
  (parent 489)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0))
    ((2 1) (6 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 566)
  (parent 489)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 567)
  (parent 489)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 568)
  (parent 489)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 569)
  (parent 490)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1)))
  (label 570)
  (parent 494)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 571)
  (parent 496)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 572)
  (parent 497)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 573)
  (parent 498)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 574)
  (parent 498)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 575)
  (parent 498)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 576)
  (parent 498)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 577)
  (parent 499)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 578)
  (parent 499)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 579)
  (parent 499)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 580)
  (parent 499)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 581)
  (parent 500)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 582)
  (parent 502)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 583)
  (parent 502)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 584)
  (parent 506)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 585)
  (parent 506)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 586)
  (parent 506)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 587)
  (parent 506)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 588)
  (parent 507)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0))
    ((2 1) (8 0)) ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 589)
  (parent 507)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 590)
  (parent 507)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 591)
  (parent 507)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 592)
  (parent 508)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1)))
  (label 593)
  (parent 512)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 594)
  (parent 515)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 595)
  (parent 516)
  (seen 514)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 596)
  (parent 518)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 597)
  (parent 519)
  (seen 517)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 598)
  (parent 521)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 599)
  (parent 522)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 600)
  (parent 523)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 601)
  (parent 532)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 602)
  (parent 532)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 603)
  (parent 533)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 604)
  (parent 534)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 605)
  (parent 535)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 606)
  (parent 536)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 607)
  (parent 545)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 608)
  (parent 545)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 609)
  (parent 546)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 610)
  (parent 547)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 611)
  (parent 548)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 612)
  (parent 549)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 613)
  (parent 558)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 614)
  (parent 558)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 615)
  (parent 559)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 616)
  (parent 560)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 617)
  (parent 569)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 618)
  (parent 570)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 619)
  (parent 571)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 620)
  (parent 572)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 621)
  (parent 581)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-2 (pv b l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 622)
  (parent 581)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (8 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 623)
  (parent 582)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (9 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 624)
  (parent 583)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 625)
  (parent 592)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 626)
  (parent 593)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 627)
  (parent 594)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 628)
  (parent 596)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 629)
  (parent 598)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (x x) (beta lb) (upsilon (mul w y)) (na na)
        (nb nb) (priv-stor priv-stor))))
  (origs (pt-3 (3 1)) (lb (2 1)) (pt-2 (2 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 630)
  (parent 599)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 631)
  (parent 600)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 632)
  (parent 601)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 633)
  (parent 602)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 634)
  (parent 603)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 635)
  (parent 604)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 636)
  (parent 605)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 637)
  (parent 606)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (9 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (9 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 638)
  (parent 607)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 639)
  (parent 608)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 640)
  (parent 609)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 641)
  (parent 610)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 0)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 642)
  (parent 611)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 643)
  (parent 612)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 644)
  (parent 613)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (10 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 645)
  (parent 614)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 646)
  (parent 615)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 647)
  (parent 616)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 648)
  (parent 617)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 649)
  (parent 617)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 650)
  (parent 618)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 651)
  (parent 618)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 652)
  (parent 619)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 653)
  (parent 620)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (9 0)) ((2 2) (0 1))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (9 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 654)
  (parent 621)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0))
    ((2 2) (0 1)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 655)
  (parent 622)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 656)
  (parent 623)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (8 0)) ((3 1) (7 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 657)
  (parent 624)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 658)
  (parent 625)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 659)
  (parent 625)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 660)
  (parent 626)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 661)
  (parent 626)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 662)
  (parent 627)
  (seen 629)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 663)
  (parent 628)
  (seen 629)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 664)
  (parent 630)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 665)
  (parent 631)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 666)
  (parent 632)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 667)
  (parent 633)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 668)
  (parent 634)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 669)
  (parent 635)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 670)
  (parent 636)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 671)
  (parent 637)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 672)
  (parent 638)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (9 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 673)
  (parent 639)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 674)
  (parent 640)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 675)
  (parent 641)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 676)
  (parent 642)
  (seen 696)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (6 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 677)
  (parent 643)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 678)
  (parent 644)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 679)
  (parent 645)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 680)
  (parent 646)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 681)
  (parent 647)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 682)
  (parent 648)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 683)
  (parent 649)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 684)
  (parent 650)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 685)
  (parent 651)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 686)
  (parent 652)
  (seen 702)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 687)
  (parent 653)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (8 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (8 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 688)
  (parent 654)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (9 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (9 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 689)
  (parent 655)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 690)
  (parent 656)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (6 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 691)
  (parent 657)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 692)
  (parent 658)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 693)
  (parent 659)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 694)
  (parent 660)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 695)
  (parent 661)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 696)
  (parent 664)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((4 2) (2 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 697)
  (parent 665)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 698)
  (parent 666)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 699)
  (parent 667)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 700)
  (parent 668)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 701)
  (parent 669)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 702)
  (parent 670)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (3 0)) ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 703)
  (parent 671)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 704)
  (parent 672)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 705)
  (parent 673)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 706)
  (parent 674)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (7 0)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 707)
  (parent 675)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 708)
  (parent 677)
  (seen 727)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 709)
  (parent 678)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 710)
  (parent 679)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 711)
  (parent 680)
  (seen 730)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (7 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 712)
  (parent 681)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 713)
  (parent 682)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 714)
  (parent 683)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 715)
  (parent 684)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 716)
  (parent 685)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 717)
  (parent 687)
  (seen 733)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 718)
  (parent 688)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (8 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 719)
  (parent 689)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 720)
  (parent 690)
  (seen 736)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((4 2) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 721)
  (parent 691)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 722)
  (parent 692)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 723)
  (parent 693)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 724)
  (parent 694)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 725)
  (parent 695)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 726)
  (parent 696)
  (realized)
  (shape)
  (maps
    ((0)
      ((a self) (b b) (la l-0) (x x) (beta l) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 0)) ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 727)
  (parent 697)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 728)
  (parent 698)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 729)
  (parent 699)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 730)
  (parent 700)
  (seen 696)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (2 0)) ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 731)
  (parent 701)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 732)
  (parent 702)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (la l) (x x) (beta l-0) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 733)
  (parent 703)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 734)
  (parent 704)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 735)
  (parent 705)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 736)
  (parent 706)
  (seen 702)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 737)
  (parent 707)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 738)
  (parent 709)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 739)
  (parent 710)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 740)
  (parent 712)
  (seen 755)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((3 1) (8 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 741)
  (parent 713)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 742)
  (parent 714)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 743)
  (parent 715)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 744)
  (parent 716)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 745)
  (parent 718)
  (seen 757)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (5 0)) ((4 2) (8 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 746)
  (parent 719)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 747)
  (parent 721)
  (seen 759)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (8 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 748)
  (parent 722)
  (seen 762)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 749)
  (parent 723)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 750)
  (parent 724)
  (seen 764)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 751)
  (parent 725)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 752)
  (parent 727)
  (seen 514)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 753)
  (parent 728)
  (seen 426)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 754)
  (parent 729)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (5 0))
    ((3 1) (5 0)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 755)
  (parent 731)
  (seen 727)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 756)
  (parent 733)
  (seen 517)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 757)
  (parent 734)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 758)
  (parent 735)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 2) (5 0)) ((4 1) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 759)
  (parent 737)
  (seen 733)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 760)
  (parent 738)
  (seen 426)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 761)
  (parent 739)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (7 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 762)
  (parent 741)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (8 0)) ((4 1) (9 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 763)
  (parent 742)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 764)
  (parent 743)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 765)
  (parent 744)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (7 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 766)
  (parent 746)
  (seen 771)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (8 0)) ((4 2) (9 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (5 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 767)
  (parent 749)
  (seen 774)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 2) (9 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 768)
  (parent 751)
  (seen 776)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 2) (0 1)) ((3 2) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 769)
  (parent 754)
  (seen 457)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 770)
  (parent 757)
  (seen 456)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 2) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 771)
  (parent 758)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 772)
  (parent 761)
  (seen 457)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (6 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 773)
  (parent 762)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (7 0)) ((4 2) (8 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 774)
  (parent 763)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 3))
    ((4 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 775)
  (parent 764)
  (seen 773)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 776)
  (parent 765)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((2 2) (0 1)) ((3 2) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 777)
  (parent 771)
  (seen 521)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 778)
  (parent 773)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 779)
  (parent 774)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 2) (7 0)) ((4 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (non-orig (privk "sig" self))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 780)
  (parent 776)
  (seen 779)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 781)
  (parent 778)
  (seen 456)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 782)
  (parent 779)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (non-orig (privk "sig" b))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 783)
  (parent 782)
  (seen 521)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb na-0 nb-0 data) (a b name) (pt pt-0 pval)
    (priv-stor priv-stor-0 locn) (la lb rndx) (alpha beta expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x la) (x beta))
  (facts (neq a b) (undisclosed la) (undisclosed lb))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb-0)))
  (label 784)
  (unrealized (0 1) (1 1))
  (origs (na-0 (1 2)) (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb na-0 nb-0 data) (a b name) (pt pt-0 pval)
    (priv-stor priv-stor-0 locn) (la lb rndx) (alpha beta expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta beta) (upsilon upsilon))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x la) (x beta))
  (gen-st (pv a la) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed la) (undisclosed lb))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb-0)))
  (label 785)
  (parent 784)
  (unrealized (0 0) (0 1) (1 0) (1 1))
  (origs (na-0 (1 2)) (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (lb rndx)
    (alpha beta expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (precedes ((2 1) (1 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x beta) (x l))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-0 (pv a l))) (1 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l)))))
  (label 786)
  (parent 785)
  (unrealized (0 0) (0 1) (1 1) (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (1 0)) ((3 2) (1 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)) (1 1))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 787)
  (parent 786)
  (unrealized (0 0) (0 1) (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 lb rndx)
    (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) zeta y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (1 3)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) zeta y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 4 0 resp 4)
    (enc na-0 nb-0 a b
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y-0 zeta-0))))
    (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))))
  (label 788)
  (parent 787)
  (unrealized (0 0) (0 1) (0 4) (1 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 lb-0 x y-0 rndx)
    (zeta-0 expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-2) (lb lb-0) (y y-0)
    (alpha (mul l l-0 (rec lb-0))) (zeta zeta-0))
  (precedes ((1 2) (4 2)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 3) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0)
    (y-0 (mul l l-0 (rec lb-0))) (y-0 lb-0) (y-0 zeta-0))
  (gen-st (pv a l) (pv b lb) (pv b lb-0))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na-0 nb-0 a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y-0 zeta-0))))
    (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-2 (cat pt-4 (pv b lb-0)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb-0))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0))))))))
  (label 789)
  (parent 787)
  (unrealized (0 0) (0 1) (1 3) (4 0) (4 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((1 2) (4 0)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na-0 nb-0 a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 790)
  (parent 787)
  (unrealized (0 0) (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x l l-0 lb rndx)
    (zeta expt) (y rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (defstrand init 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-0)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) zeta y)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 2) (0 2)) ((2 1) (1 0)) ((3 2) (1 1))
    ((4 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (precur (4 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) zeta y)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))
    (exp (gen) (mul (rec x) zeta y)) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (send nb))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) zeta y (rec w))) w))))
  (label 791)
  (parent 788)
  (unrealized (0 0) (0 1) (0 4) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (l x y-0 rndx) (zeta-0 expt) (l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (lb l-0) (y y-0) (alpha l) (zeta zeta-0))
  (precedes ((1 2) (4 2)) ((2 1) (1 0)) ((3 1) (4 0)) ((3 2) (1 1))
    ((4 3) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 l) (y-0 zeta-0)
    (y-0 l-0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0))))))))
  (label 792)
  (parent 789)
  (unrealized (0 0) (0 1) (1 3) (4 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (l l-0 x y-0 rndx) (zeta-0 expt)
    (l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-2) (lb l-1) (y y-0)
    (alpha (mul l l-0 (rec l-1))) (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (4 2)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 3) (1 3))
    ((5 1) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0)
    (y-0 (mul l l-0 (rec l-1))) (y-0 zeta-0) (y-0 l-1))
  (gen-st (pv a l) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-4 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-2 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-4 (pv b l-1)))))
  (label 793)
  (parent 789)
  (unrealized (0 0) (0 1) (1 3) (4 1))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 2) (1 1)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 794)
  (parent 790)
  (unrealized (0 0) (0 1) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x y-0 rndx) (zeta-0 expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 zeta-0) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 2 5 ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (4 1))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 795)
  (parent 792)
  (unrealized (0 0) (0 1) (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (6 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 796)
  (parent 794)
  (unrealized (0 0) (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (6 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 797)
  (parent 794)
  (unrealized (0 0) (0 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (lb-0 l x y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb-0 l)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x lb-0) (x l) (y-0 lb-0) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b lb-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed lb))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 3 resp 4) (exp (gen) y-1) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb-0)))
      (send
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))))
  (label 798)
  (parent 795)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x y-0 rndx) (zeta-0 expt) (l l-0 rndx)
    (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y-0 zeta-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l) (y y-0) (alpha l-0) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 zeta-0) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y-0 zeta-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))
    (exp (gen) (mul (rec x) y-0 zeta-0)) (1 3))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y-0 zeta-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul y-0 zeta-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y-0 zeta-0 (rec w))) w))))
  (label 799)
  (parent 795)
  (unrealized (0 0) (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0)))
  (label 800)
  (parent 796)
  (unrealized (0 0) (0 1) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (alpha expt)
    (y rndx) (zeta expt) (l x y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))))
  (label 801)
  (parent 798)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (lb l x y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-0))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 802)
  (parent 798)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (lb-0 l x rndx) (w expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb-0) (y y-0) (alpha l)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb-0 l)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x lb-0) (x l) (y-0 lb-0) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b lb-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb-0)))
      (send
        (sig (body b (exp (gen) lb-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb-0 l))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 803)
  (parent 799)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (5 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (gen-st (pv b lb) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 804)
  (parent 800)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (6 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 805)
  (parent 800)
  (unrealized (0 0) (0 1) (8 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 5 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 806)
  (parent 801)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 807)
  (parent 801)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 6 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l)))))
  (label 808)
  (parent 802)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0)) ((6 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 809)
  (parent 802)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (alpha expt)
    (y rndx) (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x l) (x l-0) (y-0 l)
    (y-0 (mul x w)) (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 810)
  (parent 803)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y alpha) (y zeta) (y l-0) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-0))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 811)
  (parent 803)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (5 0)) ((8 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y alpha) (y zeta) (y l-1) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1)))))
  (label 812)
  (parent 804)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 lb rndx) (alpha zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (precedes ((0 3) (8 0)) ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b l) (pv b lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y))))))
  (label 813)
  (parent 805)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-5 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 814)
  (parent 805)
  (unrealized (0 0) (0 1) (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (8 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 815)
  (parent 805)
  (unrealized (0 0) (0 1) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta upsilon expt) (l l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((1 2) (8 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 816)
  (parent 805)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 817)
  (parent 805)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l lb-0 rndx) (alpha-0 zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb-0) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 lb-0)
    (y-0 alpha-0) (y-0 zeta-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv b-0 lb-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-2 (cat pt-5 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 818)
  (parent 805)
  (unrealized (0 0) (0 1) (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((3 2) (8 0)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 819)
  (parent 805)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-2 (cat pt-5 ignore-1))
      (stor priv-stor-2 (cat pt-6 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 820)
  (parent 805)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (5 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 821)
  (parent 806)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 2) (0 1)) ((6 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 822)
  (parent 807)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 823)
  (parent 808)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 2) (0 1)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 824)
  (parent 809)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 6 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 825)
  (parent 810)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 826)
  (parent 810)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 7 4 ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l)))))
  (label 827)
  (parent 811)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 0))
    ((7 2) (0 1)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))))
  (label 828)
  (parent 811)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 9 7 ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 829)
  (parent 812)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (1 0))
    ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 830)
  (parent 812)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx) (beta expt)
    (x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (9 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((2 1) (9 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 831)
  (parent 814)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 1) (9 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)) ((3 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 832)
  (parent 814)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx)
    (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 na-1 l l-0 l-1)
  (uniq-gen y x x-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)) ((10 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-5 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-6 ignore-1))
      (stor priv-stor-2 (cat pt-5 (pv a-0 l-1)))))
  (label 833)
  (parent 814)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (alpha expt) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y alpha) (y zeta) (y l-1) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l-1))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) alpha) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1)))))
  (label 834)
  (parent 817)
  (unrealized (0 1) (0 4))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx)
    (alpha-0 zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((2 1) (9 0)) ((3 1) (7 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 alpha-0)
    (y-0 zeta-0) (y-0 l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((2 1) (9 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha-0 l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 835)
  (parent 818)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (lb rndx) (alpha expt)
    (y rndx) (zeta expt) (x rndx) (upsilon alpha-0 zeta-0 expt)
    (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-0) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 1) (9 0))
    ((3 2) (1 1)) ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 alpha-0)
    (y-0 zeta-0) (y-0 l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)) ((3 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 3 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha-0 l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 836)
  (parent 818)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt) (x rndx) (upsilon expt) (l rndx)
    (alpha-0 zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 2 (ignore ignore) (self a) (priv-stor priv-stor-0)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-1) (y y-0) (alpha alpha-0)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (5 0)) ((2 1) (1 0)) ((3 1) (7 0)) ((3 2) (1 1))
    ((4 1) (1 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y lb) (y alpha) (y zeta) (x l) (x l-0) (y-0 alpha-0)
    (y-0 zeta-0) (y-0 l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed lb))
  (leads-to ((2 1) (1 0)) ((3 1) (7 0)) ((10 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-5 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    ((load priv-stor-0 (cat pt-0 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-1 ignore))
      (stor priv-stor-0 (cat pt-0 (pv a l-0))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-2 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha-0) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha-0 l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-6 ignore-1))
      (stor priv-stor-2 (cat pt-5 (pv b-0 l-1)))))
  (label 837)
  (parent 818)
  (unrealized (0 0) (0 1) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (0 4)) ((6 1) (5 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 838)
  (parent 821)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 839)
  (parent 822)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 840)
  (parent 823)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 841)
  (parent 824)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 842)
  (parent 825)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 2) (0 1)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 843)
  (parent 826)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 844)
  (parent 827)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 2) (0 1)) ((8 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 845)
  (parent 828)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (9 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 846)
  (parent 829)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1))
    ((10 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 847)
  (parent 830)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0)) ((9 1) (0 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (displaced 10 8 ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0)))))
  (label 848)
  (parent 834)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0)) ((10 2) (0 1)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body self (exp (gen) l-2) (pubk "sig" self))
      (privk "sig" self)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 849)
  (parent 834)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (7 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (0 4)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 850)
  (parent 838)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (7 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (0 4)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 x) (y-0 l)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 851)
  (parent 838)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 852)
  (parent 839)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (8 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 853)
  (parent 839)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (8 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (0 0)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 854)
  (parent 840)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (0 0)) ((5 1) (8 0)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0)
    (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 855)
  (parent 840)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (9 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 856)
  (parent 841)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (9 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 857)
  (parent 841)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4)) ((7 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 858)
  (parent 842)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 859)
  (parent 843)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (0 4)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 860)
  (parent 844)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 2) (0 1)) ((8 1) (0 4)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 861)
  (parent 845)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (0 4)) ((10 1) (9 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 862)
  (parent 846)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 2) (0 1))
    ((10 1) (0 4)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 863)
  (parent 847)
  (unrealized (11 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 864)
  (parent 848)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 2) (0 1)) ((11 1) (0 4)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))) nb
    (0 4)
    (enc na nb self b
      (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 865)
  (parent 849)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 866)
  (parent 852)
  (unrealized (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-2 (cat pt-6 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1)))
  (label 867)
  (parent 856)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (8 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 868)
  (parent 858)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l) (y y-0) (alpha l-0) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (8 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (0 1)) ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0) (y-0 (mul x w)) (y-0 l)
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 869)
  (parent 858)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (9 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 870)
  (parent 859)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (9 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 871)
  (parent 859)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (9 0)) ((4 2) (0 1))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 872)
  (parent 860)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (lb x rndx) (w expt) (y-0 l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-0)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (0 1)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((6 1) (9 0)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-0) (y-0 lb) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv b l))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0) (undisclosed l))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-0)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-0))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 873)
  (parent 860)
  (unrealized (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (10 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 874)
  (parent 861)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (10 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 875)
  (parent 861)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((7 2) (5 0))
    ((8 1) (0 0)) ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 876)
  (parent 862)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (0 1)) ((7 2) (5 0)) ((8 1) (0 0))
    ((8 1) (11 0)) ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 877)
  (parent 862)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (12 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-2))
    (exp (gen) (mul l-1 l-2)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2))))
  (label 878)
  (parent 863)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (12 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-2) l-1))
    (exp (gen) (mul l-1 l-2)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1))))
  (label 879)
  (parent 863)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (0 4)) ((11 1) (10 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
  (label 880)
  (parent 864)
  (unrealized (11 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
  (label 881)
  (parent 865)
  (unrealized (12 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 882)
  (parent 866)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w))) w))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w))) w))
    (exp (gen) l-0) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w))) w))
      (send (cat (exp (gen) (mul l-0 (rec w))) w))))
  (label 883)
  (parent 866)
  (unrealized (10 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 884)
  (parent 867)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w))) w))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w))) w))
    (exp (gen) l-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w))) w))
      (send (cat (exp (gen) (mul l-0 (rec w))) w))))
  (label 885)
  (parent 867)
  (unrealized (11 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 886)
  (parent 870)
  (unrealized (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-2 (cat pt-6 (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1)))
  (label 887)
  (parent 874)
  (unrealized (10 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (12 0)
    (ch-msg priv-stor-2 (cat pt-7 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2)))
  (label 888)
  (parent 878)
  (unrealized (12 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((8 2) (7 0)) ((9 1) (0 0)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l-1))
    (exp (gen) (mul l-0 l-1)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1))))
  (label 889)
  (parent 880)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (upsilon expt) (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-1) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (0 1)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 1) (12 0)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-0))
    (exp (gen) (mul l-0 l-1)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-1))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-1) l)) (send (cat (exp (gen) l-1) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0))))
  (label 890)
  (parent 880)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (13 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-1) l-2))
    (exp (gen) (mul l-1 l-2)) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2))))
  (label 891)
  (parent 881)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (13 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-2) l-1))
    (exp (gen) (mul l-1 l-2)) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1))))
  (label 892)
  (parent 881)
  (unrealized (13 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 893)
  (parent 882)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((3 3) (10 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 l-0) (y-0 l-1)
    (y-0 x))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 894)
  (parent 883)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y l-0 l-1 rndx)
    (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (10 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y l) (y x) (y l-0) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 resp 4) (exp (gen) y-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 895)
  (parent 883)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (x-0 la) (x-0 beta))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 896)
  (parent 883)
  (unrealized (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (10 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 897)
  (parent 883)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (y-0 l la l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha la) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l la))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l la l-0)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 la) (y-0 l-0) (y-0 x) (x la)
    (x l-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a la) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed la)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y-0 x))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y-0 x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 898)
  (parent 883)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (10 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 899)
  (parent 883)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 lb rndx) (alpha zeta-0 expt)
    (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (y-1 lb) (y-1 alpha) (y-1 zeta-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 900)
  (parent 883)
  (unrealized (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((4 2) (10 0)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 x) (y-0 l-0)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 901)
  (parent 883)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((5 2) (10 0)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 5 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 902)
  (parent 883)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 903)
  (parent 883)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 904)
  (parent 884)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l l-0 lb l-1 x y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((3 3) (11 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 lb l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 l-1)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 905)
  (parent 885)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn)
    (lb l x y l-0 l-1 rndx) (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (11 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x lb) (x l) (y lb) (y l) (y x) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 0 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 906)
  (parent 885)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 la rndx) (beta expt)
    (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 la) (x-0 beta))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 907)
  (parent 885)
  (unrealized (9 0) (11 0) (12 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (11 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 908)
  (parent 885)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (y-0 l l-0 la lb x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha la) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l la))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((1 2) (11 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 la lb)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 la) (y-0 lb) (y-0 x) (x la)
    (x lb))
  (precur (11 0))
  (gen-st (pv a l) (pv a la) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed la)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul y-0 x))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul y-0 x)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 909)
  (parent 885)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((4 2) (11 0))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 x)
    (y-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 910)
  (parent 885)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (11 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 x)
    (y-0 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-3)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 911)
  (parent 885)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 lb-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb lb-0) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 lb-0) (y-1 alpha) (y-1 zeta-0))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 lb-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 912)
  (parent 885)
  (unrealized (9 0) (11 0) (12 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)) ((11 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 913)
  (parent 885)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((6 2) (11 0)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 6 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 914)
  (parent 885)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 915)
  (parent 885)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (9 0)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 916)
  (parent 886)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx) (w-0 expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
    (exp (gen) l-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul l-0 (rec w-0))) w-0))))
  (label 917)
  (parent 886)
  (unrealized (11 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (0 0)) ((11 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)) ((11 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-1)
    (10 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 918)
  (parent 887)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx) (w-0 expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
    (exp (gen) l-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul l-0 (rec w-0))) w-0))))
  (label 919)
  (parent 887)
  (unrealized (12 0))
  (comment "11 in cohort - 11 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 0)) ((13 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (12 0)) ((13 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 14 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 920)
  (parent 888)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul l-1 (rec w))) w))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-1 (rec w))) w))
    (exp (gen) l-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) (mul l-1 (rec w))) w))
      (send (cat (exp (gen) (mul l-1 (rec w))) w))))
  (label 921)
  (parent 888)
  (unrealized (14 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-2 (13 0)
    (ch-msg priv-stor-2 (cat pt-7 (pv self l-2))))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2)))
  (label 922)
  (parent 891)
  (unrealized (13 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (6 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 923)
  (parent 893)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x y-0 l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (11 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 x) (y-0 l-0)
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (11 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 924)
  (parent 896)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (11 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((2 1) (11 0)) ((4 1) (1 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 925)
  (parent 896)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 1) (11 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0) (x-0 beta) (x-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 926)
  (parent 896)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (x-0 beta) (x-0 l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 927)
  (parent 896)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (9 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 928)
  (parent 899)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x y-0 l rndx) (alpha zeta-0 expt) (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (11 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 x) (y-0 l-0)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (11 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 929)
  (parent 900)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 rndx) (alpha zeta-0 expt) (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (11 0)) ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((2 1) (11 0)) ((4 1) (1 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 930)
  (parent 900)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 rndx) (alpha zeta-0 expt) (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 1) (11 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 3) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 x)
    (y-0 l-0) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 931)
  (parent 900)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0))
    ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0))
    ((12 1) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 932)
  (parent 900)
  (unrealized (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 933)
  (parent 904)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x y-0 l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (12 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 x)
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (12 0)) ((5 1) (0 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 934)
  (parent 907)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (12 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 x)
    (y-0 l-2) (x-0 beta) (x-0 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((2 1) (12 0)) ((4 1) (1 0)) ((5 1) (0 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-3))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 935)
  (parent 907)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (12 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 beta) (x-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((5 1) (12 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 936)
  (parent 907)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 beta) (x-0 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 937)
  (parent 907)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (x-0 beta) (x-0 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0))
    ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv a-0 l-2)))))
  (label 938)
  (parent 907)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x y-0 l rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 1) (12 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 x)
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (12 0)) ((5 1) (0 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 939)
  (parent 912)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (12 0))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (0 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 x)
    (y-0 l-2) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((2 1) (12 0)) ((4 1) (1 0)) ((5 1) (0 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-3))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 940)
  (parent 912)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((5 1) (12 0)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((5 1) (12 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 5 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 941)
  (parent 912)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-2) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 942)
  (parent 912)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (0 0))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0))
    ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l) (y-0 x)
    (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) x) (gen)) (neq (exp (gen) zeta) (gen)) (neq a b)
    (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 1) (10 0))
    ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv b-0 l-2)))))
  (label 943)
  (parent 912)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 944)
  (parent 913)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 945)
  (parent 916)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx) (zeta-0 expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) zeta-0 y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((3 3) (11 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 l-0) (y-0 l-1)
    (y-0 zeta-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) zeta-0 y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta-0 y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta-0 y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
      (send (cat (exp (gen) y-0) (mul (rec x) zeta-0))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 946)
  (parent 917)
  (unrealized (1 3) (5 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x rndx) (w expt)
    (y l-0 l-1 rndx) (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (11 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y l) (y (mul x w)) (y l-0) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 0 resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 947)
  (parent 917)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 la rndx) (beta expt)
    (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (x-0 la) (x-0 beta))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 948)
  (parent 917)
  (unrealized (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (11 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w-0 l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 949)
  (parent 917)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta w expt)
    (y-0 l la l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la la) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha la) (zeta (mul w x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l la))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l la l-0)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 (mul w x)) (y-0 la) (y-0 l-0)
    (x la) (x l-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a la) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul w x)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed la) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul w y-0 x))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la l-0))
              (exp (gen) (mul w y-0 x)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 950)
  (parent 917)
  (unrealized (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (11 0)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 2 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 951)
  (parent 917)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 lb rndx)
    (alpha zeta-0 expt) (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (y-1 lb) (y-1 alpha) (y-1 zeta-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 952)
  (parent 917)
  (unrealized (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (w expt) (y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((4 2) (11 0)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 (mul x w))
    (y-0 l-0) (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 953)
  (parent 917)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((6 2) (11 0))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 6 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 954)
  (parent 917)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 955)
  (parent 917)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 956)
  (parent 918)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l l-0 lb l-1 x rndx) (zeta-0 expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul (rec x) zeta-0 y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1) (zeta zeta-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((3 3) (12 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 lb l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 l-1)
    (y-0 zeta-0))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) zeta-0 y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) zeta-0 y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul zeta-0 y-0)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) zeta-0)))
      (send (cat (exp (gen) y-0) (mul (rec x) zeta-0))))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 957)
  (parent 919)
  (unrealized (1 3) (5 0) (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (lb l x rndx)
    (w expt) (y l-0 l-1 rndx) (zeta expt) (y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y-0) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener
    (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (precedes ((0 3) (12 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)) (y-0 l-0) (y-0 l-1)
    (y-0 zeta))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 0 resp 4) (exp (gen) y-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y-0)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul zeta y-0)))))) (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send
        (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul zeta y-0)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0))))))
  (label 958)
  (parent 919)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 la rndx)
    (beta expt) (l-1 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 la) (x-0 beta))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 a-0 b-0 (exp (gen) x-0)))))
  (label 959)
  (parent 919)
  (unrealized (10 0) (12 0) (13 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (12 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w-0 l-1)) (gen) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 960)
  (parent 919)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta w expt) (y-0 l l-0 la lb x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha la) (zeta (mul w x)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l la))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((1 2) (12 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 la lb)
  (uniq-gen y y-0 x)
  (absent (y zeta) (y l) (y l-0) (y-0 (mul w x)) (y-0 la) (y-0 lb)
    (x la) (x lb))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a la) (pv b l-0) (pv b lb))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul w x)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed la) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 1 init 3) (exp (gen) x-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a la)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul w y-0 x))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul w x))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul la lb)) (exp (gen) (mul w y-0 x)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a la)))
      (send
        (sig (body a (exp (gen) la) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 961)
  (parent 919)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x rndx) (w expt) (y-0 l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((4 2) (12 0))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 (mul x w))
    (y-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 962)
  (parent 919)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (12 0)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 (mul x w))
    (y-0 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 2 ltx-gen 3) (exp (gen) l-3)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 963)
  (parent 919)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 lb-0 rndx)
    (alpha zeta-0 expt) (l-1 y-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb lb-0) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 lb-0) (y-1 alpha) (y-1 zeta-0))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 lb-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-1) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 lb-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul lb-0 alpha))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 964)
  (parent 919)
  (unrealized (10 0) (12 0) (13 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 0))
    ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0)) ((12 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 13 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 965)
  (parent 919)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((7 2) (12 0))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 7 ltx-gen 3) (exp (gen) l-2)
    (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 966)
  (parent 919)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (12 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-3 (cat pt-8 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self)))))
  (label 967)
  (parent 919)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((6 2) (4 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 0))
    ((12 2) (10 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (5 0)) ((6 1) (1 0)) ((7 1) (11 0)) ((12 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 968)
  (parent 920)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 lb rndx)
    (alpha zeta-0 expt) (l-2 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 969)
  (parent 921)
  (unrealized (12 0) (14 0) (15 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 l-2 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) (mul l-1 (rec y))))
  (precedes ((0 3) (14 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l-1) (y l-2) (y zeta))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 15 0 resp 4) (exp (gen) y-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) (mul l-1 (rec y))))
      (send (cat (exp (gen) y) (mul l-1 (rec y))))))
  (label 970)
  (parent 921)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 la rndx)
    (beta expt) (l-2 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 971)
  (parent 921)
  (unrealized (12 0) (14 0) (15 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (gen) l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (14 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-3 l-2) (w l-2)) (gen) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2))))
  (label 972)
  (parent 921)
  (unrealized (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta upsilon expt) (l l-0 la l-1 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-1) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-1))
  (deflistener
    (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) la) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l la))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((1 2) (14 0)) ((2 1) (6 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 la l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x l-1))
  (precur (14 0))
  (gen-st (pv b l-0) (pv b l-1) (pv self l) (pv self la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed la) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 15 1 init 3) (exp (gen) x-0) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul la l-1))
              (exp (gen) (mul upsilon x)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send
        (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) la) l-1)) (send (cat (exp (gen) la) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-7 (pv self l)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 973)
  (parent 921)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((7 2) (14 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-2) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 15 7 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 974)
  (parent 921)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((2 2) (14 0)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (14 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 15 2 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 975)
  (parent 921)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (11 0)) ((14 1) (0 0)) ((14 2) (13 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (13 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (12 0)) ((14 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 15 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 976)
  (parent 921)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((9 2) (14 0)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 15 9 ltx-gen 3) (exp (gen) l-3)
    (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 977)
  (parent 921)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l l-0 l-1 l-2 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-3) (14 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3)))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-10 (pv self-0 l-3)))
      (send
        (sig (body self-0 (exp (gen) l-3) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 978)
  (parent 921)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (0 0)) ((14 2) (12 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (13 0)) ((14 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 15 ltx-gen 3) (exp (gen) l-2)
    (13 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 979)
  (parent 922)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) (mul l-1 (rec w))) w))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-1 (rec w))) w))
    (exp (gen) l-1) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) (mul l-1 (rec w))) w))
      (send (cat (exp (gen) (mul l-1 (rec w))) w))))
  (label 980)
  (parent 922)
  (unrealized (15 0))
  (comment "10 in cohort - 10 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 4))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 981)
  (parent 923)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b b) (la l) (lb l-1) (alpha l-0) (beta l-1) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor) (x x)
        (upsilon y-0) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0))))
  (origs (l-1 (2 1)) (pt-0 (2 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 982)
  (parent 928)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 0)) ((7 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 983)
  (parent 933)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b self) (la l) (lb l-1) (alpha l-0) (beta lb) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor-2) (x x)
        (upsilon y-0) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0))))
  (origs (l-1 (7 1)) (pt-7 (7 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (lb (2 1)) (pt-0 (2 1))
    (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 984)
  (parent 944)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 985)
  (parent 945)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (w expt) (y-0 l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (12 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 (mul x w))
    (y-0 l-0) (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (12 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 986)
  (parent 948)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (12 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((2 1) (12 0)) ((4 1) (1 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 987)
  (parent 948)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 data)
    (a b b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0) (x-0 beta) (x-0 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 988)
  (parent 948)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (x-0 beta) (x-0 l-2))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0))
    ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a-0 l-2)))))
  (label 989)
  (parent 948)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (10 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (9 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 990)
  (parent 951)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (x rndx) (w expt) (y-0 l rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l-1) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l-1) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (12 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x l-0) (x l-1) (y-0 (mul x w))
    (y-0 l-0) (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (12 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv a l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 991)
  (parent 952)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 rndx) (alpha zeta-0 expt)
    (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 1) (12 0)) ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((2 1) (12 0)) ((4 1) (1 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 992)
  (parent 952)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 na-1 nb-1 data)
    (a b a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-0)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 1) (12 0)) ((6 2) (0 1))
    ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0))
    ((11 1) (9 0)) ((12 3) (11 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-0) (y-0 l) (y-0 (mul x w))
    (y-0 l-0) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-0)))
      (send
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 993)
  (parent 952)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (10 0)) ((6 2) (0 1)) ((7 1) (0 4))
    ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 3) (11 0)) ((13 1) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (5 0) (11 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (10 0))
    ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-6 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv b-0 l-2)))))
  (label 994)
  (parent 952)
  (unrealized (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((9 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 995)
  (parent 956)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x rndx) (w expt) (y-0 l rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (13 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 (mul x w))
    (y-0 l-1) (x-0 beta) (x-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-1) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (13 0)) ((6 1) (0 0))
    ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 996)
  (parent 959)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (13 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 (mul x w))
    (y-0 l-2) (x-0 beta) (x-0 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((2 1) (13 0)) ((4 1) (1 0)) ((6 1) (0 0))
    ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-3))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 997)
  (parent 959)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx) (beta expt)
    (x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (13 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 beta) (x-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((6 1) (13 0))
    ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 998)
  (parent 959)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 rndx) (beta expt)
    (l-0 x-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-2)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 1) (13 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 beta) (x-0 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0))
    ((7 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 999)
  (parent 959)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx)
    (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a a-0) (b b-0) (priv-stor priv-stor-3)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 2) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 na-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 x-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (x-0 beta) (x-0 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0))
    ((14 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv a-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-3 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv a-0 l-2)))))
  (label 1000)
  (parent 959)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb x rndx) (w expt) (y-0 l rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l-1)
    (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-1) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 1) (13 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l) (y l-0) (x lb) (x l-1) (y-0 lb) (y-0 (mul x w))
    (y-0 l-1) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-1)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((4 1) (13 0)) ((6 1) (0 0))
    ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 4 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l-1))
              (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-6 (pv a l)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-1 (cat pt-3 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1001)
  (parent 964)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (l x rndx) (w expt) (y-0 l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 1) (13 0))
    ((2 2) (1 1)) ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (1 3)) ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 l) (y-0 (mul x w))
    (y-0 l-2) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1) (pv b l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((2 1) (13 0)) ((4 1) (1 0)) ((6 1) (0 0))
    ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-3))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1002)
  (parent 964)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx)
    (alpha zeta-0 expt) (y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-1) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((6 1) (13 0)) ((7 1) (11 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((6 1) (13 0))
    ((7 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 6 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul (rec y-1) l-1)))
      (send (cat (exp (gen) y-1) (mul (rec y-1) l-1))))
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1003)
  (parent 964)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 rndx) (alpha zeta-0 expt)
    (l-0 y-1 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b a)
    (priv-stor priv-stor-2) (lb l-1) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 1) (13 0)) ((7 2) (0 1))
    ((8 1) (0 4)) ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0))
    ((12 1) (10 0)) ((13 3) (12 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 alpha) (y-1 zeta-0) (y-1 l-1))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-1) (pv b lb) (pv b l-0))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0))
    ((7 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 14 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-1)))
      (send
        (sig (body a (exp (gen) l-1) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-1) (mul l-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-0 (rec y-1)))))
    ((load priv-stor-2 (cat pt-6 (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-1 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-1))))))))
  (label 1004)
  (parent 964)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 rndx)
    (alpha zeta-0 expt) (l-1 y-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self a)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a-0) (b b-0)
    (priv-stor priv-stor-3) (lb l-2) (y y-1) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (5 0)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (1 3))
    ((6 1) (0 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 3) (12 0)) ((14 1) (13 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 nb-1 lb l l-0 l-1 l-2)
  (uniq-gen y x y-0 y-1)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)) (y-1 alpha) (y-1 zeta-0) (y-1 l-2))
  (precur (5 0) (12 0))
  (gen-st (pv a l) (pv a l-0) (pv b lb) (pv b l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) (mul x w)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((6 1) (0 0)) ((7 1) (11 0))
    ((14 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-8 (pv b-0 l-2))) (13 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-3 (pv a l)))
      (recv
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b lb)))
      (send
        (sig (body b (exp (gen) lb) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-1 (cat pt-2 ignore-0))
      (stor priv-stor-1 (cat pt-3 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((load priv-stor (cat pt-4 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-5 ignore-2))
      (stor priv-stor-2 (cat pt-6 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-6 (pv a l-0)))
      (stor priv-stor-2 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-1) (mul l-1 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul l-1 (rec y-1)))))
    ((load priv-stor-3 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-1 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-1)
          (enc na-1 nb-1 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-1)))))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-8 (pv b-0 l-2)))))
  (label 1005)
  (parent 964)
  (unrealized (10 0) (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (7 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (9 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (10 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)) ((11 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1006)
  (parent 965)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (1 0)) ((5 2) (3 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 0)) ((11 2) (9 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (4 0)) ((5 1) (1 0)) ((6 1) (10 0)) ((11 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1007)
  (parent 968)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (15 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-2)
    (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((7 1) (15 0)) ((8 1) (0 0))
    ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1008)
  (parent 969)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (15 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((2 1) (15 0)) ((7 1) (1 0)) ((8 1) (0 0))
    ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1009)
  (parent 969)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor) (lb l-2) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (15 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((8 1) (15 0))
    ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1010)
  (parent 969)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 1) (15 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 1) (15 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1011)
  (parent 969)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (l-2 y-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb l-3) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 3) (14 0)) ((16 1) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-3))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((16 1) (15 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 l-3)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul alpha l-3))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv b-0 l-3)))))
  (label 1012)
  (parent 969)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 1) (15 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-2) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((7 1) (15 0)) ((8 1) (0 0))
    ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 7 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1013)
  (parent 971)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (15 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((2 1) (15 0)) ((7 1) (1 0)) ((8 1) (0 0))
    ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1014)
  (parent 971)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx) (beta expt)
    (x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((8 1) (15 0))
    ((9 1) (13 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((8 1) (15 0))
    ((9 1) (13 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1015)
  (parent 971)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 1) (15 0)) ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0))
    ((12 1) (11 0)) ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 1) (15 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 16 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1016)
  (parent 971)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (beta expt) (l-2 x-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la l-3) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (1 0)) ((7 2) (5 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 2) (14 0)) ((16 1) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-3))
  (precur (14 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((7 1) (1 0)) ((8 1) (0 0)) ((9 1) (13 0))
    ((16 1) (15 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a l-3)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv a l-3)))))
  (label 1017)
  (parent 971)
  (unrealized (12 0) (14 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (1 0))
    ((6 2) (4 0)) ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0))
    ((13 1) (0 0)) ((13 2) (12 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (12 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (5 0)) ((6 1) (1 0)) ((7 1) (11 0)) ((13 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1018)
  (parent 976)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((7 2) (6 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 0)) ((13 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (5 0)) ((7 1) (1 0)) ((8 1) (12 0)) ((13 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1019)
  (parent 979)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 lb rndx)
    (alpha zeta-0 expt) (l-2 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1020)
  (parent 980)
  (unrealized (13 0) (15 0) (16 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x rndx)
    (upsilon expt) (l l-0 l-1 l-2 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y) (mul l-1 (rec y))))
  (precedes ((0 3) (15 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l-1) (y l-2) (y zeta))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 16 0 resp 4) (exp (gen) y-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y) (mul l-1 (rec y))))
      (send (cat (exp (gen) y) (mul l-1 (rec y))))))
  (label 1021)
  (parent 980)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 la rndx)
    (beta expt) (l-2 x-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la la) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0)))))
  (label 1022)
  (parent 980)
  (unrealized (13 0) (15 0) (16 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (gen) l-2))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (15 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-3 l-2) (w l-2)) (gen) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2))))
  (label 1023)
  (parent 980)
  (unrealized (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta upsilon expt) (l l-0 la l-1 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la la) (x x) (beta l-1) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-1))
  (deflistener
    (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) la) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l la))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((1 2) (15 0)) ((2 1) (6 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 la l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x l-1))
  (precur (7 0) (15 0))
  (gen-st (pv b l-0) (pv b l-1) (pv self l) (pv self la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed la) (undisclosed l-0))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 16 1 init 3) (exp (gen) x-0) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self la)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul la l-1))
              (exp (gen) (mul upsilon x)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send
        (hash (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l-1)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) la) l-1)) (send (cat (exp (gen) la) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv b l-1)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-1))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-2 (cat pt-7 (pv self l)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1024)
  (parent 980)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((8 2) (15 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-2) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 16 8 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1025)
  (parent 980)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((2 2) (15 0)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 16 2 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1026)
  (parent 980)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (11 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (13 0))
    ((9 2) (0 1)) ((10 1) (0 4)) ((11 1) (10 0)) ((12 1) (11 0))
    ((13 2) (12 0)) ((14 1) (12 0)) ((15 1) (0 0)) ((15 2) (14 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (14 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (13 0)) ((15 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 16 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1027)
  (parent 980)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (b self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((10 2) (15 0)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 16 10 ltx-gen 3) (exp (gen) l-3)
    (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2))))))
  (label 1028)
  (parent 980)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt)
    (l l-0 l-1 l-2 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
  (defstrand ltx-gen 3 (ignore ignore-3) (self self-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 l l-0 l-1 l-2 l-3)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-3) (15 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-3) (mul l-2 (rec l-3))))
      (send (cat (exp (gen) l-3) (mul l-2 (rec l-3)))))
    ((load priv-stor-3 (cat pt-9 ignore-3))
      (stor priv-stor-3 (cat pt-10 (pv self-0 l-3)))
      (send
        (sig (body self-0 (exp (gen) l-3) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1029)
  (parent 980)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 x)
    (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1030)
  (parent 982)
  (seen 981)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y-0))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 4)) ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 x))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) y-0) (gen)) (neq (exp (gen) x) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) x)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1031)
  (parent 984)
  (seen 983)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (6 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1032)
  (parent 985)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0))
    ((2 2) (1 1)) ((2 2) (9 0)) ((3 3) (1 3)) ((4 1) (1 0))
    ((4 2) (3 1)) ((5 1) (8 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1033)
  (parent 990)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1034)
  (parent 995)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((0 3) (6 0)) ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1))
    ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (8 0))
    ((5 2) (0 1)) ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1035)
  (parent 1006)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (9 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 0)) ((10 2) (8 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1036)
  (parent 1007)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (1 0)) ((5 2) (3 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (11 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (4 0)) ((5 1) (1 0)) ((6 1) (10 0)) ((12 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1037)
  (parent 1018)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4)) ((9 1) (8 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 0)) ((12 2) (10 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (4 0)) ((6 1) (1 0)) ((7 1) (11 0)) ((12 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1038)
  (parent 1019)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-1) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (16 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-2)
    (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((8 1) (16 0)) ((9 1) (0 0))
    ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1039)
  (parent 1020)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (16 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l)
    (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((2 1) (16 0)) ((8 1) (1 0)) ((9 1) (0 0))
    ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1040)
  (parent 1020)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b)
    (priv-stor priv-stor) (lb l-2) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (16 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((9 1) (16 0))
    ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-2)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1041)
  (parent 1020)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (alpha zeta-0 expt)
    (l-1 y-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 1) (16 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 3) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0))
    ((10 1) (16 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 10 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a self
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1042)
  (parent 1020)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 nb-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (alpha zeta-0 expt) (l-2 y-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
  (defstrand resp 4 (na na-1) (nb nb-1) (a a) (b b-0)
    (priv-stor priv-stor-3) (lb l-3) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-3) (self b-0)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 3) (15 0))
    ((17 1) (16 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 nb-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-3))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv b-0 l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta-0) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b) (undisclosed l-0)
    (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0))
    ((17 1) (16 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv b-0 l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-2 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-2 (rec y-0)))))
    ((load priv-stor-3 (cat pt-9 (pv b-0 l-3)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-1 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-1 nb-1 a b-0
            (hash (exp (gen) (mul alpha l-3))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv b-0 l-3)))))
  (label 1043)
  (parent 1020)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-2) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-2) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-2))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-1)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 1) (16 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-2) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((8 1) (16 0)) ((9 1) (0 0))
    ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 8 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-2) l)) (send (cat (exp (gen) l-2) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1044)
  (parent 1022)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l) (x x) (beta l-2) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l-2))
  (deflistener
    (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-2))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-2))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 1) (16 0))
    ((2 2) (1 1)) ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0))
    ((9 1) (0 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-2) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l-1) (pv b l-2) (pv self l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((2 1) (16 0)) ((8 1) (1 0)) ((9 1) (0 0))
    ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 2 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l)))
      (recv
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-2))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l-2)))
      (send
        (sig (body b (exp (gen) l-2) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-2)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-2)) (send (cat (exp (gen) l) l-2)))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-2))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-2 (cat pt-7 (pv self l-0)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1045)
  (parent 1022)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx) (beta expt)
    (x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
  (defstrand init 3 (na na-1) (a b) (b b-0) (priv-stor priv-stor)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 1) (16 0)) ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((9 1) (16 0))
    ((10 1) (14 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 9 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-2)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-2))))
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 b b-0 (exp (gen) x-0)))))
  (label 1046)
  (parent 1022)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg)
    (na nb na-0 nb-0 na-1 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 rndx) (beta expt)
    (l-1 x-0 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-2) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-2))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-2))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-1) (a self) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 1) (16 0)) ((10 2) (0 1)) ((11 1) (0 4))
    ((12 1) (11 0)) ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0))
    ((16 2) (15 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-2))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-1) (pv self l-0) (pv self l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-1))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0))
    ((10 1) (16 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 17 10 ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-1)))
      (recv
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-1))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-2)))
      (send
        (sig (body self (exp (gen) l-2) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-2))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv self l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-1 self b-0 (exp (gen) x-0)))))
  (label 1047)
  (parent 1022)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 ignore-3 mesg)
    (na nb na-0 nb-0 na-1 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pt-10 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 priv-stor-3 locn)
    (y rndx) (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 rndx)
    (beta expt) (l-2 x-0 l-3 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor-0)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b) (priv-stor priv-stor)
    (l l-2))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-2) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
  (defstrand init 3 (na na-1) (a a) (b b-0) (priv-stor priv-stor-3)
    (la l-3) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-3) (self a)
    (priv-stor priv-stor-3) (l l-3))
  (precedes ((0 3) (12 0)) ((1 2) (4 0)) ((2 1) (6 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((10 1) (14 0)) ((10 2) (0 1)) ((11 1) (0 4)) ((12 1) (11 0))
    ((13 1) (12 0)) ((14 2) (13 0)) ((15 1) (13 0)) ((16 2) (15 0))
    ((17 1) (16 0)))
  (non-orig (privk "sig" b) (privk "sig" self))
  (uniq-orig nb na-0 na-1 l l-0 l-1 l-2 l-3)
  (uniq-gen y x x-0)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0) (x-0 beta) (x-0 l-3))
  (precur (7 0) (15 0))
  (gen-st (pv b l) (pv b l-2) (pv self l-0) (pv self l-1) (pv a l-3))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)) ((10 1) (14 0))
    ((17 1) (16 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-3 (cat pt-9 (pv a l-3))) (16 0))
  (traces
    ((load priv-stor (cat pt (pv b l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-1 (cat pt-4 (pv self l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv b l)))
      (send
        (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 ignore-0))
      (stor priv-stor-1 (cat pt-4 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-5 ignore-1))
      (stor priv-stor (cat pt (pv b l-2))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (stor priv-stor-2 (cat pt-8 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-2 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-2 (rec x-0)))))
    ((load priv-stor-3 (cat pt-9 (pv a l-3)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-1 a b-0 (exp (gen) x-0))))
    ((load priv-stor-3 (cat pt-10 ignore-3))
      (stor priv-stor-3 (cat pt-9 (pv a l-3)))))
  (label 1048)
  (parent 1022)
  (unrealized (13 0) (15 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (10 0)) ((1 2) (3 0)) ((2 1) (5 0)) ((2 2) (1 1))
    ((3 1) (1 3)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((7 2) (6 0)) ((8 1) (12 0)) ((8 2) (0 1))
    ((9 1) (0 4)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (11 0)) ((14 1) (0 0)) ((14 2) (13 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (6 0) (13 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (5 0)) ((7 1) (1 0)) ((8 1) (12 0)) ((14 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1049)
  (parent 1027)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (0 4))
    ((2 2) (1 1)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0)))
  (label 1050)
  (parent 1032)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b b) (la l) (lb l-1) (alpha l-0) (beta l-1) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor) (x x)
        (upsilon (mul w y-0)) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0))))
  (origs (l-1 (2 1)) (pt-0 (2 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (8 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1051)
  (parent 1033)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 0)) ((7 2) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1052)
  (parent 1034)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a a) (b self) (la l) (lb l-1) (alpha l-0) (beta lb) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor-2) (x x)
        (upsilon (mul w y-0)) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0))))
  (origs (l-1 (7 1)) (pt-7 (7 1)) (pt-5 (6 1)) (l-0 (5 1)) (pt-4 (5 1))
    (nb-0 (3 3)) (l (4 1)) (pt-2 (4 1)) (lb (2 1)) (pt-0 (2 1))
    (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (7 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (8 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1053)
  (parent 1035)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (6 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (8 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1054)
  (parent 1036)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (7 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (9 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 1) (0 0)) ((11 2) (10 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (10 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (9 0)) ((11 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1055)
  (parent 1037)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 3)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 0)) ((11 2) (9 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((5 1) (1 0)) ((6 1) (10 0)) ((11 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1056)
  (parent 1038)
  (seen 1036)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (9 0)) ((2 1) (4 0)) ((2 2) (1 1)) ((3 1) (1 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((6 2) (5 0))
    ((7 1) (11 0)) ((7 2) (0 1)) ((8 1) (0 4)) ((9 1) (8 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (10 0)) ((13 1) (0 0))
    ((13 2) (12 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (5 0) (12 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (4 0)) ((6 1) (1 0)) ((7 1) (11 0)) ((13 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1057)
  (parent 1049)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta l-1)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((1 2) (3 2)) ((2 1) (0 0)) ((2 1) (3 0)) ((2 2) (1 1))
    ((2 2) (7 0)) ((3 3) (1 3)) ((4 1) (1 0)) ((4 2) (3 1))
    ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4)) ((7 1) (0 4)))
  (non-orig (privk "sig" a) (privk "sig" b))
  (uniq-orig nb na-0 nb-0 l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x l) (x l-1) (y-0 l) (y-0 (mul x w))
    (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv b l-1)))
      (send
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b))))
    ((load priv-stor (cat pt-0 (pv b l-1)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1058)
  (parent 1051)
  (seen 1050)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a self name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (lb l x rndx) (w expt) (y-0 l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-2) (lb l-1) (y y) (alpha l-0) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb)
    (upsilon (mul w y-0)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb lb) (y y-0) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self)
    (priv-stor priv-stor-2) (l l-1))
  (precedes ((1 2) (3 2)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 3) (1 3))
    ((4 1) (1 0)) ((4 2) (3 1)) ((5 1) (6 0)) ((5 2) (0 1))
    ((6 2) (0 4)) ((7 1) (0 4)) ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a) (privk "sig" self))
  (uniq-orig nb na-0 nb-0 lb l l-0 l-1)
  (uniq-gen y x y-0)
  (absent (y zeta) (y l-0) (y l-1) (x lb) (x l) (y-0 lb) (y-0 l)
    (y-0 (mul x w)))
  (precur (7 0))
  (gen-st (pv a l) (pv a l-0) (pv self lb) (pv self l-1))
  (facts (neq (exp (gen) (mul w y-0)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self) (undisclosed l) (undisclosed l-1))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-7 (pv self l-1)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na-0 a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y-0))
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0))))))
      (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y-0)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-1 (cat pt-4 (pv a l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-2 (cat pt-6 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1059)
  (parent 1053)
  (seen 1052)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1060)
  (parent 1054)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (6 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 0)) ((4 2) (1 3)) ((5 1) (8 0)) ((5 2) (0 1))
    ((6 1) (0 4)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1061)
  (parent 1055)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener
    (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 3) (8 0)) ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3))
    ((4 1) (1 3)) ((5 1) (1 0)) ((5 2) (4 0)) ((6 1) (10 0))
    ((6 2) (0 1)) ((7 1) (0 4)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 0)) ((12 2) (11 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (4 0) (11 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((5 1) (1 0)) ((6 1) (10 0)) ((12 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1062)
  (parent 1057)
  (seen 1055)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4))
    ((7 1) (0 0)) ((7 2) (0 4)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1063)
  (parent 1060)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((a self) (b self-0) (la l-0) (lb l-2) (alpha l-1) (beta l) (y y)
        (zeta zeta) (na na) (nb nb) (priv-stor priv-stor-2) (x x)
        (upsilon upsilon) (na-0 na-0) (nb-0 nb-0)
        (priv-stor-0 priv-stor-0))))
  (origs (l-2 (7 1)) (pt-8 (7 1)) (pt-6 (6 1)) (l-1 (5 1)) (pt-5 (5 1))
    (l-0 (4 1)) (pt-3 (4 1)) (pt-1 (3 1)) (l (2 1)) (pt-0 (2 1))
    (na-0 (1 2)) (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (7 0)) ((5 2) (0 1)) ((6 1) (0 4))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 2) (8 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-2) l-1)) (send (cat (exp (gen) l-2) l-1)))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1064)
  (parent 1061)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (y rndx)
    (zeta expt) (x rndx) (upsilon expt) (l l-0 l-1 l-2 rndx))
  (defstrand resp 5 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-2) (lb l-2) (y y) (alpha l-1) (zeta zeta))
  (defstrand init 5 (na na-0) (nb nb-0) (a self) (b self-0)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self-0)
    (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((2 1) (3 0)) ((2 2) (1 1)) ((3 2) (1 3)) ((4 1) (1 0))
    ((4 2) (1 3)) ((5 1) (6 0)) ((5 2) (0 1)) ((6 2) (0 4))
    ((7 1) (0 4)) ((8 1) (0 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" self) (privk "sig" self-0))
  (uniq-orig nb na-0 l l-0 l-1 l-2)
  (uniq-gen y x)
  (absent (y zeta) (y l-1) (y l-2) (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv self l-0) (pv self l-1) (pv self-0 l) (pv self-0 l-2))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self self-0) (undisclosed l-0) (undisclosed l-2))
  (leads-to ((2 1) (3 0)) ((4 1) (1 0)) ((5 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (recv
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l-1 l-2)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na-0 self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na-0 nb-0 self self-0
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))) (send nb-0))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((load priv-stor (cat pt-0 (pv self-0 l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-1 (cat pt-5 (pv self l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1065)
  (parent 1064)
  (seen 1063)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b) (undisclosed lb) (undisclosed alpha))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1066)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed lb)
    (undisclosed alpha))
  (rule assume-resp-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1067)
  (parent 1066)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (alpha expt) (y rndx) (zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed alpha))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1068)
  (parent 1067)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1069)
  (parent 1068)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1070)
  (parent 1069)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1071)
  (parent 1070)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1072)
  (parent 1071)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l)
    (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1073)
  (parent 1071)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b) (undisclosed alpha))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1074)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb rndx)
    (alpha expt) (y rndx) (zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed alpha))
  (rule assume-resp-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1075)
  (parent 1074)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (alpha expt) (y rndx) (zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed alpha))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1076)
  (parent 1075)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1077)
  (parent 1076)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1078)
  (parent 1077)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1079)
  (parent 1078)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1080)
  (parent 1079)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1081)
  (parent 1079)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1082)
  (parent 1081)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1083)
  (parent 1082)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1084)
  (parent 1082)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1085)
  (parent 1083)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb lb l)
  (uniq-gen y)
  (absent (y lb) (y l) (y zeta))
  (precur (7 0))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b lb))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) lb)) (send (cat (exp (gen) l) lb)))
    ((load priv-stor (cat pt (pv b lb)))
      (stor priv-stor (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 1086)
  (parent 1084)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 1087)
  (parent 1084)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1088)
  (parent 1084)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1089)
  (parent 1084)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1090)
  (parent 1084)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1091)
  (parent 1084)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1092)
  (parent 1084)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 4)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1093)
  (parent 1085)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 1094)
  (parent 1087)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 1095)
  (parent 1087)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1096)
  (parent 1087)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1097)
  (parent 1090)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1098)
  (parent 1090)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1099)
  (parent 1090)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1100)
  (parent 1091)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1101)
  (parent 1093)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (lb l) (alpha l-0) (y y) (zeta zeta) (na na) (nb nb)
        (priv-stor priv-stor))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1102)
  (parent 1100)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 4)) ((4 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1103)
  (parent 1102)
  (seen 1101)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b) (undisclosed lb))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1104)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed lb))
  (rule assume-resp-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1105)
  (parent 1104)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (y rndx) (alpha zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1106)
  (parent 1105)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1107)
  (parent 1106)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1108)
  (parent 1107)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1109)
  (parent 1108)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1110)
  (parent 1109)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1111)
  (parent 1109)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1112)
  (parent 1110)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((6 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1113)
  (parent 1112)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1114)
  (parent 1112)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1115)
  (parent 1113)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y l) (y l-0) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 1116)
  (parent 1114)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 1117)
  (parent 1114)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1118)
  (parent 1114)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (5 0)) ((7 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1119)
  (parent 1114)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1120)
  (parent 1114)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1121)
  (parent 1114)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1122)
  (parent 1114)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1123)
  (parent 1115)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 1124)
  (parent 1117)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 1125)
  (parent 1117)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1126)
  (parent 1117)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1127)
  (parent 1119)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1128)
  (parent 1120)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1129)
  (parent 1120)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b) (undisclosed l-0))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1130)
  (parent 1120)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 0))
    ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1131)
  (parent 1123)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b self) (lb l-0) (y y) (alpha l) (zeta zeta) (na na)
        (nb nb) (priv-stor priv-stor-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1132)
  (parent 1127)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 4))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self) (undisclosed l-0))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1133)
  (parent 1132)
  (seen 1131)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (facts (neq a b))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1134)
  (unrealized (0 1))
  (origs (nb (0 3)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (na nb data) (a b name) (pt pval) (priv-stor locn) (lb y rndx)
    (alpha zeta expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha alpha) (zeta zeta))
  (non-orig (privk "sig" a))
  (uniq-orig nb)
  (uniq-gen y)
  (absent (y lb) (y alpha) (y zeta))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (rule assume-resp-0 gen-st-ltx-disclose-0)
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb)))
  (label 1135)
  (parent 1134)
  (unrealized (0 0) (0 1))
  (origs (nb (0 3)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b name) (pt pt-0 pval)
    (priv-stor locn) (y rndx) (alpha zeta expt) (l rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l)
  (uniq-gen y)
  (absent (y alpha) (y zeta) (y l))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv b l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul alpha l)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l)))))
  (label 1136)
  (parent 1135)
  (unrealized (0 1) (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 2) (0 1)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand ltx-gen 3)
    (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)) (0 1))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))))
  (label 1137)
  (parent 1136)
  (unrealized (0 4))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) nb (0 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1138)
  (parent 1137)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
  (label 1139)
  (parent 1138)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1140)
  (parent 1139)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1141)
  (parent 1139)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1142)
  (parent 1140)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1143)
  (parent 1141)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (5 0)) ((6 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1144)
  (parent 1142)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1145)
  (parent 1142)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (5 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 2 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1146)
  (parent 1143)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1147)
  (parent 1143)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1148)
  (parent 1144)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y l) (y l-0) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 1149)
  (parent 1145)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 1150)
  (parent 1145)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1151)
  (parent 1145)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (5 0)) ((7 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1152)
  (parent 1145)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1153)
  (parent 1145)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1154)
  (parent 1145)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1155)
  (parent 1145)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (4 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1156)
  (parent 1146)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l rndx) (zeta expt) (y rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb lb) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
  (deflistener (cat (exp (gen) l) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (precedes ((0 3) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb lb l)
  (uniq-gen y)
  (absent (y lb) (y l) (y zeta))
  (precur (7 0))
  (gen-st (pv b lb))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b lb))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (hash (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y))))
      (send (cat (exp (gen) (mul lb l)) (exp (gen) (mul zeta y)))))
    ((recv (cat (exp (gen) l) lb)) (send (cat (exp (gen) l) lb)))
    ((load priv-stor (cat pt (pv b lb)))
      (stor priv-stor (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y))))))
  (label 1157)
  (parent 1147)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x la) (x beta))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x)))))
  (label 1158)
  (parent 1147)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1159)
  (parent 1147)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 1 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1160)
  (parent 1147)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1161)
  (parent 1147)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((2 2) (7 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1162)
  (parent 1147)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0 l-1)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1163)
  (parent 1147)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1164)
  (parent 1148)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 1165)
  (parent 1150)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 1166)
  (parent 1150)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1167)
  (parent 1150)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv b l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1168)
  (parent 1151)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 4))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1169)
  (parent 1152)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1170)
  (parent 1153)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1171)
  (parent 1153)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1172)
  (parent 1153)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (0 4)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1173)
  (parent 1156)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta beta expt) (l x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x)))))
  (label 1174)
  (parent 1158)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (beta expt) (x l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul (rec x) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x) (beta beta))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul (rec x) l-0)))
      (send (cat (exp (gen) x) (mul (rec x) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x)))))
  (label 1175)
  (parent 1158)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb na-0 l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x beta) (x l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv a-0 l-1))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1176)
  (parent 1158)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-2 (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1177)
  (parent 1159)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv b l-0))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor (cat pt (pv b l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1178)
  (parent 1161)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a b a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1179)
  (parent 1161)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 3) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb nb-0 l l-0 l-1)
  (uniq-gen y y-0)
  (absent (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (7 0))
  (gen-st (pv b l) (pv b-0 l-1))
  (facts (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((9 1) (8 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1180)
  (parent 1161)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 2) (0 1))
    ((2 2) (6 0)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1181)
  (parent 1162)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 0))
    ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1182)
  (parent 1164)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b self) (lb l-0) (y y) (alpha l) (zeta zeta) (na na)
        (nb nb) (priv-stor priv-stor-0))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1183)
  (parent 1168)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 4)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1184)
  (parent 1169)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1185)
  (parent 1173)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (lb l) (y y) (alpha l-0) (zeta zeta) (na na) (nb nb)
        (priv-stor priv-stor))))
  (origs (l-0 (2 1)) (pt-2 (2 1)) (pt-3 (3 1)) (l (1 1)) (pt (1 1))
    (nb (0 3))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 3) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0))
    ((2 2) (0 1)) ((3 1) (0 4)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (7 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1186)
  (parent 1177)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1187)
  (parent 1181)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1188)
  (parent 1183)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 4)) ((3 1) (0 4))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt-0 (pv a l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1189)
  (parent 1184)
  (seen 1182)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)) ((2 2) (0 1))
    ((3 1) (0 4)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (6 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1190)
  (parent 1186)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 2) (0 4)) ((4 1) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1191)
  (parent 1187)
  (seen 1185)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)) ((2 2) (0 1))
    ((3 2) (0 4)) ((4 1) (0 4)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor-0 (cat pt-2 (pv a l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1192)
  (parent 1188)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)) ((2 2) (0 1))
    ((3 2) (0 4)) ((4 1) (0 4)) ((5 2) (4 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1193)
  (parent 1190)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((2 2) (0 1)) ((3 1) (0 4))
    ((4 2) (3 0)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1194)
  (parent 1192)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (y rndx) (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((2 2) (0 1))
    ((3 2) (0 4)) ((4 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv b l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l-0)))
      (send
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv a l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1195)
  (parent 1193)
  (seen 1196)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand resp 5 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (lb l-0) (y y) (alpha l) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore) (self b) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 2) (0 1)) ((3 2) (0 4)))
  (non-orig (privk "sig" a))
  (uniq-orig nb l l-0)
  (uniq-gen y)
  (absent (y zeta) (y l) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-resp-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv b l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))
      (recv nb))
    ((load priv-stor (cat pt-0 ignore))
      (stor priv-stor (cat pt (pv b l-0))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv a l)))
      (send
        (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a))))
    ((load priv-stor (cat pt (pv b l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1196)
  (parent 1194)
  (realized)
  (shape)
  (maps
    ((0)
      ((a a) (b b) (lb l-0) (y y) (alpha l) (zeta zeta) (na na) (nb nb)
        (priv-stor priv-stor))))
  (origs (pt-3 (3 1)) (l (2 1)) (pt-2 (2 1)) (l-0 (1 1)) (pt (1 1))
    (nb (0 3))))

(comment "Nothing left to do")

(defprotocol dhcr-um diffie-hellman
  (defrole init
    (vars (la x rndx) (beta upsilon expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv a la))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))) (send nb))
    (uniq-orig na)
    (uniq-gen x)
    (absent (x la) (x beta))
    (assume (fact neq (exp (gen) upsilon) (gen)))
    (gen-st (pv a la)))
  (defrole resp
    (vars (lb y rndx) (alpha zeta expt) (a b name) (na nb data)
      (priv-stor locn))
    (trace (load priv-stor (pv b lb))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul y zeta)))))) (recv nb))
    (uniq-orig nb)
    (uniq-gen y)
    (absent (y lb) (y alpha) (y zeta))
    (assume (fact neq (exp (gen) zeta) (gen)))
    (gen-st (pv b lb)))
  (defrole ltx-gen
    (vars (self name) (l rndx) (priv-stor locn) (ignore mesg))
    (trace (load priv-stor ignore) (stor priv-stor (pv self l))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    (uniq-orig l))
  (defrole ltx-disclose
    (vars (self name) (l rndx) (priv-stor locn))
    (trace (load priv-stor (pv self l)) (stor priv-stor "nil") (send l))
    (gen-st (pv self l)))
  (defrule undisclosed-not-disclosed
    (forall ((z strd) (l rndx))
      (implies
        (and (fact undisclosed l) (p "ltx-disclose" z (idx 2))
          (p "ltx-disclose" "l" z l))
        (false))))
  (defgenrule neqRl_indx
    (forall ((x indx)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_strd
    (forall ((x strd)) (implies (fact neq x x) (false))))
  (defgenrule neqRl_mesg
    (forall ((x mesg)) (implies (fact neq x x) (false))))
  (defgenrule scissorsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (leads-to z0 i0 z2 i2))
        (and (= z1 z2) (= i1 i2)))))
  (defgenrule cakeRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (leads-to z0 i0 z1 i1)
          (leads-to z0 i0 z2 i2) (prec z1 i1 z2 i2)) (false))))
  (defgenrule no-interruption
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (leads-to z0 i0 z2 i2) (trans z1 i1)
          (same-locn z0 i0 z1 i1) (prec z0 i0 z1 i1) (prec z1 i1 z2 i2))
        (false))))
  (defgenrule shearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (trans z2 i2)
          (leads-to z0 i0 z1 i1) (same-locn z0 i0 z2 i2)
          (prec z0 i0 z2 i2))
        (or (and (= z1 z2) (= i1 i2)) (prec z1 i1 z2 i2)))))
  (defgenrule invShearsRule
    (forall ((z0 z1 z2 strd) (i0 i1 i2 indx))
      (implies
        (and (trans z0 i0) (trans z1 i1) (same-locn z0 i0 z1 i1)
          (leads-to z1 i1 z2 i2) (prec z0 i0 z2 i2))
        (or (and (= z0 z1) (= i0 i1)) (prec z0 i0 z1 i1)))))
  (defgenrule assume-init-0
    (forall ((z strd) (upsilon expt))
      (implies (and (p "init" z (idx 4)) (p "init" "upsilon" z upsilon))
        (fact neq (exp (gen) upsilon) (gen)))))
  (defgenrule assume-resp-0
    (forall ((z strd) (zeta expt))
      (implies (and (p "resp" z (idx 3)) (p "resp" "zeta" z zeta))
        (fact neq (exp (gen) zeta) (gen)))))
  (defgenrule trRl_ltx-gen-at-1
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-gen-at-0
    (forall ((z strd))
      (implies (p "ltx-gen" z (idx 2)) (trans z (idx 0)))))
  (defgenrule trRl_ltx-disclose-at-1
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 1)))))
  (defgenrule trRl_ltx-disclose-at-0
    (forall ((z strd))
      (implies (p "ltx-disclose" z (idx 2)) (trans z (idx 0)))))
  (defgenrule gen-st-init-0
    (forall ((z strd) (la rndx) (a name))
      (implies
        (and (p "init" z (idx 1)) (p "init" "la" z la)
          (p "init" "a" z a)) (gen-st (pv a la)))))
  (defgenrule gen-st-resp-0
    (forall ((z strd) (lb rndx) (b name))
      (implies
        (and (p "resp" z (idx 1)) (p "resp" "lb" z lb)
          (p "resp" "b" z b)) (gen-st (pv b lb)))))
  (defgenrule gen-st-ltx-disclose-0
    (forall ((z strd) (l rndx) (self name))
      (implies
        (and (p "ltx-disclose" z (idx 1)) (p "ltx-disclose" "l" z l)
          (p "ltx-disclose" "self" z self)) (gen-st (pv self l)))))
  (lang (sig sign) (body (tuple 3)) (pv (tuple 2))))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (la beta x rndx) (upsilon expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (uniq-orig na beta)
  (uniq-gen x)
  (absent (x la) (x beta))
  (facts (neq a b))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1197)
  (unrealized (0 1) (0 3))
  (origs (pt-1 (1 1)) (na (0 2)) (beta (1 1)))
  (comment "Not closed under rules"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (a b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (la beta x rndx) (upsilon expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (uniq-orig na beta)
  (uniq-gen x)
  (absent (x la) (x beta))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1198)
  (parent 1197)
  (unrealized (0 0) (0 1) (0 3))
  (origs (pt-1 (1 1)) (na (0 2)) (beta (1 1)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 2 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (0 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1199)
  (parent 1198)
  (unrealized (0 1) (0 3))
  (origs (l (1 1)) (pt-0 (1 1)) (na (0 2)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (beta x rndx)
    (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((2 1) (0 0)))
  (uniq-orig na beta l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor (cat pt (pv a l))) (0 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l)))))
  (label 1200)
  (parent 1198)
  (unrealized (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (precedes ((1 1) (0 0)) ((1 2) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 1 ltx-gen 3) (exp (gen) l) (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))))
  (label 1201)
  (parent 1199)
  (unrealized (0 3))
  (origs (l (1 1)) (pt-0 (1 1)) (na (0 2)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (0 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1202)
  (parent 1199)
  (unrealized (0 3) (2 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 3 1 ltx-gen 3) (exp (gen) l-0) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l)))))
  (label 1203)
  (parent 1200)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (beta x rndx)
    (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul beta (rec w))) w))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)))
  (uniq-orig na beta l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul beta (rec w))) w))
    (exp (gen) beta) (0 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul beta (rec w))) w))
      (send (cat (exp (gen) (mul beta (rec w))) w))))
  (label 1204)
  (parent 1200)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (l lb x y rndx) (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta zeta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y zeta))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta))))))))
  (label 1205)
  (parent 1201)
  (unrealized (0 3) (2 0) (2 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (2 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 1206)
  (parent 1201)
  (unrealized (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (la rndx) (beta expt) (l x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la la) (x x-0) (beta beta))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 la) (x-0 beta))
  (precur (2 0))
  (gen-st (pv self l) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1207)
  (parent 1202)
  (unrealized (0 1) (0 3) (2 0) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((2 1) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-0 l) (w l)) (gen) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l))))
  (label 1208)
  (parent 1202)
  (unrealized (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (lb rndx) (alpha zeta expt) (l y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 3) (2 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y lb) (y alpha) (y zeta))
  (precur (2 0))
  (gen-st (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1209)
  (parent 1202)
  (unrealized (0 1) (0 3) (2 0) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (precedes ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 3 1 ltx-gen 3) (exp (gen) l-0) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one)))))
  (label 1210)
  (parent 1202)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1211)
  (parent 1202)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 1212)
  (parent 1203)
  (unrealized (0 3) (3 0) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1213)
  (parent 1203)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (beta y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul beta (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (3 0)))
  (uniq-orig na nb-0 l beta)
  (uniq-gen x y)
  (absent (x l) (x beta) (y lb) (y alpha) (y zeta))
  (precur (3 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul beta (rec y))))
      (send (cat (exp (gen) y) (mul beta (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1214)
  (parent 1204)
  (unrealized (0 1) (0 3) (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (beta-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na na-0 l beta-0)
  (uniq-gen x x-0)
  (absent (x l) (x beta-0) (x-0 la) (x-0 beta))
  (precur (3 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta-0) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta-0)))
      (send
        (sig (body self (exp (gen) beta-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul beta-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1215)
  (parent 1204)
  (unrealized (0 1) (0 3) (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l beta rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) beta))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((3 1) (0 1)))
  (uniq-orig na l beta)
  (uniq-gen x)
  (absent (x l) (x beta))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (beta-0 beta) (w beta)) (gen) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) beta)) (send (cat (gen) beta))))
  (label 1216)
  (parent 1204)
  (unrealized (0 3) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 1 ltx-gen 3) (exp (gen) l-1) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1217)
  (parent 1204)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (beta l rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l beta))
  (deflistener (cat (exp (gen) l) (mul beta (rec l))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((2 1) (0 1)) ((3 1) (0 0)) ((3 2) (2 0)))
  (uniq-orig na beta l)
  (uniq-gen x)
  (absent (x beta) (x l))
  (precur (2 0))
  (gen-st (pv self-0 l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 4 ltx-gen 3) (exp (gen) l-0) (3 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (mul beta (rec l))))
      (send (cat (exp (gen) l) (mul beta (rec l)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1218)
  (parent 1204)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l beta l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (mul beta (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na l beta l-0)
  (uniq-gen x)
  (absent (x l) (x beta))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (mul beta (rec l-0))))
      (send (cat (exp (gen) l-0) (mul beta (rec l-0)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1219)
  (parent 1204)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0)))))
  (label 1220)
  (parent 1205)
  (unrealized (0 3) (2 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))) (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 1221)
  (parent 1206)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self b-0 name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon beta expt) (x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l) (x x-0) (beta beta))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1222)
  (parent 1207)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 2) (2 0)) ((4 1) (3 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0)))))
  (label 1223)
  (parent 1207)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l (2 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1224)
  (parent 1208)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pval) (priv-stor locn) (x rndx) (upsilon alpha zeta expt)
    (y l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 3) (2 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul (rec y) l)))
      (send (cat (exp (gen) y) (mul (rec y) l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 1225)
  (parent 1209)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (0 0)) ((2 1) (0 1)) ((3 3) (2 0)) ((4 1) (3 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b-0 l-0)))))
  (label 1226)
  (parent 1209)
  (unrealized (0 1) (0 3) (2 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor priv-stor-0 locn) (l lb x y rndx) (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y zeta))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta))))))))
  (label 1227)
  (parent 1210)
  (unrealized (0 3) (3 0) (3 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (enc na nb self b
      (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 1228)
  (parent 1210)
  (unrealized (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 4 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 1229)
  (parent 1212)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3))
    ((4 1) (3 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 1230)
  (parent 1212)
  (unrealized (0 3) (3 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 1) (0 3))
    ((4 1) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1231)
  (parent 1213)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (3 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1232)
  (parent 1214)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (beta y l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul beta (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 3) (3 0)))
  (uniq-orig na nb-0 beta l)
  (uniq-gen x y)
  (absent (x beta) (x l) (y alpha) (y zeta) (y l))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul beta (rec y))))
      (send (cat (exp (gen) y) (mul beta (rec y)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 1233)
  (parent 1214)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (beta y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) y) (mul beta (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (3 0)) ((5 1) (4 0)))
  (uniq-orig na nb-0 l beta l-0)
  (uniq-gen x y)
  (absent (x l) (x beta) (y alpha) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta)))
      (send
        (sig (body self (exp (gen) beta) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) y) (mul beta (rec y))))
      (send (cat (exp (gen) y) (mul beta (rec y)))))
    ((load priv-stor-1 (cat pt-3 (pv b-0 l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b-0 l-0)))))
  (label 1234)
  (parent 1214)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1235)
  (parent 1215)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (beta-0 x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor) (la l)
    (x x-0) (beta beta))
  (precedes ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na na-0 beta-0 l)
  (uniq-gen x x-0)
  (absent (x beta-0) (x l) (x-0 beta) (x-0 l))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta-0) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul beta-0 l))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta-0)))
      (send
        (sig (body self (exp (gen) beta-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul beta-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1236)
  (parent 1215)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (beta-0 x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta beta-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l beta-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (4 0)))
  (uniq-orig na na-0 l beta-0 l-0)
  (uniq-gen x x-0)
  (absent (x l) (x beta-0) (x-0 beta) (x-0 l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv a-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-0))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) beta-0) (pubk "sig" b))
          (privk "sig" b))) (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l beta-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self beta-0)))
      (send
        (sig (body self (exp (gen) beta-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) x-0) (mul beta-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul beta-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-0)))))
  (label 1237)
  (parent 1215)
  (unrealized (0 1) (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (3 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1238)
  (parent 1216)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 1239)
  (parent 1217)
  (unrealized (0 3) (4 0) (4 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1240)
  (parent 1217)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (2 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul l l)) l-0))
    (exp (gen) (mul l l (rec l-0))) (2 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0))))
  (label 1241)
  (parent 1220)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l l (rec l-0))) (2 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1242)
  (parent 1220)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 1243)
  (parent 1221)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((2 2) (0 1)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1244)
  (parent 1224)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l) (a self) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (self self)
        (priv-stor-0 priv-stor) (ignore ignore))))
  (origs (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0)))))
  (label 1245)
  (parent 1227)
  (unrealized (0 3) (3 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))) (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))))
  (label 1246)
  (parent 1228)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 4 ltx-gen 3) (exp (gen) l-0) (3 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1247)
  (parent 1229)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1248)
  (parent 1229)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1249)
  (parent 1230)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 1250)
  (parent 1230)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (3 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 1251)
  (parent 1230)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1252)
  (parent 1231)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1253)
  (parent 1231)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx)
    (zeta expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y zeta))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation encryption-test (added-strand resp 4)
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 1254)
  (parent 1238)
  (unrealized (0 3) (5 0) (5 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (5 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (enc na nb a b
      (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send
        (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1255)
  (parent 1238)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 5 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 1256)
  (parent 1239)
  (unrealized (0 3) (4 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)) ((5 1) (4 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 1257)
  (parent 1239)
  (unrealized (0 3) (4 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 1) (0 3)) ((5 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1258)
  (parent 1240)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1259)
  (parent 1241)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1260)
  (parent 1242)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (la rndx) (beta expt) (l x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv self l) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1261)
  (parent 1243)
  (unrealized (3 0) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (upsilon expt) (l x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) x) (mul l l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) x) (mul l l (rec x))))
      (send (cat (exp (gen) x) (mul l l (rec x))))))
  (label 1262)
  (parent 1243)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-0 l) (w (mul l l))) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l)))))
  (label 1263)
  (parent 1243)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (lb rndx) (alpha zeta expt) (l y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 3) (4 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y lb) (y alpha) (y zeta))
  (precur (4 0))
  (gen-st (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1264)
  (parent 1243)
  (unrealized (3 0) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l))))
  (label 1265)
  (parent 1243)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1266)
  (parent 1243)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul l l)) l-0))
    (exp (gen) (mul l l (rec l-0))) (3 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0))))
  (label 1267)
  (parent 1245)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l l (rec l-0))) (3 1))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1268)
  (parent 1245)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 1269)
  (parent 1246)
  (unrealized (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (0 0)) ((3 2) (2 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1270)
  (parent 1247)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a self-0) (b self) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore))))
  (origs (nb (2 3)) (l (3 1)) (pt-2 (3 1)) (lb (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1271)
  (parent 1247)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 1272)
  (parent 1248)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 0 init 3) (exp (gen) x-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1273)
  (parent 1248)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1274)
  (parent 1248)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 1 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1275)
  (parent 1248)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1276)
  (parent 1248)
  (unrealized (0 3) (3 1) (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 5 ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1277)
  (parent 1248)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1278)
  (parent 1248)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 1279)
  (parent 1249)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 1280)
  (parent 1250)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 1281)
  (parent 1251)
  (unrealized (0 3) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1282)
  (parent 1252)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1283)
  (parent 1253)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta))))))))
  (label 1284)
  (parent 1254)
  (unrealized (0 3) (5 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 1285)
  (parent 1254)
  (unrealized (0 3) (5 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))))
  (label 1286)
  (parent 1255)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (0 0)) ((4 2) (3 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 5 ltx-gen 3) (exp (gen) l-0) (4 1))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1287)
  (parent 1256)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (l x y rndx)
    (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1288)
  (parent 1256)
  (unrealized (0 3) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (4 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1289)
  (parent 1257)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 1290)
  (parent 1257)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (4 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 1291)
  (parent 1257)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1292)
  (parent 1258)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1293)
  (parent 1258)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 1294)
  (parent 1259)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1295)
  (parent 1260)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self b-0 name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon beta expt) (x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l l)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l) (x x-0) (beta beta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l l)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1296)
  (parent 1261)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (5 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0)))))
  (label 1297)
  (parent 1261)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l) (mul l l) (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l)))
  (label 1298)
  (parent 1263)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pval) (priv-stor locn) (x rndx) (upsilon alpha zeta expt)
    (y l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul (rec y) l l)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 3) (4 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul (rec y) l l)))
      (send (cat (exp (gen) y) (mul (rec y) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 1299)
  (parent 1264)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 3) (4 0)) ((6 1) (5 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b-0 l-0)))))
  (label 1300)
  (parent 1264)
  (unrealized (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l (4 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1301)
  (parent 1265)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1302)
  (parent 1267)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1303)
  (parent 1268)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (la rndx) (beta expt) (l x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 la) (x-0 beta))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1304)
  (parent 1269)
  (unrealized (4 0) (5 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (upsilon expt) (l x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) x) (mul l l (rec x))))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) x) (mul l l (rec x))))
      (send (cat (exp (gen) x) (mul l l (rec x))))))
  (label 1305)
  (parent 1269)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-0 l) (w (mul l l))) (gen) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l)))))
  (label 1306)
  (parent 1269)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (lb rndx) (alpha zeta expt) (l y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 3) (5 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y lb) (y alpha) (y zeta))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1307)
  (parent 1269)
  (unrealized (4 0) (5 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((1 2) (5 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l))))
  (label 1308)
  (parent 1269)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1309)
  (parent 1269)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (4 0))
    ((3 1) (0 0)) ((3 2) (2 1)) ((4 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (4 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1310)
  (parent 1271)
  (seen 1247)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 1311)
  (parent 1272)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 1312)
  (parent 1272)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (5 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 1313)
  (parent 1272)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (4 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1314)
  (parent 1274)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (3 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1315)
  (parent 1276)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 6 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1316)
  (parent 1276)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 3) (4 0)) ((6 1) (5 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 1317)
  (parent 1276)
  (unrealized (0 3) (3 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1318)
  (parent 1277)
  (seen 1270)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1319)
  (parent 1277)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1320)
  (parent 1279)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1321)
  (parent 1279)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1322)
  (parent 1280)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1323)
  (parent 1280)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1324)
  (parent 1281)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1325)
  (parent 1281)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 7 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1326)
  (parent 1282)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1327)
  (parent 1282)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1328)
  (parent 1283)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1329)
  (parent 1283)
  (unrealized (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (0 0)) ((5 2) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 6 ltx-gen 3) (exp (gen) l-0) (5 1))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1330)
  (parent 1284)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y l) (y zeta) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1331)
  (parent 1284)
  (unrealized (0 3) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l-0)) l-1))
    (exp (gen) (mul l l-0 (rec l-1))) (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1))))
  (label 1332)
  (parent 1285)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec l-1))) l-0))
    (exp (gen) (mul l l-0 (rec l-1))) (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0))))
  (label 1333)
  (parent 1285)
  (seen 1410)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l-0 (rec l-1))) l))
    (exp (gen) (mul l l-0 (rec l-1))) (5 1))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l))))
  (label 1334)
  (parent 1285)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1335)
  (parent 1286)
  (seen 1412)
  (unrealized (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1336)
  (parent 1286)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (0 0)) ((4 2) (3 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1337)
  (parent 1287)
  (seen 1270)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1338)
  (parent 1287)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 1339)
  (parent 1288)
  (unrealized (0 3) (4 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (5 0)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 0 init 3) (exp (gen) x-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1340)
  (parent 1288)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1341)
  (parent 1288)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 1 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1342)
  (parent 1288)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1343)
  (parent 1288)
  (unrealized (0 3) (4 1) (5 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 6 ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1344)
  (parent 1288)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1345)
  (parent 1288)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1)))
  (label 1346)
  (parent 1289)
  (unrealized (0 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1)))
  (label 1347)
  (parent 1290)
  (unrealized (0 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (6 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1)))
  (label 1348)
  (parent 1291)
  (unrealized (0 3) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0)))
  (label 1349)
  (parent 1292)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1350)
  (parent 1293)
  (unrealized (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1351)
  (parent 1294)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 l-0 (rec x))))))
  (label 1352)
  (parent 1294)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w (mul l-0 l-0))) (gen)
    (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0)))))
  (label 1353)
  (parent 1294)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((1 2) (6 0))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0))))
  (label 1354)
  (parent 1294)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1355)
  (parent 1294)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (2 0))
    ((6 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((6 1) (2 0)) ((6 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 3 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1356)
  (parent 1294)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1357)
  (parent 1294)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1358)
  (parent 1295)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l la x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la la)) (y l) (x la))
  (precur (6 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1359)
  (parent 1295)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w (rec l-0))) (gen)
    (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0)))))
  (label 1360)
  (parent 1295)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))))
  (label 1361)
  (parent 1295)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1362)
  (parent 1295)
  (unrealized (0 3) (4 0) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 3 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1363)
  (parent 1295)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1364)
  (parent 1295)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l (5 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1365)
  (parent 1298)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((1 2) (3 0)) ((2 1) (0 3)) ((3 1) (2 0)) ((4 2) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1366)
  (parent 1301)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l x y rndx) (zeta expt) (l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec l-0))) (y zeta) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l l (rec w))) w))
    (exp (gen) (mul l l)) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l (rec w))) w))
      (send (cat (exp (gen) (mul l l (rec w))) w))))
  (label 1367)
  (parent 1302)
  (unrealized (0 3) (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1368)
  (parent 1303)
  (unrealized (0 3) (7 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 data) (b self b-0 name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon beta expt) (x-0 l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l l)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na na-0 l)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l l)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1369)
  (parent 1304)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x-0 beta) (x-0 l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((7 1) (6 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv a l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv a l-0)))))
  (label 1370)
  (parent 1304)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)))
  (rule assume-init-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l) (mul l l) (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l)))
  (label 1371)
  (parent 1306)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pval) (priv-stor locn) (x rndx) (upsilon alpha zeta expt)
    (y l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul (rec y) l l)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 3) (5 0)))
  (uniq-orig na nb-0 l)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul (rec y) l l)))
      (send (cat (exp (gen) y) (mul (rec y) l l))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l))
              (exp (gen) (mul zeta y))))))))
  (label 1372)
  (parent 1307)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) y) (mul l l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 3) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (y alpha) (y zeta) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv b-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((7 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-0 (cat pt-1 (pv b-0 l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) y) (mul l l (rec y))))
      (send (cat (exp (gen) y) (mul l l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv b-0 l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b-0 l-0)))))
  (label 1373)
  (parent 1307)
  (unrealized (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((1 2) (5 0)) ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l (5 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1374)
  (parent 1308)
  (seen 1301)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1375)
  (parent 1314)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1376)
  (parent 1314)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 3) (5 0))
    ((3 1) (2 1)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 2 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1377)
  (parent 1319)
  (seen 1310)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 1378)
  (parent 1320)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1379)
  (parent 1321)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1380)
  (parent 1322)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1381)
  (parent 1323)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1382)
  (parent 1324)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 1383)
  (parent 1325)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1384)
  (parent 1326)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1385)
  (parent 1327)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1386)
  (parent 1327)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1387)
  (parent 1327)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (7 0))
    ((2 1) (0 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1388)
  (parent 1327)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1389)
  (parent 1327)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1390)
  (parent 1327)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1391)
  (parent 1327)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1392)
  (parent 1328)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1393)
  (parent 1329)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (7 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1394)
  (parent 1329)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1395)
  (parent 1329)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1396)
  (parent 1329)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1397)
  (parent 1329)
  (unrealized (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((7 1) (0 0)) ((7 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1398)
  (parent 1329)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1399)
  (parent 1329)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (0 0)) ((5 2) (4 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1400)
  (parent 1330)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1401)
  (parent 1330)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self a-0 b name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0)))))
  (label 1402)
  (parent 1331)
  (unrealized (0 3) (5 1) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y l) (y l-0) (x l) (x l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 0 init 3) (exp (gen) x-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1403)
  (parent 1331)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1404)
  (parent 1331)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 2) (6 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 1 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1405)
  (parent 1331)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 lb) (y-0 alpha)
    (y-0 zeta-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv b lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1406)
  (parent 1331)
  (unrealized (0 3) (5 1) (6 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 7 ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1407)
  (parent 1331)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1408)
  (parent 1331)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (8 0))
    ((7 1) (5 1)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1409)
  (parent 1332)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1))))
  (label 1410)
  (parent 1333)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1411)
  (parent 1334)
  (unrealized (0 3) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1412)
  (parent 1335)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1413)
  (parent 1336)
  (unrealized (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (5 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1414)
  (parent 1338)
  (seen 1310)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 1) (6 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 1415)
  (parent 1339)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 1416)
  (parent 1339)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((7 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv a-0 l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv a-0 l-1)))))
  (label 1417)
  (parent 1339)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (5 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 1418)
  (parent 1341)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 1) (6 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1419)
  (parent 1343)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 3) (5 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 7 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1420)
  (parent 1343)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 3) (5 0))
    ((7 1) (6 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((7 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1)))))
  (label 1421)
  (parent 1343)
  (unrealized (0 3) (4 1) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0) (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1422)
  (parent 1344)
  (seen 1318)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1423)
  (parent 1344)
  (unrealized (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1424)
  (parent 1346)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1425)
  (parent 1346)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1426)
  (parent 1347)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1427)
  (parent 1347)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1428)
  (parent 1348)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1429)
  (parent 1348)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1430)
  (parent 1349)
  (seen 1326)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1431)
  (parent 1349)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1432)
  (parent 1350)
  (seen 1328)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1433)
  (parent 1350)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1434)
  (parent 1351)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1435)
  (parent 1351)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 1436)
  (parent 1351)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-0) (mul l-0 l-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0)))
  (label 1437)
  (parent 1353)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1438)
  (parent 1354)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1439)
  (parent 1355)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1440)
  (parent 1355)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1441)
  (parent 1355)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l) (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1442)
  (parent 1358)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1443)
  (parent 1358)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((8 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 1444)
  (parent 1358)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-0) (rec l-0) (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0)))
  (label 1445)
  (parent 1360)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((3 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l) (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1446)
  (parent 1362)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 3 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1447)
  (parent 1362)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((8 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1448)
  (parent 1362)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (7 0)) ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (2 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-0) (mul (rec l-0) (rec l-0))
    (5 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0)))
  (label 1449)
  (parent 1363)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (0 3)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1450)
  (parent 1365)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (2 0))
    ((2 1) (0 3)) ((3 2) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1451)
  (parent 1366)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 la)
    (x-0 beta))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1452)
  (parent 1367)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 l-0 (rec x))))))
  (label 1453)
  (parent 1367)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w (mul l-0 l-0))) (gen)
    (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0)))))
  (label 1454)
  (parent 1367)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((1 2) (7 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0))))
  (label 1455)
  (parent 1367)
  (unrealized (0 3) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1456)
  (parent 1367)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (3 0)) ((7 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((7 1) (3 0)) ((7 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1457)
  (parent 1367)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1458)
  (parent 1367)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self a b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 la)
    (x-0 beta))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv a la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1459)
  (parent 1368)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (y rndx)
    (zeta expt) (l la x rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta la) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) la) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) la la))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l la)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la la)) (y l) (x la))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self la))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 0 init 3) (exp (gen) x-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self la)))
      (recv
        (sig (body b (exp (gen) la) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self la)))
      (send
        (sig (body self (exp (gen) la) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) la) (one)))
      (send (cat (exp (gen) la) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) la la)) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul la la)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt-0 (pv self la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1460)
  (parent 1368)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w (rec l-0))) (gen)
    (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0)))))
  (label 1461)
  (parent 1368)
  (unrealized (0 3) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l) (rec l-0))))))
  (label 1462)
  (parent 1368)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l lb rndx) (alpha zeta-0 expt) (l-0 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 lb)
    (y-0 alpha) (y-0 zeta-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1463)
  (parent 1368)
  (unrealized (0 3) (5 0) (7 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((7 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 8 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1464)
  (parent 1368)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1465)
  (parent 1368)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l (6 0)
    (ch-msg priv-stor (cat pt-0 (pv self l))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1466)
  (parent 1371)
  (seen 1365)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1467)
  (parent 1375)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (6 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1468)
  (parent 1376)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1469)
  (parent 1378)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1470)
  (parent 1378)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 1471)
  (parent 1379)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1472)
  (parent 1379)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1473)
  (parent 1380)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1474)
  (parent 1380)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1475)
  (parent 1381)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 1476)
  (parent 1382)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1477)
  (parent 1382)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1478)
  (parent 1383)
  (unrealized (0 3) (9 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1479)
  (parent 1384)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1480)
  (parent 1385)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1481)
  (parent 1385)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((9 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1482)
  (parent 1385)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1483)
  (parent 1387)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((1 1) (8 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1484)
  (parent 1389)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1485)
  (parent 1389)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1486)
  (parent 1389)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1487)
  (parent 1390)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1488)
  (parent 1392)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1489)
  (parent 1393)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1490)
  (parent 1393)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1491)
  (parent 1393)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1492)
  (parent 1395)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1493)
  (parent 1396)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1494)
  (parent 1397)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (8 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (7 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1495)
  (parent 1397)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (7 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1496)
  (parent 1397)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (0 0)) ((4 2) (3 1)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1497)
  (parent 1400)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (6 0)) ((5 1) (0 0)) ((5 2) (4 1))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (6 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1498)
  (parent 1401)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 self b (exp (gen) x-0)))))
  (label 1499)
  (parent 1402)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a self b name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b) (priv-stor priv-stor) (la l-0)
    (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a b (exp (gen) x-0)))))
  (label 1500)
  (parent 1402)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((8 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b (exp (gen) beta) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na-0 a-0 b (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1501)
  (parent 1402)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (6 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1502)
  (parent 1404)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha l) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (7 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1503)
  (parent 1406)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (a self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 3) (6 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-0))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 8 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1504)
  (parent 1406)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a self a-0 b name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 3) (6 0)) ((8 1) (7 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0) (y-0 alpha) (y-0 zeta-0)
    (y-0 l-1))
  (precur (3 0) (6 0))
  (gen-st (pv a l-0) (pv self l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((8 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1)))))
  (label 1505)
  (parent 1406)
  (unrealized (0 3) (5 1) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (2 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1506)
  (parent 1407)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1507)
  (parent 1407)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1508)
  (parent 1409)
  (seen 1634)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 x y rndx)
    (zeta expt) (l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l-0) l))
    (exp (gen) (mul l l-0)) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l))))
  (label 1509)
  (parent 1409)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1510)
  (parent 1410)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1511)
  (parent 1410)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) l) l-0))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0))))
  (label 1512)
  (parent 1411)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec l-0)) l))
    (exp (gen) (mul l (rec l-0))) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l))))
  (label 1513)
  (parent 1411)
  (seen 1639)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 8 ltx-gen 3) (exp (gen) l-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1514)
  (parent 1412)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1515)
  (parent 1412)
  (unrealized (8 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1516)
  (parent 1413)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1517)
  (parent 1413)
  (unrealized (9 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1518)
  (parent 1418)
  (seen 1375)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1519)
  (parent 1418)
  (unrealized (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pval) (priv-stor priv-stor-0 locn) (lb l x rndx)
    (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (3 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (4 0) (6 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 7 3 resp 4) (exp (gen) y-0) (6 0))
  (traces
    ((load priv-stor-0 (cat pt-2 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1520)
  (parent 1423)
  (seen 1377)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 1521)
  (parent 1424)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1522)
  (parent 1425)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1523)
  (parent 1426)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1524)
  (parent 1427)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1525)
  (parent 1428)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1)))
  (label 1526)
  (parent 1429)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1527)
  (parent 1431)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1528)
  (parent 1431)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1529)
  (parent 1431)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1530)
  (parent 1431)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1531)
  (parent 1431)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1532)
  (parent 1431)
  (seen 1390)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1533)
  (parent 1431)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1534)
  (parent 1433)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (8 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (3 0) (8 0))
  (gen-st (pv a la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-3 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1535)
  (parent 1433)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1536)
  (parent 1433)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1537)
  (parent 1433)
  (seen 1396)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1538)
  (parent 1433)
  (unrealized (6 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (2 0)) ((2 1) (0 1)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((8 1) (0 0)) ((8 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-1 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1539)
  (parent 1433)
  (unrealized (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1540)
  (parent 1433)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1541)
  (parent 1437)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1542)
  (parent 1438)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (8 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1543)
  (parent 1438)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1544)
  (parent 1445)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (2 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor-0 (cat pt-3 (pv self-0 l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1545)
  (parent 1449)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1546)
  (parent 1450)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1547)
  (parent 1451)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l) (a self) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (self self)
        (priv-stor-0 priv-stor) (ignore ignore))))
  (origs (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1548)
  (parent 1452)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l l (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1549)
  (parent 1452)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 1550)
  (parent 1452)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-0) (mul l-0 l-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0)))
  (label 1551)
  (parent 1454)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (7 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1552)
  (parent 1455)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-0 l-0))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1553)
  (parent 1456)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l l)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l l (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l)) l-0))
      (send (cat (exp (gen) (mul l l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l l (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l l (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1554)
  (parent 1456)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1555)
  (parent 1456)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l) (rec x-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1556)
  (parent 1459)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb na-0 l l-0)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1557)
  (parent 1459)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (x-0 beta)
    (x-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv a l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a l-1)))))
  (label 1558)
  (parent 1459)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-0) (rec l-0) (7 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0)))
  (label 1559)
  (parent 1461)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta alpha zeta-0 expt) (l y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b self)
    (priv-stor priv-stor) (lb l-0) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0))
    ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((1 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l) (rec y-0)))))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1560)
  (parent 1463)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data) (b self a name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l rndx) (alpha zeta-0 expt) (y-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)))
  (uniq-orig na nb nb-0 l l-0)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 9 4 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1561)
  (parent 1463)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (b self a b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 3) (7 0)) ((9 1) (8 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0) (y-0 alpha)
    (y-0 zeta-0) (y-0 l-1))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((9 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a (exp (gen) alpha) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na-0 a b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1562)
  (parent 1463)
  (unrealized (0 3) (5 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (8 0)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((7 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-0) (mul (rec l-0) (rec l-0))
    (6 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0)))
  (label 1563)
  (parent 1464)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1564)
  (parent 1467)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1565)
  (parent 1468)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1566)
  (parent 1469)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1567)
  (parent 1469)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1568)
  (parent 1470)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1569)
  (parent 1470)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1570)
  (parent 1470)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1571)
  (parent 1470)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1572)
  (parent 1470)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1573)
  (parent 1470)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1574)
  (parent 1470)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1575)
  (parent 1470)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 1576)
  (parent 1471)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1577)
  (parent 1471)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1578)
  (parent 1472)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1579)
  (parent 1472)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1580)
  (parent 1472)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1581)
  (parent 1472)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1582)
  (parent 1472)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1583)
  (parent 1472)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1584)
  (parent 1472)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1585)
  (parent 1472)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1586)
  (parent 1473)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1587)
  (parent 1473)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1588)
  (parent 1474)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1589)
  (parent 1474)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1590)
  (parent 1474)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1591)
  (parent 1474)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1592)
  (parent 1474)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1593)
  (parent 1474)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1594)
  (parent 1474)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1595)
  (parent 1474)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1596)
  (parent 1475)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1597)
  (parent 1475)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 1598)
  (parent 1475)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1599)
  (parent 1475)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1600)
  (parent 1475)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 1601)
  (parent 1475)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1602)
  (parent 1475)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1603)
  (parent 1475)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1604)
  (parent 1476)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1605)
  (parent 1476)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1606)
  (parent 1477)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1607)
  (parent 1477)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (3 2)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1608)
  (parent 1477)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1609)
  (parent 1477)
  (unrealized (0 3) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 2) (0 1)) ((2 3) (0 3)) ((3 1) (2 0))
    ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1610)
  (parent 1477)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1611)
  (parent 1477)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0)) ((9 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0)) ((9 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1612)
  (parent 1477)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1613)
  (parent 1477)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1614)
  (parent 1478)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1615)
  (parent 1478)
  (unrealized (0 3) (7 0) (9 0) (10 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 1616)
  (parent 1478)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (9 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1617)
  (parent 1478)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (5 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((9 1) (0 0)) ((9 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1618)
  (parent 1478)
  (unrealized (0 3) (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((1 2) (9 0))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 1619)
  (parent 1478)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 4 10 ltx-gen 3) (exp (gen) l-2)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1620)
  (parent 1478)
  (unrealized (0 3) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1621)
  (parent 1478)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1622)
  (parent 1479)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b) (x x) (upsilon upsilon)
        (na na) (nb nb) (priv-stor priv-stor-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1623)
  (parent 1483)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1624)
  (parent 1487)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1625)
  (parent 1488)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-3 (3 1)) (l (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1626)
  (parent 1492)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1627)
  (parent 1493)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1628)
  (parent 1497)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (5 0)) ((4 1) (0 0)) ((4 2) (3 1)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1629)
  (parent 1498)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (3 0) (6 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1630)
  (parent 1502)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha l-0) (zeta zeta))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y l) (y l-0))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (recv
        (sig (body a (exp (gen) l-0) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1631)
  (parent 1502)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (0 3)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1632)
  (parent 1506)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 2) (2 0)) ((4 3) (7 0)) ((5 1) (4 1)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (2 0) (5 0) (7 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1633)
  (parent 1507)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 4 ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1))))
  (label 1634)
  (parent 1508)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 1635)
  (parent 1509)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1)))
  (label 1636)
  (parent 1510)
  (unrealized (0 3) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1)))
  (label 1637)
  (parent 1511)
  (unrealized (0 3) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 1638)
  (parent 1512)
  (unrealized (0 3) (9 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 4 ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1))))
  (label 1639)
  (parent 1513)
  (unrealized (0 3) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1640)
  (parent 1514)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1641)
  (parent 1515)
  (unrealized (7 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (l l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 1) (6 0))
    ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 0 init 3) (exp (gen) x-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1642)
  (parent 1515)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1643)
  (parent 1515)
  (unrealized (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 1 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0))))))
  (label 1644)
  (parent 1515)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1645)
  (parent 1515)
  (unrealized (7 0) (8 0) (9 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (7 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 9 ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1646)
  (parent 1515)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1647)
  (parent 1515)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1648)
  (parent 1516)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l la rndx) (beta expt) (l-0 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 la) (x-0 beta))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1649)
  (parent 1517)
  (unrealized (7 0) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (3 0) (9 0))
  (gen-st (pv a la) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 0 init 3) (exp (gen) x-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))))
  (label 1650)
  (parent 1517)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-1 l-0) (w l-0)) (gen) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1651)
  (parent 1517)
  (seen 1803)
  (unrealized (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1652)
  (parent 1517)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l lb rndx) (alpha zeta expt) (l-0 y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y lb) (y alpha) (y zeta))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta y))))))))
  (label 1653)
  (parent 1517)
  (unrealized (7 0) (9 0) (10 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul l (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 1) (0 0)) ((9 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (2 0) (8 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((9 1) (0 0)) ((9 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-2 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) (mul l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec l-0)))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-4 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1654)
  (parent 1517)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na l l-0 l-1)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1655)
  (parent 1517)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) lb) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (4 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (5 0) (7 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 8 4 resp 4) (exp (gen) y-0) (7 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) lb) (one)))
      (send (cat (exp (gen) lb) (one))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1656)
  (parent 1519)
  (seen 1468)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1657)
  (parent 1521)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1658)
  (parent 1521)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 1659)
  (parent 1522)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1660)
  (parent 1522)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1661)
  (parent 1523)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1662)
  (parent 1523)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1663)
  (parent 1524)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 1 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0)))
  (label 1664)
  (parent 1525)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1665)
  (parent 1525)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1666)
  (parent 1526)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 1) (9 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((1 1) (9 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1667)
  (parent 1527)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1668)
  (parent 1527)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((10 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1669)
  (parent 1527)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1670)
  (parent 1529)
  (seen 1483)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 1) (9 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((1 1) (9 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1671)
  (parent 1531)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1672)
  (parent 1531)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((10 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1673)
  (parent 1531)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1674)
  (parent 1534)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((2 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1675)
  (parent 1534)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1676)
  (parent 1534)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 gen-st-ltx-disclose-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1677)
  (parent 1536)
  (seen 1492)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1678)
  (parent 1538)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((2 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1679)
  (parent 1538)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1680)
  (parent 1538)
  (unrealized (6 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1681)
  (parent 1541)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (6 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1682)
  (parent 1541)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((1 2) (5 0))
    ((2 1) (4 0)) ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1683)
  (parent 1542)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((1 2) (6 0)) ((2 3) (8 0)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (2 1)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0))
    ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1684)
  (parent 1543)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 1685)
  (parent 1544)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (6 0) (9 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1686)
  (parent 1544)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (0 3)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (2 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 1687)
  (parent 1545)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (2 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1688)
  (parent 1545)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (x rndx) (upsilon expt) (l rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 2) (2 0)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))) ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 1689)
  (parent 1546)
  (seen 1547)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor (cat pt-0 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1690)
  (parent 1551)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1691)
  (parent 1552)
  (seen 1542)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1692)
  (parent 1552)
  (unrealized (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv b l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1693)
  (parent 1559)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((7 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor-0 (cat pt-3 (pv self-0 l-0))))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1694)
  (parent 1563)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1695)
  (parent 1564)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1696)
  (parent 1565)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1697)
  (parent 1566)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1698)
  (parent 1567)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1699)
  (parent 1568)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1700)
  (parent 1568)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1701)
  (parent 1568)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1702)
  (parent 1568)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1703)
  (parent 1569)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1704)
  (parent 1569)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1705)
  (parent 1569)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1706)
  (parent 1569)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 1707)
  (parent 1570)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1708)
  (parent 1572)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (8 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1709)
  (parent 1572)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 1710)
  (parent 1576)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1711)
  (parent 1577)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1712)
  (parent 1578)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1713)
  (parent 1578)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1714)
  (parent 1578)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1715)
  (parent 1578)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1716)
  (parent 1579)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1717)
  (parent 1579)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1718)
  (parent 1579)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1719)
  (parent 1579)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 1720)
  (parent 1580)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1721)
  (parent 1583)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1722)
  (parent 1583)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1723)
  (parent 1586)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (9 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1724)
  (parent 1587)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1725)
  (parent 1588)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1726)
  (parent 1588)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1727)
  (parent 1588)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1728)
  (parent 1588)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1729)
  (parent 1589)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1730)
  (parent 1589)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1731)
  (parent 1589)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1732)
  (parent 1589)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 1733)
  (parent 1590)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (0 3))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1734)
  (parent 1592)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (8 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1735)
  (parent 1592)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1736)
  (parent 1596)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1737)
  (parent 1596)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1738)
  (parent 1596)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1739)
  (parent 1596)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1740)
  (parent 1597)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1741)
  (parent 1597)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1742)
  (parent 1597)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1743)
  (parent 1597)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 1744)
  (parent 1598)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 1745)
  (parent 1602)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1746)
  (parent 1604)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1747)
  (parent 1605)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1748)
  (parent 1606)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1749)
  (parent 1606)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1750)
  (parent 1606)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1751)
  (parent 1606)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1752)
  (parent 1607)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1753)
  (parent 1607)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1754)
  (parent 1607)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1755)
  (parent 1607)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 1756)
  (parent 1608)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1757)
  (parent 1611)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1758)
  (parent 1611)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1759)
  (parent 1614)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1760)
  (parent 1614)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 3) (9 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1761)
  (parent 1614)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1762)
  (parent 1614)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((2 1) (10 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1763)
  (parent 1615)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 1) (10 0)) ((1 2) (0 1))
    ((2 1) (0 0)) ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1764)
  (parent 1615)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 4 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1765)
  (parent 1615)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1766)
  (parent 1615)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (10 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 1767)
  (parent 1616)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (10 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 1768)
  (parent 1620)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1769)
  (parent 1623)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1770)
  (parent 1624)
  (seen 1622)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1771)
  (parent 1626)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1772)
  (parent 1627)
  (seen 1625)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1773)
  (parent 1628)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (4 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1774)
  (parent 1629)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta x))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y x))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1775)
  (parent 1630)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (8 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (3 0) (6 0) (8 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self))
  (leads-to ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 9 5 resp 4) (exp (gen) y-0) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) lb)) (send (cat (gen) lb)))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1776)
  (parent 1631)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 0))
    ((4 2) (3 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1777)
  (parent 1632)
  (seen 1628)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 2) (0 1))
    ((3 3) (6 0)) ((4 1) (3 1)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (4 0) (6 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (recv (cat na self-0 self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1778)
  (parent 1633)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1779)
  (parent 1634)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1780)
  (parent 1634)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 1781)
  (parent 1635)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l l-1)) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y (mul l (rec l-0) l-1)) (y zeta) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) l-0))
      (send (cat (exp (gen) (mul l l-1)) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1782)
  (parent 1635)
  (unrealized (0 3) (11 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 10 ltx-gen 3) (exp (gen) l-1)
    (8 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1783)
  (parent 1636)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1784)
  (parent 1636)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (8 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1785)
  (parent 1637)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-1)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0)))
  (label 1786)
  (parent 1638)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l x y rndx) (zeta expt)
    (l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec l-1))) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec l-1))) (y zeta) (y l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul l (rec w))) w)) (exp (gen) l)
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l (rec l-1))) l-0))
      (send (cat (exp (gen) (mul l (rec l-1))) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul l (rec w))) w))
      (send (cat (exp (gen) (mul l (rec w))) w))))
  (label 1787)
  (parent 1638)
  (unrealized (0 3) (11 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) l-1))
  (deflistener (cat (exp (gen) (mul (rec l) (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec l) (rec w))) w))
    (exp (gen) (rec l)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-1)) l-0))
      (send (cat (exp (gen) (mul (rec l) l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) l-1))
      (send (cat (exp (gen) (rec l)) l-1)))
    ((recv (cat (exp (gen) (mul (rec l) (rec w))) w))
      (send (cat (exp (gen) (mul (rec l) (rec w))) w))))
  (label 1788)
  (parent 1639)
  (unrealized (0 3) (10 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1789)
  (parent 1640)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1790)
  (parent 1641)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1791)
  (parent 1641)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (9 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((10 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv a-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv a-0 l-1)))))
  (label 1792)
  (parent 1641)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-0 (8 0)
    (ch-msg priv-stor (cat pt (pv a l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1793)
  (parent 1643)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (9 0)) ((2 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1794)
  (parent 1645)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 10 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1795)
  (parent 1645)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 3) (8 0)) ((10 1) (9 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((10 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-4 (pv b-0 l-1))) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-4 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b-0 l-1)))))
  (label 1796)
  (parent 1645)
  (unrealized (7 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (5 0))
    ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1797)
  (parent 1646)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1798)
  (parent 1648)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l rndx) (beta expt) (x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (10 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-0)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1799)
  (parent 1649)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 data) (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon beta expt) (l x-0 l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-0) (x x-0) (beta beta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na na-0 l l-0)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1800)
  (parent 1649)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self a-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 2) (9 0))
    ((11 1) (10 0)))
  (uniq-orig na na-0 l l-0 l-1)
  (uniq-gen x x-0)
  (absent (x l) (x l-0) (x-0 beta) (x-0 l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv a-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((11 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv a-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-5 (pv a-0 l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv a-0 l-1)))))
  (label 1801)
  (parent 1649)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (upsilon expt) (la l x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
  (deflistener (cat (exp (gen) l) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l (rec x))))
  (deflistener x)
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na la l)
  (uniq-gen x)
  (absent (x la) (x l))
  (precur (3 0) (9 0))
  (gen-st (pv a la) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul l (rec x)) (9 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul la l))
              (exp (gen) (mul upsilon x)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (hash (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x))))
      (send (cat (exp (gen) (mul la l)) (exp (gen) (mul upsilon x)))))
    ((recv (cat (exp (gen) l) la)) (send (cat (exp (gen) l) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-4 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l (rec x))))
      (send (cat (exp (gen) x) (mul l (rec x))))) ((recv x) (send x)))
  (label 1802)
  (parent 1650)
  (unrealized (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 4 ltx-disclose 3) l-0 (9 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-0))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1803)
  (parent 1651)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1804)
  (parent 1652)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon expt) (l rndx)
    (alpha zeta expt) (y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul (rec y) l-0)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 1) (10 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 1 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul (rec y) l-0)))
      (send (cat (exp (gen) y) (mul (rec y) l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1805)
  (parent 1653)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval)
    (priv-stor priv-stor-0 locn) (x rndx) (upsilon alpha zeta expt)
    (l y l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) (mul l (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-0) (y y) (alpha alpha) (zeta zeta))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 3) (9 0)))
  (uniq-orig na nb-0 l l-0)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-0))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 11 2 ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) (mul l (rec y))))
      (send (cat (exp (gen) y) (mul l (rec y)))))
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-0))
              (exp (gen) (mul zeta y))))))))
  (label 1806)
  (parent 1653)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x rndx) (upsilon expt)
    (l rndx) (alpha zeta expt) (l-0 y l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) (mul l-0 (rec y))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha alpha) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 3) (9 0))
    ((11 1) (10 0)))
  (uniq-orig na nb-0 l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y alpha) (y zeta) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv b-0 l-1))
  (facts (neq (exp (gen) upsilon) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((11 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-1 (cat pt-5 (pv b-0 l-1))) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) (mul l-0 (rec y))))
      (send (cat (exp (gen) y) (mul l-0 (rec y)))))
    ((load priv-stor-1 (cat pt-5 (pv b-0 l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta y)))))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv b-0 l-1)))))
  (label 1807)
  (parent 1653)
  (unrealized (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1808)
  (parent 1657)
  (seen 1566)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1809)
  (parent 1657)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1810)
  (parent 1658)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1811)
  (parent 1658)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1812)
  (parent 1658)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1813)
  (parent 1658)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1814)
  (parent 1658)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1815)
  (parent 1658)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((10 1) (4 0)) ((10 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1816)
  (parent 1658)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1817)
  (parent 1658)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 1818)
  (parent 1659)
  (seen 1576)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1819)
  (parent 1659)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1820)
  (parent 1660)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1821)
  (parent 1660)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (10 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1822)
  (parent 1660)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1823)
  (parent 1660)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((4 1) (3 0)) ((4 1) (6 0)) ((10 1) (0 0)) ((10 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1824)
  (parent 1660)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1825)
  (parent 1660)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (4 0)) ((10 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1826)
  (parent 1660)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1827)
  (parent 1660)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1828)
  (parent 1661)
  (seen 1586)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1829)
  (parent 1661)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1830)
  (parent 1662)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1831)
  (parent 1662)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1832)
  (parent 1662)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1833)
  (parent 1662)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1834)
  (parent 1662)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1835)
  (parent 1662)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((10 1) (4 0)) ((10 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1836)
  (parent 1662)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1837)
  (parent 1662)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1838)
  (parent 1663)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1839)
  (parent 1663)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 1840)
  (parent 1663)
  (unrealized (0 3) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1841)
  (parent 1663)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((10 1) (0 0)) ((10 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1842)
  (parent 1663)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 1843)
  (parent 1663)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1844)
  (parent 1663)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1845)
  (parent 1663)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1846)
  (parent 1664)
  (seen 1604)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1847)
  (parent 1664)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1848)
  (parent 1665)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1849)
  (parent 1665)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (4 2)) ((1 1) (10 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1850)
  (parent 1665)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1851)
  (parent 1665)
  (unrealized (0 3) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 2) (2 0)) ((2 1) (0 1)) ((3 3) (0 3))
    ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((4 1) (3 0)) ((4 1) (8 0)) ((10 1) (0 0)) ((10 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1852)
  (parent 1665)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1853)
  (parent 1665)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (4 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((10 1) (4 0)) ((10 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1854)
  (parent 1665)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1855)
  (parent 1665)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1856)
  (parent 1666)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1857)
  (parent 1666)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 1858)
  (parent 1666)
  (unrealized (0 3) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-5 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1859)
  (parent 1666)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((5 1) (3 1)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (6 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((10 1) (0 0)) ((10 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-3 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1860)
  (parent 1666)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((1 2) (10 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 1861)
  (parent 1666)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((10 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 5 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1862)
  (parent 1666)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1863)
  (parent 1666)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1864)
  (parent 1681)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((3 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1865)
  (parent 1682)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((1 2) (4 0))
    ((2 1) (3 0)) ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1866)
  (parent 1683)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((1 2) (5 0)) ((2 1) (4 0)) ((3 1) (7 0)) ((4 2) (3 0))
    ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1867)
  (parent 1684)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 1868)
  (parent 1685)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 0)) ((3 1) (8 0)) ((4 1) (2 1))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (6 0) (9 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1869)
  (parent 1686)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 2) (2 0)) ((4 1) (2 0)) ((5 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 1870)
  (parent 1687)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 3) (9 0)) ((3 1) (2 1)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (2 0)) ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (5 0) (9 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((6 1) (2 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 2 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1871)
  (parent 1688)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1872)
  (parent 1690)
  (seen 1681)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-0))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) (mul l-0 l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l-0) (y zeta) (y (mul (rec l) l-0 l-0)) (y l))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self (exp (gen) (mul (rec l) l-0 l-0))
            (pubk "sig" self)) (privk "sig" self)))
      (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l-0 l-0))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-0)) l))
      (send (cat (exp (gen) (mul l-0 l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (mul l-0 l-0))) (send (cat (gen) (mul l-0 l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1873)
  (parent 1690)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (8 0)) ((1 2) (2 0))
    ((1 2) (7 0)) ((2 1) (0 1)) ((3 3) (9 0)) ((4 1) (3 0))
    ((4 1) (6 0)) ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (7 0) (9 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (9 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1874)
  (parent 1692)
  (seen 1684)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 1875)
  (parent 1693)
  (seen 1685)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec l-0)))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec l-0))) (send (cat (gen) (rec l-0))))
    ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1876)
  (parent 1693)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y x))
  (precur (2 0) (6 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((7 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self))) (recv (cat na self self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 1877)
  (parent 1694)
  (seen 1687)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x y rndx) (zeta expt) (l l-0 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l l (rec l-0)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener l-0)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x y)
  (absent (x l) (y zeta) (y (mul l l (rec l-0))) (y l-0))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((7 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec l-0))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec l-0)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-0) (send l-0))
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1878)
  (parent 1694)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization separated self-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1879)
  (parent 1695)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a a) (b self-0) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore))))
  (origs (pt-3 (3 1)) (lb (1 1)) (pt-1 (1 1)) (l (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 1880)
  (parent 1696)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1881)
  (parent 1697)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1882)
  (parent 1698)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 1883)
  (parent 1707)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1884)
  (parent 1707)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1885)
  (parent 1708)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (7 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1886)
  (parent 1709)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 1887)
  (parent 1710)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1888)
  (parent 1711)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 1889)
  (parent 1720)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1890)
  (parent 1720)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1891)
  (parent 1721)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1892)
  (parent 1722)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1893)
  (parent 1723)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1894)
  (parent 1724)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 1895)
  (parent 1733)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1896)
  (parent 1733)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1897)
  (parent 1734)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (2 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 3) (10 0))
    ((3 1) (2 0)) ((3 1) (7 0)) ((4 1) (2 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((3 1) (2 0)) ((3 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 2 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1898)
  (parent 1735)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 1899)
  (parent 1744)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 1900)
  (parent 1745)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 1901)
  (parent 1746)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1902)
  (parent 1747)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 1903)
  (parent 1756)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1904)
  (parent 1756)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1905)
  (parent 1757)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 2) (0 1)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1906)
  (parent 1758)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 1907)
  (parent 1767)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 1908)
  (parent 1768)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1909)
  (parent 1769)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 1910)
  (parent 1771)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 self-1 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self-1)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 self-1))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization separated self-1)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self-1 (exp (gen) lb) (pubk "sig" self-1))
          (privk "sig" self-1)))
      (send (cat na self-0 self-1 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 self-1
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1911)
  (parent 1773)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a self-0) (b self-1) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore))))
  (origs (l (3 1)) (pt-3 (3 1)) (lb (1 1)) (pt-0 (1 1)) (pt-1 (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1912)
  (parent 1774)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (4 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1913)
  (parent 1775)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha l) (zeta (mul x w)))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 3) (7 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x y)
  (absent (x lb) (x l) (y lb) (y l) (y (mul x w)))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (recv (sig (body a (exp (gen) l) (pubk "sig" a)) (privk "sig" a)))
      (recv (cat na a self (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1914)
  (parent 1776)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (3 0) (5 0))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1915)
  (parent 1778)
  (seen 1774)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1916)
  (parent 1779)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1917)
  (parent 1779)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1918)
  (parent 1780)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1919)
  (parent 1780)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1920)
  (parent 1780)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (8 0))
    ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1921)
  (parent 1780)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1922)
  (parent 1780)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1923)
  (parent 1780)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 1) (5 0))
    ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((10 1) (5 0)) ((10 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1924)
  (parent 1780)
  (unrealized (0 3) (8 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1925)
  (parent 1780)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 1926)
  (parent 1781)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1927)
  (parent 1781)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1928)
  (parent 1782)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1929)
  (parent 1782)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1930)
  (parent 1782)
  (seen 2118)
  (unrealized (0 3) (11 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (11 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1931)
  (parent 1782)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (4 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (0 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((11 1) (0 0))
    ((11 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1932)
  (parent 1782)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1933)
  (parent 1782)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (5 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((11 1) (5 0))
    ((11 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1934)
  (parent 1782)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1935)
  (parent 1782)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 10 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1936)
  (parent 1783)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (10 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1937)
  (parent 1783)
  (unrealized (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1938)
  (parent 1784)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1939)
  (parent 1784)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1940)
  (parent 1784)
  (unrealized (0 3) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l l-0 l-1 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0)) ((6 1) (9 0))
    ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) l-0 l-1)) (y l) (x l-0) (x l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1941)
  (parent 1784)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1942)
  (parent 1784)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1))))))
  (label 1943)
  (parent 1784)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (5 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((10 1) (5 0)) ((10 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1944)
  (parent 1784)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1945)
  (parent 1784)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1946)
  (parent 1785)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1947)
  (parent 1785)
  (unrealized (0 3) (8 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 1948)
  (parent 1785)
  (unrealized (0 3) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) la)) l-0))
  (deflistener (cat (exp (gen) (rec l)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) la)) l-0))
      (send (cat (exp (gen) (mul (rec l) la)) l-0)))
    ((recv (cat (exp (gen) (rec l)) la))
      (send (cat (exp (gen) (rec l)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1949)
  (parent 1785)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (0 0))
    ((10 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((10 1) (0 0)) ((10 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1950)
  (parent 1785)
  (unrealized (0 3) (7 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 1951)
  (parent 1785)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1952)
  (parent 1785)
  (unrealized (0 3) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1953)
  (parent 1785)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 1954)
  (parent 1786)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1955)
  (parent 1786)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1956)
  (parent 1787)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 la) (x-0 beta))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1957)
  (parent 1787)
  (unrealized (0 3) (9 0) (11 0) (12 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w l-1)) (gen) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 1958)
  (parent 1787)
  (seen 2155)
  (unrealized (0 3) (11 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (precedes ((0 2) (11 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 0 init 3) (exp (gen) x-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))))
  (label 1959)
  (parent 1787)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0)) ((6 1) (4 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (0 0)) ((11 1) (7 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (10 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (9 0)) ((11 1) (0 0))
    ((11 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1960)
  (parent 1787)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 1 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one)))))
  (label 1961)
  (parent 1787)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (8 0))
    ((11 1) (5 0)) ((11 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((11 1) (5 0))
    ((11 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 12 ltx-gen 3) (exp (gen) l-2)
    (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1)))))
    ((load priv-stor-1 (cat pt-6 ignore-1))
      (stor priv-stor-1 (cat pt-7 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1962)
  (parent 1787)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2)))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-9 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1963)
  (parent 1787)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 lb rndx) (alpha zeta-0 expt) (l-1 y-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb lb) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 lb) (y-0 alpha) (y-0 zeta-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 lb))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 lb)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul lb alpha))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1964)
  (parent 1788)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 la rndx) (beta expt) (l-1 x-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la la) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 la) (x-0 beta))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 la))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 la)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 a-0 b-0 (exp (gen) x-0)))))
  (label 1965)
  (parent 1788)
  (unrealized (0 3) (9 0) (10 0) (11 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (contracted (l-2 l-1) (w (rec l-1))) (gen)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1)))))
  (label 1966)
  (parent 1788)
  (unrealized (0 3) (9 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) (rec l)) l-0))
  (deflistener (cat (exp (gen) x) (mul (rec l) (rec x))))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 0 init 3) (exp (gen) x-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) (rec l)) l-0))
      (send (cat (exp (gen) (rec l)) l-0)))
    ((recv (cat (exp (gen) x) (mul (rec l) (rec x))))
      (send (cat (exp (gen) x) (mul (rec l) (rec x))))))
  (label 1967)
  (parent 1788)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((6 1) (4 1))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 1) (0 0))
    ((10 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((10 1) (0 0)) ((10 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 2 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-4 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1968)
  (parent 1788)
  (unrealized (0 3) (8 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 1 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-0) (rec l-1))))))
  (label 1969)
  (parent 1788)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 6 11 ltx-gen 3) (exp (gen) l-2)
    (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1970)
  (parent 1788)
  (unrealized (0 3) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb data)
    (a b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 l-1 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
  (defstrand ltx-gen 3 (ignore ignore-2) (self self-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1 l-2)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-gen 3) (exp (gen) l-2) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec l-1) (rec l-2)))))
    ((load priv-stor-2 (cat pt-7 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv self-0 l-2)))
      (send
        (sig (body self-0 (exp (gen) l-2) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1971)
  (parent 1788)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1972)
  (parent 1789)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 1973)
  (parent 1793)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (3 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (0 0))
    ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1974)
  (parent 1797)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 1975)
  (parent 1798)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 1976)
  (parent 1803)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 1977)
  (parent 1804)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1978)
  (parent 1809)
  (seen 1698)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1979)
  (parent 1810)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1980)
  (parent 1810)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1981)
  (parent 1810)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1982)
  (parent 1810)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1983)
  (parent 1811)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1984)
  (parent 1811)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1985)
  (parent 1811)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1986)
  (parent 1811)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 1987)
  (parent 1812)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 1988)
  (parent 1814)
  (seen 1708)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv b l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 1989)
  (parent 1814)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 1990)
  (parent 1819)
  (seen 1711)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1991)
  (parent 1820)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1992)
  (parent 1820)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 1993)
  (parent 1820)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 1994)
  (parent 1820)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 1995)
  (parent 1821)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 1996)
  (parent 1821)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 1997)
  (parent 1821)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 1998)
  (parent 1821)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 1999)
  (parent 1822)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2000)
  (parent 1825)
  (seen 1721)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l-0) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2001)
  (parent 1825)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2002)
  (parent 1829)
  (seen 1724)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2003)
  (parent 1830)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2004)
  (parent 1830)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2005)
  (parent 1830)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2006)
  (parent 1830)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2007)
  (parent 1831)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2008)
  (parent 1831)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2009)
  (parent 1831)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2010)
  (parent 1831)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2011)
  (parent 1832)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2012)
  (parent 1834)
  (seen 1734)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2013)
  (parent 1834)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((2 1) (11 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2014)
  (parent 1838)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2015)
  (parent 1838)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2016)
  (parent 1838)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2017)
  (parent 1838)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((2 1) (11 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2018)
  (parent 1839)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2019)
  (parent 1839)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2020)
  (parent 1839)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2021)
  (parent 1839)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2022)
  (parent 1840)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 2023)
  (parent 1844)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2024)
  (parent 1847)
  (seen 1747)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2025)
  (parent 1848)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2026)
  (parent 1848)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2027)
  (parent 1848)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2028)
  (parent 1848)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2029)
  (parent 1849)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2030)
  (parent 1849)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2031)
  (parent 1849)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (9 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2032)
  (parent 1849)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 gen-st-ltx-disclose-0
    trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0
    trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1)))
  (label 2033)
  (parent 1850)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2034)
  (parent 1853)
  (seen 1757)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2035)
  (parent 1853)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((2 1) (11 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2036)
  (parent 1856)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2037)
  (parent 1856)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2038)
  (parent 1856)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2039)
  (parent 1856)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((2 1) (11 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-5 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2040)
  (parent 1857)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 1) (11 0)) ((1 2) (3 0))
    ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3))
    ((5 1) (4 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2041)
  (parent 1857)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 5 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2042)
  (parent 1857)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2043)
  (parent 1857)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2044)
  (parent 1858)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((10 1) (4 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 2045)
  (parent 1862)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2046)
  (parent 1864)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (7 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (5 0) (8 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (7 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2047)
  (parent 1865)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((1 2) (3 0))
    ((2 1) (0 3)) ((3 1) (0 3)) ((4 2) (3 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2048)
  (parent 1866)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((1 2) (4 0)) ((2 1) (3 0)) ((3 2) (6 0)) ((4 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0) (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2049)
  (parent 1867)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 2050)
  (parent 1868)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 1) (7 0)) ((3 1) (8 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (5 0) (8 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2051)
  (parent 1869)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3))
    ((3 1) (0 3)) ((4 1) (6 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 2052)
  (parent 1870)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (8 0)) ((3 2) (2 0)) ((4 1) (2 0)) ((5 1) (7 0))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0) (8 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2053)
  (parent 1871)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (9 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (9 0)) ((4 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l)) lb))
      (send (cat (exp (gen) (mul l l)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2054)
  (parent 1873)
  (seen 1865)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (9 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (7 0) (10 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2055)
  (parent 1876)
  (seen 1869)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (rec lb)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (2 0))
    ((2 1) (0 1)) ((3 3) (10 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (3 0)) ((7 1) (9 0)) ((7 2) (6 0))
    ((8 1) (6 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x y)
  (absent (x l) (y (mul l l (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (6 0) (10 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)) ((7 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 3 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (recv
        (sig
          (body self (exp (gen) (mul l l (rec lb))) (pubk "sig" self))
          (privk "sig" self)))
      (recv (cat na self self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (rec lb)) l))
      (send (cat (exp (gen) (rec lb)) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2056)
  (parent 1878)
  (seen 1871)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2057)
  (parent 1880)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2058)
  (parent 1881)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2059)
  (parent 1882)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2060)
  (parent 1883)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2061)
  (parent 1884)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2062)
  (parent 1885)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((2 1) (4 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2063)
  (parent 1886)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 2064)
  (parent 1887)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2065)
  (parent 1888)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 2066)
  (parent 1889)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (6 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2067)
  (parent 1890)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2068)
  (parent 1891)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2069)
  (parent 1892)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2070)
  (parent 1893)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (7 0)) ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0))
    ((6 2) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2071)
  (parent 1894)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2072)
  (parent 1895)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2073)
  (parent 1896)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2074)
  (parent 1897)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((2 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2075)
  (parent 1898)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2076)
  (parent 1899)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2077)
  (parent 1899)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2078)
  (parent 1900)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2079)
  (parent 1900)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2080)
  (parent 1901)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2081)
  (parent 1902)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 2082)
  (parent 1903)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (10 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (10 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2083)
  (parent 1904)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2084)
  (parent 1905)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (9 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2085)
  (parent 1906)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2086)
  (parent 1907)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2087)
  (parent 1907)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (0 3)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2088)
  (parent 1908)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2089)
  (parent 1908)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2090)
  (parent 1909)
  (seen 2199)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2091)
  (parent 1910)
  (seen 2199)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b self)
    (priv-stor priv-stor-0) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv self lb) (pv self-0 l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 self))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na self-0 self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self lb)))
      (stor priv-stor (cat pt-1 "nil")) (send lb))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l)))
      (send
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2092)
  (parent 1912)
  (seen 2169)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2093)
  (parent 1913)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2094)
  (parent 1914)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2095)
  (parent 1916)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2096)
  (parent 1917)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2097)
  (parent 1918)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2098)
  (parent 1918)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2099)
  (parent 1918)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2100)
  (parent 1918)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2101)
  (parent 1919)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2102)
  (parent 1919)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2103)
  (parent 1919)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (8 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2104)
  (parent 1919)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2105)
  (parent 1920)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2106)
  (parent 1922)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l) (y y) (alpha (mul (rec l) l-0 l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt-0 (pv self l-0)))
      (stor priv-stor (cat pt-1 "nil")) (send l-0))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul (rec l) l-0 l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-0 (cat pt-2 (pv b l)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2107)
  (parent 1922)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 2108)
  (parent 1926)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2109)
  (parent 1927)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((2 1) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2110)
  (parent 1928)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2111)
  (parent 1928)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2112)
  (parent 1928)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)) ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 2113)
  (parent 1928)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((2 1) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2114)
  (parent 1929)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((6 1) (5 0)) ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2115)
  (parent 1929)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l l-0)) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)) ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l l-0)) l-1))
      (send (cat (exp (gen) (mul l l-0)) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2116)
  (parent 1929)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)) ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 2117)
  (parent 1929)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2118)
  (parent 1930)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul la l-0)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-0) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (deflistener x)
  (precedes ((0 2) (12 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (11 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul l-0 (rec x)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul la l-0)) l))
      (send (cat (exp (gen) (mul la l-0)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) la)) (send (cat (exp (gen) l-0) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-7 "nil")) (send la))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))) ((recv x) (send x)))
  (label 2119)
  (parent 1931)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2120)
  (parent 1933)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2121)
  (parent 1933)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2122)
  (parent 1936)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (10 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 0))
    ((9 2) (7 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (10 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2123)
  (parent 1937)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2124)
  (parent 1938)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2125)
  (parent 1938)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2126)
  (parent 1938)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2127)
  (parent 1938)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2128)
  (parent 1939)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2129)
  (parent 1939)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2130)
  (parent 1939)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((3 1) (0 1))
    ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((6 1) (5 0)) ((6 1) (9 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2131)
  (parent 1939)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (10 0)
    (ch-msg priv-stor (cat pt (pv a l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1)))
  (label 2132)
  (parent 1940)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (2 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 11 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2133)
  (parent 1942)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-1) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb l-0) (y y) (alpha (mul l (rec l-0) l-1))
    (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l (rec l-0) l-1))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-2 (pv b l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2134)
  (parent 1942)
  (unrealized (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((2 1) (11 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2135)
  (parent 1946)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2136)
  (parent 1946)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2137)
  (parent 1946)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2138)
  (parent 1946)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((2 1) (11 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2139)
  (parent 1947)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2140)
  (parent 1947)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2141)
  (parent 1947)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2142)
  (parent 1947)
  (unrealized (0 3) (8 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2143)
  (parent 1948)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 2144)
  (parent 1952)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2145)
  (parent 1954)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (11 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2146)
  (parent 1955)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2147)
  (parent 1956)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((6 1) (5 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) l-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2148)
  (parent 1956)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)) ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) y-0) (mul l-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-0 (rec y-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2149)
  (parent 1956)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 3) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)) ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv b-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y-0) (mul l-1 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul l-1 (rec y-0)))))
    ((load priv-stor-2 (cat pt-8 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv b-0 l-2)))))
  (label 2150)
  (parent 1956)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (12 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) l-1))
      (send (cat (exp (gen) (mul (rec l) l-0)) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2151)
  (parent 1957)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (12 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((6 1) (5 0)) ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) l-1)))
      (send (cat (exp (gen) x-0) (mul (rec x-0) l-1))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2152)
  (parent 1957)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)) ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 13 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) x-0) (mul l-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-0 (rec x-0)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2153)
  (parent 1957)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pt-9 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 2) (11 0)) ((13 1) (12 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)) ((13 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-8 (pv a-0 l-2))) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) x-0) (mul l-1 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul l-1 (rec x-0)))))
    ((load priv-stor-2 (cat pt-8 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-9 ignore-2))
      (stor priv-stor-2 (cat pt-8 (pv a-0 l-2)))))
  (label 2154)
  (parent 1957)
  (unrealized (0 3) (9 0) (11 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-0 (cat pt-1 (pv self l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1))))
  (label 2155)
  (parent 1958)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (y rndx) (zeta expt)
    (l la l-0 x rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la la) (x x) (beta l-0) (upsilon (mul y zeta (rec x))))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l la))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) la l-0)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul (rec l) l-0)) la))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l la))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) x) (mul l-0 (rec x))))
  (deflistener x)
  (precedes ((0 2) (12 0)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3)) ((6 1) (5 0))
    ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (11 0)))
  (uniq-orig na nb l la l-0)
  (uniq-gen y x)
  (absent (y zeta) (y (mul (rec l) la l-0)) (y l) (x la) (x l-0))
  (precur (3 0) (11 0))
  (gen-st (pv a la) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul y zeta (rec x))) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener x) (mul l-0 (rec x)) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a la)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul y zeta (rec x)))
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a la))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) la l-0)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul la l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul (rec l) l-0)) la))
      (send (cat (exp (gen) (mul (rec l) l-0)) la)))
    ((load priv-stor (cat pt (pv a la)))
      (stor priv-stor (cat pt-6 "nil")) (send la))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) x) (mul l-0 (rec x))))
      (send (cat (exp (gen) x) (mul l-0 (rec x))))) ((recv x) (send x)))
  (label 2156)
  (parent 1959)
  (unrealized (12 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2157)
  (parent 1961)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2158)
  (parent 1961)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b a)
    (priv-stor priv-stor) (lb l-1) (y y-0) (alpha alpha) (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (11 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 a (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 a
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2159)
  (parent 1964)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (alpha zeta-0 expt) (l-0 y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b self)
    (priv-stor priv-stor-0) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-0) (rec y-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 self (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 self
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2160)
  (parent 1964)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (alpha zeta-0 expt) (y-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b)
    (priv-stor priv-stor-1) (lb l-1) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 3) (10 0)))
  (uniq-orig na nb nb-0 l l-0 l-1)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1))))
      (send (cat (exp (gen) y-0) (mul (rec y-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0))) (recv (cat na-0 a-0 b (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b
            (hash (exp (gen) (mul alpha l-1))
              (exp (gen) (mul zeta-0 y-0))))))))
  (label 2161)
  (parent 1964)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 nb-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (alpha zeta-0 expt) (l-1 y-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
  (defstrand resp 4 (na na-0) (nb nb-0) (a a-0) (b b-0)
    (priv-stor priv-stor-2) (lb l-2) (y y-0) (alpha alpha)
    (zeta zeta-0))
  (defstrand ltx-gen 2 (ignore ignore-2) (self b-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 3) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb nb-0 l l-0 l-1 l-2)
  (uniq-gen x y y-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (y-0 alpha) (y-0 zeta-0) (y-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv b-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta-0) (gen)) (neq (exp (gen) zeta) (gen))
    (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv b-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec l-1) (rec y-0)))))
    ((load priv-stor-2 (cat pt-7 (pv b-0 l-2)))
      (recv
        (sig (body a-0 (exp (gen) alpha) (pubk "sig" a-0))
          (privk "sig" a-0)))
      (recv (cat na-0 a-0 b-0 (exp (gen) zeta-0)))
      (send
        (cat (exp (gen) y-0)
          (enc na-0 nb-0 a-0 b-0
            (hash (exp (gen) (mul alpha l-2))
              (exp (gen) (mul zeta-0 y-0)))))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv b-0 l-2)))))
  (label 2162)
  (parent 1964)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul l (rec l-0))) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (rec l-0)) l))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a a) (b b-0) (priv-stor priv-stor)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((2 1) (11 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 2 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul l (rec l-0))) l-1))
      (send (cat (exp (gen) (mul l (rec l-0))) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-6 "nil")) (send l-1))
    ((recv (cat (exp (gen) (rec l-0)) l))
      (send (cat (exp (gen) (rec l-0)) l)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a b-0 (exp (gen) x-0)))))
  (label 2163)
  (parent 1965)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l rndx) (beta expt) (l-0 x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-0)) l-1))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
  (defstrand init 3 (na na-0) (a self) (b b-0) (priv-stor priv-stor-0)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0))
    ((5 3) (0 3)) ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 1 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-0)) l-1))
      (send (cat (exp (gen) (rec l-0)) l-1)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-0) (rec x-0)))))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0)))
      (send (cat na-0 self b-0 (exp (gen) x-0)))))
  (label 2164)
  (parent 1965)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb na-0 data)
    (a b self b-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 rndx) (beta expt) (x-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
  (defstrand init 3 (na na-0) (a b) (b b-0) (priv-stor priv-stor-1)
    (la l-1) (x x-0) (beta beta))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb na-0 l l-0 l-1)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (displaced 12 6 ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1))))
      (send (cat (exp (gen) x-0) (mul (rec x-0) (rec l-1)))))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 b b-0 (exp (gen) x-0)))))
  (label 2165)
  (parent 1965)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 ignore-2 mesg) (na nb na-0 data)
    (a b self a-0 b-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pt-8 pval)
    (priv-stor priv-stor-0 priv-stor-1 priv-stor-2 locn) (x y rndx)
    (zeta expt) (l l-0 rndx) (beta expt) (l-1 x-0 l-2 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
  (defstrand init 3 (na na-0) (a a-0) (b b-0) (priv-stor priv-stor-2)
    (la l-2) (x x-0) (beta beta))
  (defstrand ltx-gen 2 (ignore ignore-2) (self a-0)
    (priv-stor priv-stor-2) (l l-2))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((7 1) (5 1)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (11 0)))
  (uniq-orig na nb na-0 l l-0 l-1 l-2)
  (uniq-gen x y x-0)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1)
    (x-0 beta) (x-0 l-2))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0) (pv a-0 l-2))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((12 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation channel-test (added-strand ltx-gen 2)
    (ch-msg priv-stor-2 (cat pt-7 (pv a-0 l-2))) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec l-1) (rec x-0)))))
    ((load priv-stor-2 (cat pt-7 (pv a-0 l-2)))
      (recv
        (sig (body b-0 (exp (gen) beta) (pubk "sig" b-0))
          (privk "sig" b-0))) (send (cat na-0 a-0 b-0 (exp (gen) x-0))))
    ((load priv-stor-2 (cat pt-8 ignore-2))
      (stor priv-stor-2 (cat pt-7 (pv a-0 l-2)))))
  (label 2166)
  (parent 1965)
  (unrealized (0 3) (9 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (11 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (rec l-1) (10 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1)))
  (label 2167)
  (parent 1966)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (11 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (5 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-listener l-1) (mul (rec l-1) (rec l-1))
    (9 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1)))
  (label 2168)
  (parent 1970)
  (unrealized (0 3) (11 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2169)
  (parent 1972)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b) (x x) (upsilon upsilon)
        (na na) (nb nb) (priv-stor priv-stor-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (pt-0 (1 1)) (na (0 2))
    (l (1 1))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2170)
  (parent 1973)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2171)
  (parent 1974)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2172)
  (parent 1975)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener
    (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (0 3)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul x upsilon)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2173)
  (parent 1976)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2174)
  (parent 1977)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2175)
  (parent 1987)
  (seen 1883)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2176)
  (parent 1987)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (8 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2177)
  (parent 1989)
  (seen 1886)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 2178)
  (parent 1999)
  (seen 1889)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-3 (pv b l)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2179)
  (parent 1999)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2180)
  (parent 2001)
  (seen 1892)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2181)
  (parent 2011)
  (seen 1895)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2182)
  (parent 2011)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (2 0)) ((2 1) (0 1))
    ((3 3) (11 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (6 0)) ((4 1) (3 0)) ((4 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 3 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2183)
  (parent 2013)
  (seen 1898)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2184)
  (parent 2022)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2185)
  (parent 2023)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 2186)
  (parent 2033)
  (seen 1903)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-3 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2187)
  (parent 2033)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 2) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (11 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0)) ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2188)
  (parent 2035)
  (seen 1906)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-3 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2189)
  (parent 2044)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2190)
  (parent 2045)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (0 3)) ((4 1) (3 0)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2191)
  (parent 2046)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (6 0)) ((1 2) (0 1))
    ((2 1) (3 0)) ((3 2) (7 0)) ((4 1) (7 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (6 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2192)
  (parent 2047)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((1 2) (2 0))
    ((2 1) (0 3)) ((3 2) (2 0)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2193)
  (parent 2048)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((1 2) (3 0)) ((2 1) (5 0)) ((3 1) (5 0)) ((4 2) (3 0))
    ((5 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0) (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2194)
  (parent 2049)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 2195)
  (parent 2050)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (6 0)) ((3 2) (7 0)) ((4 1) (7 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0) (7 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2196)
  (parent 2051)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3))
    ((3 1) (5 0)) ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 2197)
  (parent 2052)
  (seen 2195)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 2) (7 0)) ((3 1) (7 0)) ((4 1) (6 0)) ((4 2) (3 0))
    ((5 1) (3 0)) ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0) (7 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2198)
  (parent 2053)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization separated self-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2199)
  (parent 2057)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a a) (b self-0) (x x) (upsilon (mul w y))
        (na na) (nb nb) (priv-stor priv-stor) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore))))
  (origs (pt-3 (3 1)) (lb (1 1)) (pt-1 (1 1)) (l (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (0 0)) ((4 2) (2 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2200)
  (parent 2058)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (0 0)) ((5 2) (3 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2201)
  (parent 2059)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2202)
  (parent 2060)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2203)
  (parent 2061)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2204)
  (parent 2062)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (5 0)) ((2 1) (3 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2205)
  (parent 2063)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((1 2) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2206)
  (parent 2064)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2207)
  (parent 2065)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 2208)
  (parent 2066)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (5 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2209)
  (parent 2067)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2210)
  (parent 2068)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2211)
  (parent 2069)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 0)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2212)
  (parent 2070)
  (seen 2286)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (6 0)) ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2213)
  (parent 2071)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2214)
  (parent 2072)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2215)
  (parent 2073)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0)) ((3 2) (0 3))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2216)
  (parent 2074)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2217)
  (parent 2075)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2218)
  (parent 2076)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2219)
  (parent 2077)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2220)
  (parent 2078)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2221)
  (parent 2079)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l)))
  (label 2222)
  (parent 2080)
  (seen 2292)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2223)
  (parent 2081)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 2224)
  (parent 2082)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (9 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2225)
  (parent 2083)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2226)
  (parent 2084)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2227)
  (parent 2085)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (10 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2228)
  (parent 2086)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 3) (12 0)) ((4 1) (3 0)) ((4 1) (11 0))
    ((5 1) (3 1)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (3 0))
    ((4 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2229)
  (parent 2087)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2230)
  (parent 2088)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 3) (12 0)) ((4 1) (3 1)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (3 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((9 1) (3 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 3 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2231)
  (parent 2089)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2232)
  (parent 2093)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (precur (5 0))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2233)
  (parent 2094)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2234)
  (parent 2095)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2235)
  (parent 2096)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2236)
  (parent 2105)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2237)
  (parent 2105)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2238)
  (parent 2106)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (7 0))
    ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2239)
  (parent 2107)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 2240)
  (parent 2108)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2241)
  (parent 2109)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2242)
  (parent 2118)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l) (y y) (alpha (mul (rec l) l-0 l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l-0) (x l-1) (y zeta) (y (mul (rec l) l-0 l-1)) (y l))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l-0) (pv b l) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (recv
        (sig (body a (exp (gen) (mul (rec l) l-0 l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l-0 l-1))
              (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) l))
      (send (cat (exp (gen) (mul l-0 l-1)) l)))
    ((load priv-stor-1 (cat pt-4 (pv b l)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2243)
  (parent 2118)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2244)
  (parent 2120)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (10 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2245)
  (parent 2121)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (6 0))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (0 0))
    ((7 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2246)
  (parent 2122)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (9 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 0)) ((8 2) (6 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((8 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2247)
  (parent 2123)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2248)
  (parent 2132)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-1) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-1))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-1) (pv b l-0) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-1)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-1))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor (cat pt (pv a l-1)))
      (stor priv-stor (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2249)
  (parent 2132)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (0 3)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2250)
  (parent 2133)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 2) (2 0))
    ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0)) ((5 1) (8 0))
    ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (2 0) (9 0) (11 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (4 0)) ((5 1) (8 0)) ((10 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 4 resp 4) (exp (gen) y-0) (11 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2251)
  (parent 2134)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2252)
  (parent 2143)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2253)
  (parent 2144)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2254)
  (parent 2145)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (10 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2255)
  (parent 2146)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 12 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2256)
  (parent 2155)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-1) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-1))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-1))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-0) (y y) (alpha (mul l (rec l-0) l-1)) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec l-0) l-1)) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-1) (y zeta) (y (mul l (rec l-0) l-1)) (y l-0))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b l-0) (pv self l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-1) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-1)))
      (send
        (sig (body self (exp (gen) l-1) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((load priv-stor-0 (cat pt-1 (pv self l-1)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (recv
        (sig (body a (exp (gen) (mul l (rec l-0) l-1)) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-1)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-0))))
    ((recv (cat (exp (gen) (mul (rec l-0) l-1)) l))
      (send (cat (exp (gen) (mul (rec l-0) l-1)) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((load priv-stor-1 (cat pt-4 (pv b l-0)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2257)
  (parent 2155)
  (unrealized (12 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2258)
  (parent 2157)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((1 2) (11 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 2) (3 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2259)
  (parent 2158)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-4 (pv b l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2260)
  (parent 2167)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (added-strand ltx-disclose 3) l-1 (11 0)
    (ch-msg priv-stor-1 (cat pt-6 (pv self-0 l-1))))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1)))
  (label 2261)
  (parent 2168)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2262)
  (parent 2170)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn) (x rndx)
    (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 0))
    ((4 2) (3 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2263)
  (parent 2171)
  (seen 2169)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization weakened ((1 2) (0 3)))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2264)
  (parent 2172)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b) (x x) (upsilon upsilon) (na na)
        (nb nb) (priv-stor priv-stor) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-4 (4 1)) (pt-3 (3 1)) (l (2 1))
    (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2265)
  (parent 2173)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2266)
  (parent 2174)
  (seen 2172)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2267)
  (parent 2176)
  (seen 2061)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2268)
  (parent 2179)
  (seen 2067)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (11 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2269)
  (parent 2182)
  (seen 2073)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2270)
  (parent 2184)
  (seen 2076)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2271)
  (parent 2184)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2272)
  (parent 2185)
  (seen 2078)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2273)
  (parent 2185)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (11 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (11 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2274)
  (parent 2187)
  (seen 2083)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2275)
  (parent 2189)
  (seen 2086)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-3 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2276)
  (parent 2189)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 4 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2277)
  (parent 2190)
  (seen 2088)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2278)
  (parent 2190)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 1) (2 0)) ((4 2) (3 0)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2279)
  (parent 2191)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (5 0)) ((1 2) (0 1))
    ((2 1) (6 0)) ((3 1) (6 0)) ((4 1) (3 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0) (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2280)
  (parent 2192)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2281)
  (parent 2193)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((1 2) (2 0)) ((2 1) (4 0)) ((3 2) (2 0)) ((4 1) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l) l)) (send (cat (exp (gen) l) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2282)
  (parent 2194)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb)))
  (label 2283)
  (parent 2195)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (5 0)) ((3 2) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2284)
  (parent 2196)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-0)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 2) (6 0)) ((3 1) (5 0)) ((3 2) (6 0)) ((4 1) (6 0))
    ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (6 0))
  (gen-st (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self self-0))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0)))
      (send (cat na self self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self self-0
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-0 (cat pt-3 (pv self-0 lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2285)
  (parent 2198)
  (seen 2284)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2286)
  (parent 2200)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (2 0)) ((4 1) (0 0)) ((4 2) (2 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2287)
  (parent 2201)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2288)
  (parent 2202)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2289)
  (parent 2203)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (2 0))
    ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2290)
  (parent 2204)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (7 0))
    ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2291)
  (parent 2205)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2292)
  (parent 2206)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2293)
  (parent 2207)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 2294)
  (parent 2208)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2295)
  (parent 2209)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2296)
  (parent 2210)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (7 0)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2297)
  (parent 2211)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (5 0))
    ((3 2) (5 0)) ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2298)
  (parent 2213)
  (seen 2363)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2299)
  (parent 2214)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2300)
  (parent 2215)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2301)
  (parent 2216)
  (seen 2366)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (4 0))
    ((3 2) (7 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((2 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2302)
  (parent 2217)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2303)
  (parent 2218)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2304)
  (parent 2219)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2305)
  (parent 2220)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2306)
  (parent 2221)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2307)
  (parent 2223)
  (seen 2369)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0)))
  (label 2308)
  (parent 2224)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (8 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2309)
  (parent 2225)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2310)
  (parent 2226)
  (seen 2372)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 2) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2311)
  (parent 2227)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2312)
  (parent 2228)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (10 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2313)
  (parent 2229)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2314)
  (parent 2230)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (11 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2315)
  (parent 2231)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta lb) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization separated self-0)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2316)
  (parent 2232)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta lb) (a a) (b self-0) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor) (self self)
        (priv-stor-0 priv-stor-0) (ignore ignore))))
  (origs (pt-4 (4 1)) (lb (1 1)) (pt-1 (1 1)) (pt-3 (3 1)) (l (2 1))
    (pt (2 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na nb lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2317)
  (parent 2233)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2318)
  (parent 2234)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2319)
  (parent 2235)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2320)
  (parent 2236)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2321)
  (parent 2237)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0)) ((3 1) (5 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2322)
  (parent 2238)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (7 0))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((5 1) (3 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (6 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2323)
  (parent 2239)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l)))
  (label 2324)
  (parent 2240)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2325)
  (parent 2241)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2326)
  (parent 2242)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((6 1) (5 0))
    ((6 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2327)
  (parent 2243)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2328)
  (parent 2244)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 2) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2329)
  (parent 2245)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (0 3))
    ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2330)
  (parent 2246)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (6 0)) ((4 1) (8 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (0 0)) ((7 2) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (6 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2331)
  (parent 2247)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((5 1) (4 0))
    ((5 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2332)
  (parent 2248)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (11 0)) ((6 1) (5 0))
    ((6 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2333)
  (parent 2249)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (6 0))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2334)
  (parent 2250)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand resp 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 2)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 3) (10 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((5 1) (3 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (10 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (3 0)) ((4 1) (7 0)) ((9 1) (0 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (recv
        (sig
          (body self-0 (exp (gen) (mul l l-0 (rec lb)))
            (pubk "sig" self-0)) (privk "sig" self-0)))
      (recv (cat na self-0 b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2335)
  (parent 2251)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2336)
  (parent 2252)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b l-1) (pv self l))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2337)
  (parent 2252)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2338)
  (parent 2253)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2339)
  (parent 2253)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2340)
  (parent 2254)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (9 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2341)
  (parent 2255)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2342)
  (parent 2256)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (11 0)) ((5 3) (12 0))
    ((6 1) (5 0)) ((6 1) (10 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (9 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (11 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (10 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (12 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2343)
  (parent 2257)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2344)
  (parent 2258)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((1 2) (10 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2345)
  (parent 2259)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (0 3))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2346)
  (parent 2260)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb l-1) (y y) (alpha (mul l l-0 (rec l-1))) (zeta zeta))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (gen) (rec l-1)))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b l-1) (pv self l-0))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b l-1))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (gen) (rec l-1))) (send (cat (gen) (rec l-1))))
    ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-4 (pv b l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2347)
  (parent 2260)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (0 3))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (3 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 13 5 resp 4) (exp (gen) y-0) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2348)
  (parent 2261)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (x y rndx) (zeta expt)
    (l l-0 l-1 rndx) (w expt))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0)
    (upsilon (mul (rec x) y zeta)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb l-1) (y y) (alpha (mul l l-0 (rec l-1)))
    (zeta zeta))
  (deflistener (cat (exp (gen) (mul l-0 (rec l-1))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec l-1)) l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-1))
  (deflistener l-1)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l l-1))
  (deflistener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 l-1)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y zeta) (y (mul l l-0 (rec l-1))) (y l-1))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 l-1))
  (facts (neq (exp (gen) (mul (rec x) y zeta)) (gen))
    (neq (exp (gen) zeta) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
    (exp (gen) (mul (rec x) y zeta)) (0 3))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul (rec x) y zeta))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec l-1))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) zeta)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul y zeta)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec l-1))) l))
      (send (cat (exp (gen) (mul l-0 (rec l-1))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec l-1)) l-0))
      (send (cat (exp (gen) (rec l-1)) l-0)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec l-1)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (send
        (sig (body self-0 (exp (gen) l-1) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv l-1) (send l-1))
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-1)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send l-1))
    ((recv (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y zeta (rec w))) w))))
  (label 2349)
  (parent 2261)
  (unrealized (13 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2350)
  (parent 2262)
  (seen 2264)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (x rndx) (upsilon expt) (l l-0 rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon upsilon))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) upsilon) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) upsilon)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0))
              (exp (gen) (mul x upsilon)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2351)
  (parent 2265)
  (seen 2264)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (7 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (9 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2352)
  (parent 2271)
  (seen 2219)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (6 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2353)
  (parent 2273)
  (seen 2221)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (9 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 0))
    ((5 1) (12 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (9 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2354)
  (parent 2276)
  (seen 2229)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (8 0)) ((1 2) (3 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 3) (13 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (4 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (8 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((10 1) (4 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 4 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2355)
  (parent 2278)
  (seen 2231)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 2) (2 0)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))) ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2356)
  (parent 2279)
  (seen 2281)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) (mul l l)))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (4 0)) ((1 2) (0 1))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (precur (2 0) (5 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (gen) (mul l l))) (send (cat (gen) (mul l l))))
    ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2357)
  (parent 2280)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)))
  (uniq-orig na l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2358)
  (parent 2281)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l) (a self) (b b) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (self self) (priv-stor-0 priv-stor)
        (ignore ignore))))
  (origs (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1)) (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (3 0)) ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1))
    ((2 2) (3 0)) ((3 1) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (precur (3 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2359)
  (parent 2282)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 3))
    ((3 2) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2360)
  (parent 2283)
  (seen 2281)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 2) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2361)
  (parent 2284)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2362)
  (parent 2286)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l-0) (beta l) (a self-0) (b b) (x x) (upsilon y) (na na)
        (nb nb) (priv-stor priv-stor-0) (self self)
        (priv-stor-0 priv-stor) (ignore ignore))))
  (origs (l-0 (3 1)) (pt-3 (3 1)) (pt-1 (2 1)) (l (1 1)) (pt-0 (1 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (4 0))
    ((3 1) (0 0)) ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2363)
  (parent 2287)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2364)
  (parent 2288)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2365)
  (parent 2289)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2366)
  (parent 2290)
  (seen 2286)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (2 0)) ((4 1) (2 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2367)
  (parent 2291)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2368)
  (parent 2292)
  (realized)
  (shape)
  (maps
    ((0 1)
      ((la l) (beta l-0) (a a) (b b) (x x) (upsilon y) (na na) (nb nb)
        (priv-stor priv-stor) (self self) (priv-stor-0 priv-stor-0)
        (ignore ignore))))
  (origs (l-0 (1 1)) (pt-1 (1 1)) (pt-3 (3 1)) (l (2 1)) (pt (2 1))
    (na (0 2))))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2369)
  (parent 2293)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2370)
  (parent 2294)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 1) (8 0)) ((4 1) (8 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2371)
  (parent 2295)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2372)
  (parent 2296)
  (seen 2292)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2373)
  (parent 2297)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2374)
  (parent 2299)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (5 0)) ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (7 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2375)
  (parent 2300)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (6 0))
    ((3 2) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (3 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2376)
  (parent 2302)
  (seen 2427)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (6 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2377)
  (parent 2303)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2378)
  (parent 2304)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2379)
  (parent 2305)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2380)
  (parent 2306)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0)))
  (label 2381)
  (parent 2308)
  (seen 2429)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (7 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (5 0)) ((4 2) (8 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (7 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2382)
  (parent 2309)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv b lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2383)
  (parent 2311)
  (seen 2431)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (8 0)) ((4 2) (0 3)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2384)
  (parent 2312)
  (seen 2434)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (9 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2385)
  (parent 2313)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (3 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2386)
  (parent 2314)
  (seen 2436)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (3 0)) ((3 2) (10 0)) ((4 1) (10 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (3 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2387)
  (parent 2315)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (lb l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self) (priv-stor priv-stor)
    (la l) (x x) (beta lb) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na lb l)
  (uniq-gen x)
  (absent (x lb) (x l))
  (gen-st (pv a l) (pv self lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))) (send (cat na a self (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self
            (hash (exp (gen) (mul lb l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self lb)))
      (send
        (sig (body self (exp (gen) lb) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self lb)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send lb))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2388)
  (parent 2317)
  (seen 2264)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2389)
  (parent 2318)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2390)
  (parent 2319)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2391)
  (parent 2320)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2392)
  (parent 2321)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0)) ((3 1) (4 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 1) (0 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2393)
  (parent 2322)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (6 0))
    ((3 1) (5 0)) ((4 1) (9 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2394)
  (parent 2323)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 2395)
  (parent 2324)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2396)
  (parent 2325)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 2) (0 1))
    ((3 2) (9 0)) ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2397)
  (parent 2326)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((5 1) (4 0))
    ((5 1) (7 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2398)
  (parent 2327)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2399)
  (parent 2328)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (8 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2400)
  (parent 2329)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (0 3))
    ((5 1) (0 0)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2401)
  (parent 2330)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (7 0)) ((5 2) (4 0)) ((6 1) (0 0)) ((6 2) (4 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2402)
  (parent 2331)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 1) (7 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2403)
  (parent 2332)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (10 0)) ((5 1) (4 0))
    ((5 1) (8 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2404)
  (parent 2333)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0)) ((4 1) (0 3))
    ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2405)
  (parent 2334)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (6 0)) ((4 1) (9 0)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (6 0)) ((8 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2406)
  (parent 2335)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2407)
  (parent 2336)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (7 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 1) (7 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2408)
  (parent 2337)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (4 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2409)
  (parent 2338)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (6 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l)) (send (cat (gen) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2410)
  (parent 2339)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2411)
  (parent 2340)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2412)
  (parent 2341)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (9 0)) ((4 1) (8 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2413)
  (parent 2342)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (10 0)) ((4 3) (11 0)) ((5 1) (4 0))
    ((5 1) (9 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (8 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (10 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (9 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2414)
  (parent 2343)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2415)
  (parent 2344)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((1 2) (9 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2416)
  (parent 2345)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta x))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (0 3)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2417)
  (parent 2346)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (9 0)) ((5 3) (13 0))
    ((6 1) (5 0)) ((6 1) (12 0)) ((7 1) (5 1)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 1) (10 0)) ((12 2) (11 0))
    ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (10 0) (13 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((6 1) (5 0))
    ((6 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2418)
  (parent 2347)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta x))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 3) (0 3)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y x))
  (precur (8 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq (exp (gen) x) (gen))
    (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((9 1) (4 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a self-0 (exp (gen) x)))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2419)
  (parent 2348)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 2)) ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 1) (0 1)) ((4 2) (3 0)) ((4 2) (8 0)) ((5 3) (13 0))
    ((6 1) (5 1)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 1) (5 0)) ((10 1) (12 0)) ((10 2) (9 0)) ((11 1) (9 0))
    ((12 2) (11 0)) ((13 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (3 0) (9 0) (13 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((10 1) (5 0))
    ((10 1) (12 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation nonce-test (displaced 14 5 resp 4) (exp (gen) y-0) (13 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2420)
  (parent 2349)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (deflistener l)
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 2) (2 0)) ((4 1) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self)))) ((recv l) (send l))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2421)
  (parent 2357)
  (seen 2359)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore mesg) (na nb data) (b self name) (pt pt-0 pt-1 pval)
    (priv-stor locn) (l x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (0 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)))
  (uniq-orig na nb l)
  (uniq-gen x)
  (absent (x l))
  (gen-st (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (2 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l)))
  (label 2422)
  (parent 2359)
  (seen 1547)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l lb x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (0 0)) ((1 1) (3 0)) ((1 2) (0 1))
    ((2 1) (4 0)) ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb l lb)
  (uniq-gen x)
  (absent (x l))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self b))
  (leads-to ((1 1) (0 0)) ((1 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt-0 (pv self l)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self b
            (hash (exp (gen) (mul l l)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-1 (pv b lb))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2423)
  (parent 2361)
  (seen 2359)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (0 3)) ((3 1) (0 0))
    ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2424)
  (parent 2363)
  (seen 1622)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2425)
  (parent 2364)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2426)
  (parent 2365)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((1 2) (0 1)) ((2 2) (5 0))
    ((3 1) (5 0)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (2 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2427)
  (parent 2367)
  (seen 2363)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l)))
  (label 2428)
  (parent 2369)
  (seen 1625)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2429)
  (parent 2370)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (7 0)) ((4 2) (3 0)) ((5 1) (3 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2430)
  (parent 2371)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((1 2) (4 0)) ((2 1) (0 0))
    ((2 1) (3 0)) ((3 2) (5 0)) ((4 1) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (5 0))
  (gen-st (pv a l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2431)
  (parent 2373)
  (seen 2369)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 2432)
  (parent 2374)
  (seen 2473)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 1) (4 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (6 0)) ((3 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-5 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2433)
  (parent 2375)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (7 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2434)
  (parent 2377)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (8 0)) ((4 1) (9 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2435)
  (parent 2378)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (0 3)) ((5 1) (7 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2436)
  (parent 2379)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (9 0)) ((4 2) (3 0)) ((5 1) (3 0)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2437)
  (parent 2380)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (6 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (7 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (6 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2438)
  (parent 2382)
  (seen 2476)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 1) (8 0)) ((4 2) (9 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-5 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2439)
  (parent 2385)
  (seen 2479)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (3 0)) ((3 2) (9 0)) ((4 2) (9 0)) ((5 1) (9 0))
    ((6 1) (8 0)) ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0))
    ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (9 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (3 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2440)
  (parent 2387)
  (seen 2481)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0)) ((3 1) (0 3))
    ((4 1) (0 0)) ((4 2) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2441)
  (parent 2389)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (6 0)) ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2442)
  (parent 2390)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2443)
  (parent 2391)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (6 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2444)
  (parent 2392)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0)) ((3 1) (0 3))
    ((4 1) (0 3)) ((5 1) (4 0)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2445)
  (parent 2393)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (5 0))
    ((3 1) (4 0)) ((4 2) (8 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 1) (0 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2446)
  (parent 2394)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (4 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2447)
  (parent 2395)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2448)
  (parent 2396)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((3 2) (8 0)) ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2449)
  (parent 2397)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l l-0)) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (9 0)) ((4 1) (6 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l l-0)) lb))
      (send (cat (exp (gen) (mul l l-0)) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2450)
  (parent 2398)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2451)
  (parent 2399)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (7 0)) ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2452)
  (parent 2400)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 0))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2453)
  (parent 2401)
  (seen 2482)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0))
    ((4 2) (6 0)) ((5 1) (0 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2454)
  (parent 2402)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((3 2) (0 1))
    ((4 1) (6 0)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2455)
  (parent 2403)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (7 0)) ((5 1) (10 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (9 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2456)
  (parent 2404)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0)) ((4 2) (0 3))
    ((5 1) (0 3)) ((6 1) (0 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2457)
  (parent 2405)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (8 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (0 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (5 0)) ((7 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2458)
  (parent 2406)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 1) (10 0)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2459)
  (parent 2407)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 1) (6 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2460)
  (parent 2408)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2461)
  (parent 2409)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 3) (12 0)) ((5 1) (4 1))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((9 1) (4 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2462)
  (parent 2410)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l)))
  (label 2463)
  (parent 2411)
  (seen 2488)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (7 0))
    ((6 2) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2464)
  (parent 2412)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (8 0)) ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2465)
  (parent 2413)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (9 0)) ((4 1) (8 0)) ((5 1) (10 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (9 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2466)
  (parent 2414)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2467)
  (parent 2415)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((1 2) (8 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2468)
  (parent 2416)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (10 0)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (8 0)) ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2469)
  (parent 2417)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor-1)
    (lb lb) (y y) (alpha (mul l l-0 (rec lb))) (zeta (mul x w)))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 3) (12 0)) ((5 1) (4 0))
    ((5 1) (11 0)) ((6 1) (4 1)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 1) (8 0)) ((10 1) (9 0)) ((11 2) (10 0)) ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (9 0) (12 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((5 1) (4 0))
    ((5 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a))) (recv (cat na a b (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2470)
  (parent 2418)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 1) (10 0)) ((8 2) (7 0)) ((9 1) (7 0))
    ((10 2) (9 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2471)
  (parent 2419)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand resp 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor-1) (lb lb) (y y) (alpha (mul l l-0 (rec lb)))
    (zeta (mul x w)))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 2)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 3) (12 0)) ((5 1) (4 1))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (4 0))
    ((9 1) (11 0)) ((9 2) (8 0)) ((10 1) (8 0)) ((11 2) (10 0))
    ((12 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x y)
  (absent (x l) (x l-0) (y (mul l l-0 (rec lb))) (y lb) (y (mul x w)))
  (precur (8 0) (12 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen))
    (neq (exp (gen) (mul x w)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((9 1) (4 0))
    ((9 1) (11 0)))
  (rule assume-init-0 assume-resp-0 trRl_ltx-disclose-at-0
    trRl_ltx-disclose-at-1 trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (recv
        (sig (body a (exp (gen) (mul l l-0 (rec lb))) (pubk "sig" a))
          (privk "sig" a)))
      (recv (cat na a self-0 (exp (gen) (mul x w))))
      (send
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2472)
  (parent 2420)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2473)
  (parent 2425)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2474)
  (parent 2426)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (0 3)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0)))
  (label 2475)
  (parent 2429)
  (seen 2512)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (5 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (3 0)) ((3 2) (6 0)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (5 0)) ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2476)
  (parent 2430)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 1) (6 0)) ((4 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2477)
  (parent 2433)
  (seen 2513)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2478)
  (parent 2434)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (7 0)) ((4 2) (8 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2479)
  (parent 2435)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3))
    ((4 1) (6 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2480)
  (parent 2436)
  (seen 2478)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (8 0)) ((4 1) (8 0)) ((5 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2481)
  (parent 2437)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2482)
  (parent 2441)
  (seen 1911)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (5 0)) ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2483)
  (parent 2442)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2484)
  (parent 2443)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (5 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2485)
  (parent 2444)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0)) ((3 1) (0 3))
    ((4 1) (3 0)) ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2486)
  (parent 2445)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (4 0))
    ((3 1) (7 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 1) (0 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2487)
  (parent 2446)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2488)
  (parent 2447)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (4 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2489)
  (parent 2448)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2490)
  (parent 2449)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 1) (5 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-7 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2491)
  (parent 2450)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2492)
  (parent 2451)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (6 0)) ((3 2) (0 1)) ((4 1) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2493)
  (parent 2452)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2494)
  (parent 2454)
  (seen 2518)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((3 2) (0 1))
    ((4 1) (5 0)) ((5 2) (0 3)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0)))
  (label 2495)
  (parent 2455)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 1) (9 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (8 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2496)
  (parent 2456)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 3))
    ((5 1) (0 0)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2497)
  (parent 2457)
  (seen 2521)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (4 0))
    ((4 2) (7 0)) ((5 1) (7 0)) ((6 1) (0 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (4 0)) ((6 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((load priv-stor-0 (cat pt-2 (pv b lb)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2498)
  (parent 2458)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((4 1) (9 0)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2499)
  (parent 2459)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (10 0)) ((5 1) (11 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (11 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2500)
  (parent 2460)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 1) (9 0))
    ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2501)
  (parent 2461)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (4 0)) ((4 1) (11 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (11 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2502)
  (parent 2462)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2503)
  (parent 2464)
  (seen 2524)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (7 0)) ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2504)
  (parent 2465)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (8 0)) ((4 1) (7 0)) ((5 2) (9 0))
    ((6 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (9 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) lb)) (send (cat (exp (gen) l-0) lb)))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2505)
  (parent 2466)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2506)
  (parent 2467)
  (seen 2527)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((1 2) (7 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (8 0))
    ((6 2) (8 0)) ((7 1) (8 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2507)
  (parent 2468)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (9 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2508)
  (parent 2469)
  (seen 2532)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (10 0)) ((5 1) (11 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (7 0)) ((9 1) (8 0))
    ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0) (11 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2509)
  (parent 2470)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2510)
  (parent 2471)
  (seen 2534)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) (mul l-0 (rec lb))) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (11 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (11 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (10 0)) ((8 2) (7 0))
    ((9 1) (7 0)) ((10 2) (9 0)) ((11 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (11 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((8 1) (10 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) (mul l-0 (rec lb))) l))
      (send (cat (exp (gen) (mul l-0 (rec lb))) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2511)
  (parent 2472)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2512)
  (parent 2473)
  (seen 1879)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2513)
  (parent 2474)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (4 0)) ((1 2) (0 1)) ((2 1) (0 0))
    ((2 1) (3 0)) ((3 2) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (4 0)) ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-3 "nil")) (send l))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2514)
  (parent 2476)
  (seen 2541)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 2) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb)))
  (label 2515)
  (parent 2478)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2516)
  (parent 2479)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (7 0)) ((4 1) (6 0)) ((4 2) (7 0)) ((5 1) (7 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-5 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2517)
  (parent 2481)
  (seen 2516)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2518)
  (parent 2483)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((3 2) (4 0)) ((4 1) (0 3)) ((5 1) (4 0)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2519)
  (parent 2484)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 1) (8 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2520)
  (parent 2485)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 3)) ((4 1) (0 0))
    ((4 2) (3 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) y) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2521)
  (parent 2486)
  (seen 2482)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((2 2) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 1) (0 0)) ((5 2) (4 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2522)
  (parent 2487)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization forgot nb)
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2523)
  (parent 2488)
  (seen 2316)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 2) (0 1)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2524)
  (parent 2489)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((6 1) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2525)
  (parent 2490)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (8 0)) ((5 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2526)
  (parent 2491)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (precedes ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one)))))
  (label 2527)
  (parent 2492)
  (seen 2488)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2528)
  (parent 2493)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2529)
  (parent 2495)
  (seen 2545)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0))
    ((3 2) (0 1)) ((4 1) (5 0)) ((5 2) (8 0)) ((6 1) (8 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (7 0)) ((4 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-7 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2530)
  (parent 2496)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (b self self-0 name) (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-1) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self b)
    (priv-stor priv-stor-0) (l lb))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 1) (0 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv b lb) (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((5 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-3 ignore-0))
      (stor priv-stor-0 (cat pt-2 (pv b lb))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-5 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2531)
  (parent 2498)
  (seen 2547)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (8 0)) ((5 2) (0 3)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2532)
  (parent 2499)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (9 0)) ((5 1) (10 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 1) (7 0)) ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)) ((4 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2533)
  (parent 2500)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)) ((5 1) (0 3)) ((6 1) (8 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 2) (7 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2534)
  (parent 2501)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (10 0)) ((5 2) (4 0)) ((6 1) (4 0))
    ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (10 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2535)
  (parent 2502)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((3 2) (6 0)) ((4 1) (0 3)) ((5 2) (0 3)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2536)
  (parent 2504)
  (seen 2549)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (7 0)) ((4 1) (6 0)) ((5 2) (8 0))
    ((6 2) (8 0)) ((7 1) (8 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (8 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2537)
  (parent 2505)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((1 2) (6 0)) ((2 1) (0 0))
    ((2 1) (5 0)) ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2538)
  (parent 2507)
  (seen 2551)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (9 0)) ((5 2) (10 0))
    ((6 1) (10 0)) ((7 1) (6 0)) ((8 1) (7 0)) ((9 2) (8 0))
    ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0) (10 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2539)
  (parent 2509)
  (seen 2554)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) (rec lb)) l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (10 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 2) (10 0)) ((5 1) (10 0))
    ((6 1) (5 0)) ((7 1) (9 0)) ((7 2) (6 0)) ((8 1) (6 0))
    ((9 2) (8 0)) ((10 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (10 0))
  (gen-st (pv a l) (pv self l-0) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((7 1) (9 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) (rec lb)) l-0))
      (send (cat (exp (gen) (rec lb)) l-0)))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2540)
  (parent 2511)
  (seen 2556)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0))
    ((3 2) (4 0)) ((4 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2541)
  (parent 2513)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (0 3))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0)))
  (label 2542)
  (parent 2515)
  (seen 2512)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)) ((3 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-3 (pv b lb)))
      (stor priv-stor-1 (cat pt-6 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2543)
  (parent 2516)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (precedes ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (0 0)) ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((3 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0)))))
  (label 2544)
  (parent 2518)
  (seen 2169)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (4 0)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2545)
  (parent 2519)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((3 2) (4 0)) ((4 1) (7 0)) ((5 1) (4 0))
    ((6 2) (5 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2546)
  (parent 2520)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (b self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a self-0) (b b)
    (priv-stor priv-stor-0) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 3 (ignore ignore-0) (self self-0)
    (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (2 0)) ((2 2) (0 1)) ((3 1) (5 0))
    ((4 1) (0 0)) ((4 2) (3 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (3 0) (5 0))
  (gen-st (pv self l) (pv self-0 l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq self-0 b))
  (leads-to ((1 1) (2 0)) ((4 1) (0 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (3 0))
  (traces
    ((load priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na self-0 b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb self-0 b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor (cat pt ignore))
      (stor priv-stor (cat pt-0 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-0 (pv self l)))
      (stor priv-stor (cat pt-1 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((load priv-stor-0 (cat pt-2 ignore-0))
      (stor priv-stor-0 (cat pt-3 (pv self-0 l-0)))
      (send
        (sig (body self-0 (exp (gen) l-0) (pubk "sig" self-0))
          (privk "sig" self-0))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2547)
  (parent 2522)
  (seen 2518)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (precedes ((1 1) (3 0)) ((1 2) (0 3)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l)))
  (label 2548)
  (parent 2524)
  (seen 2172)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((3 2) (5 0)) ((4 2) (0 3)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0))))
  (label 2549)
  (parent 2525)
  (seen 2560)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (7 0)) ((5 2) (4 0))
    ((6 1) (4 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2550)
  (parent 2526)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((1 2) (5 0)) ((2 1) (0 0))
    ((2 1) (4 0)) ((3 2) (0 1)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2551)
  (parent 2528)
  (seen 2524)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 1) (7 0)) ((6 2) (5 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2552)
  (parent 2530)
  (seen 2561)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3)) ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2553)
  (parent 2532)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (gen) (rec lb)))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (8 0)) ((5 2) (9 0)) ((6 1) (9 0))
    ((7 1) (6 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (9 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (gen) (rec lb))) (send (cat (gen) (rec lb))))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2554)
  (parent 2533)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)) ((5 1) (7 0)) ((5 2) (0 3)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) y) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2555)
  (parent 2534)
  (seen 2553)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (9 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (9 0)) ((5 1) (9 0)) ((6 1) (8 0))
    ((6 2) (5 0)) ((7 1) (5 0)) ((8 2) (7 0)) ((9 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (9 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((6 1) (8 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) lb) (mul (rec lb) (rec lb))))
      (send (cat (exp (gen) lb) (mul (rec lb) (rec lb)))))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2556)
  (parent 2535)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((3 2) (6 0)) ((4 1) (7 0)) ((5 2) (7 0))
    ((6 1) (7 0)) ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0) (7 0))
  (gen-st (pv a l) (pv b lb) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-6 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2557)
  (parent 2537)
  (seen 2562)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (3 0)) ((3 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (3 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-3 "nil")) (send l-0)))
  (label 2558)
  (parent 2541)
  (seen 2199)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 2) (0 1)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-1 (cat pt-4 ignore-1))
      (stor priv-stor-1 (cat pt-3 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-5 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2559)
  (parent 2543)
  (seen 2541)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2560)
  (parent 2545)
  (seen 2316)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (4 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (4 0) (6 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2561)
  (parent 2546)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l) (x x) (beta l-0) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l-0))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((3 2) (5 0)) ((4 2) (6 0)) ((5 1) (6 0))
    ((6 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0) (6 0))
  (gen-st (pv a l) (pv self l-0))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l)))
      (recv
        (sig (body b (exp (gen) l-0) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l-0)))
      (send
        (sig (body self (exp (gen) l-0) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l))))
    ((load priv-stor-0 (cat pt-1 (pv self l-0)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l-0))
    ((load priv-stor (cat pt (pv a l)))
      (stor priv-stor (cat pt-4 "nil")) (send l))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2562)
  (parent 2550)
  (seen 2566)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (6 0)) ((5 2) (0 3)) ((6 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb)))
  (label 2563)
  (parent 2553)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (7 0)) ((5 2) (8 0)) ((6 1) (8 0))
    ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2564)
  (parent 2554)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data)
    (a self self-0 name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b self-0)
    (priv-stor priv-stor) (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-gen 3 (ignore ignore-1) (self self-0)
    (priv-stor priv-stor-1) (l lb))
  (deflistener lb)
  (defstrand ltx-disclose 3 (self self-0) (priv-stor priv-stor-1)
    (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (8 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (8 0)) ((5 1) (7 0)) ((5 2) (8 0))
    ((6 1) (8 0)) ((7 2) (6 0)) ((8 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (8 0))
  (gen-st (pv a l-0) (pv self l) (pv self-0 lb))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a self-0))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((5 1) (7 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv
        (sig (body self-0 (exp (gen) l) (pubk "sig" self-0))
          (privk "sig" self-0))) (send (cat na a self-0 (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a self-0
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (send
        (sig (body self-0 (exp (gen) lb) (pubk "sig" self-0))
          (privk "sig" self-0)))) ((recv lb) (send lb))
    ((load priv-stor-1 (cat pt-6 (pv self-0 lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2565)
  (parent 2556)
  (seen 2564)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (5 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0))
    ((3 2) (0 1)) ((4 2) (5 0)) ((5 1) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (5 0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (4 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2566)
  (parent 2561)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon y))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((3 2) (0 1))
    ((4 1) (0 3)) ((5 2) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) y) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) y)
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0)))
  (label 2567)
  (parent 2563)
  (seen 2560)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pt-7 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (defstrand ltx-disclose 3 (self b) (priv-stor priv-stor-1) (l lb))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (7 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (7 0)) ((6 2) (7 0))
    ((7 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (7 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)) ((4 1) (6 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((load priv-stor-1 (cat pt-4 (pv b lb)))
      (stor priv-stor-1 (cat pt-7 "nil")) (send lb))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2568)
  (parent 2564)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pval) (priv-stor priv-stor-0 locn)
    (l l-0 x rndx) (w expt) (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (precedes ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)) ((3 2) (0 1))
    ((4 2) (0 3)))
  (uniq-orig na nb l l-0)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (gen-st (pv a l-0) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (4 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (5 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-4 "nil")) (send l-0)))
  (label 2569)
  (parent 2566)
  (seen 2264)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton dhcr-um
  (vars (ignore ignore-0 ignore-1 mesg) (na nb data) (a b self name)
    (pt pt-0 pt-1 pt-2 pt-3 pt-4 pt-5 pt-6 pval)
    (priv-stor priv-stor-0 priv-stor-1 locn) (l l-0 lb x rndx) (w expt)
    (y rndx))
  (defstrand init 4 (na na) (nb nb) (a a) (b b) (priv-stor priv-stor)
    (la l-0) (x x) (beta l) (upsilon (mul w y)))
  (defstrand ltx-gen 3 (ignore ignore) (self self)
    (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-0) (self a) (priv-stor priv-stor)
    (l l-0))
  (defstrand ltx-disclose 3 (self self) (priv-stor priv-stor-0) (l l))
  (defstrand ltx-gen 2 (ignore ignore-1) (self b)
    (priv-stor priv-stor-1) (l lb))
  (defstrand ltx-disclose 3 (self a) (priv-stor priv-stor) (l l-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (6 0)) ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0))
    ((3 2) (0 1)) ((4 1) (6 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (uniq-orig na nb l l-0 lb)
  (uniq-gen x)
  (absent (x l) (x l-0))
  (precur (6 0))
  (gen-st (pv a l-0) (pv b lb) (pv self l))
  (facts (neq (exp (gen) (mul w y)) (gen)) (neq a b))
  (leads-to ((1 1) (3 0)) ((2 1) (0 0)) ((2 1) (5 0)))
  (rule assume-init-0 trRl_ltx-disclose-at-0 trRl_ltx-disclose-at-1
    trRl_ltx-gen-at-0 trRl_ltx-gen-at-1)
  (operation generalization deleted (6 0))
  (traces
    ((load priv-stor (cat pt (pv a l-0)))
      (recv (sig (body b (exp (gen) l) (pubk "sig" b)) (privk "sig" b)))
      (send (cat na a b (exp (gen) x)))
      (recv
        (cat (exp (gen) (mul w y))
          (enc na nb a b
            (hash (exp (gen) (mul l l-0)) (exp (gen) (mul x w y)))))))
    ((load priv-stor-0 (cat pt-0 ignore))
      (stor priv-stor-0 (cat pt-1 (pv self l)))
      (send
        (sig (body self (exp (gen) l) (pubk "sig" self))
          (privk "sig" self))))
    ((load priv-stor (cat pt-2 ignore-0))
      (stor priv-stor (cat pt (pv a l-0))))
    ((load priv-stor-0 (cat pt-1 (pv self l)))
      (stor priv-stor-0 (cat pt-3 "nil")) (send l))
    ((load priv-stor-1 (cat pt-5 ignore-1))
      (stor priv-stor-1 (cat pt-4 (pv b lb))))
    ((load priv-stor (cat pt (pv a l-0)))
      (stor priv-stor (cat pt-6 "nil")) (send l-0))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 2570)
  (parent 2568)
  (seen 2566)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")
